@@ -60,7 +60,48 @@ inline int32 CountHash(IntRange &&range) {
 
 class ApiWrap : public MTP::Sender, private base::Subscriber {
 public:
-	ApiWrap(not_null<AuthSession*> session);
+	struct Privacy {
+		enum class Key {
+			PhoneNumber,
+			LastSeen,
+			Calls,
+			Invites,
+			CallsPeer2Peer,
+			Forwards,
+			ProfilePhoto,
+		};
+		enum class Option {
+			Everyone,
+			Contacts,
+			Nobody,
+		};
+		Option option = Option::Everyone;
+		std::vector<not_null<PeerData*>> always;
+		std::vector<not_null<PeerData*>> never;
+
+		static MTPInputPrivacyKey Input(Key key);
+		static std::optional<Key> KeyFromMTP(mtpTypeId type);
+	};
+
+	struct BlockedUsersSlice {
+		struct Item {
+			UserData *user = nullptr;
+			TimeId date = 0;
+
+			bool operator==(const Item &other) const;
+			bool operator!=(const Item &other) const;
+		};
+
+		QVector<Item> list;
+		int total = 0;
+
+		bool operator==(const BlockedUsersSlice &other) const;
+		bool operator!=(const BlockedUsersSlice &other) const;
+	};
+
+	explicit ApiWrap(not_null<AuthSession*> session);
+
+	AuthSession &session() const;
 
 	void applyUpdates(const MTPUpdates &updates, uint64 sentMessageRandomId = 0);
 	void applyNotifySettings(
@@ -109,6 +150,7 @@ public:
 	void requestFullPeer(not_null<PeerData*> peer);
 	void requestPeer(not_null<PeerData*> peer);
 	void requestPeers(const QList<PeerData*> &peers);
+	void requestPeerSettings(not_null<PeerData*> peer);
 	void requestLastParticipants(not_null<ChannelData*> channel);
 	void requestBots(not_null<ChannelData*> channel);
 	void requestAdmins(not_null<ChannelData*> channel);
@@ -207,15 +249,19 @@ public:
 	void leaveChannel(not_null<ChannelData*> channel);
 
 	void blockUser(not_null<UserData*> user);
-	void unblockUser(not_null<UserData*> user);
+	void unblockUser(not_null<UserData*> user, Fn<void()> onDone = nullptr);
 
 	void exportInviteLink(not_null<PeerData*> peer);
 	void requestNotifySettings(const MTPInputNotifyPeer &peer);
 	void updateNotifySettingsDelayed(not_null<const PeerData*> peer);
 	void saveDraftToCloudDelayed(not_null<History*> history);
 
-	void savePrivacy(const MTPInputPrivacyKey &key, QVector<MTPInputPrivacyRule> &&rules);
-	void handlePrivacyChange(mtpTypeId keyTypeId, const MTPVector<MTPPrivacyRule> &rules);
+	void savePrivacy(
+		const MTPInputPrivacyKey &key,
+		QVector<MTPInputPrivacyRule> &&rules);
+	void handlePrivacyChange(
+		Privacy::Key key,
+		const MTPVector<MTPPrivacyRule> &rules);
 	static int OnlineTillFromStatus(
 		const MTPUserStatus &status,
 		int currentOnlineTill);
@@ -285,7 +331,7 @@ public:
 		const MTPchannels_ChannelParticipants &result,
 		Fn<void(
 			int availableCount,
-			const QVector<MTPChannelParticipant> &list)> callbackList,
+			const QVector<MTPChannelParticipant> &list)> callbackList = nullptr,
 		Fn<void()> callbackNotModified = nullptr);
 	void addChatParticipants(
 		not_null<PeerData*> peer,
@@ -401,29 +447,12 @@ public:
 
 	void saveSelfBio(const QString &text, FnMut<void()> done);
 
-	struct Privacy {
-		enum class Key {
-			LastSeen,
-			Calls,
-			Invites,
-			CallsPeer2Peer,
-			Forwards,
-			ProfilePhoto,
-		};
-		enum class Option {
-			Everyone,
-			Contacts,
-			Nobody,
-		};
-		Option option = Option::Everyone;
-		std::vector<not_null<UserData*>> always;
-		std::vector<not_null<UserData*>> never;
-
-		static MTPInputPrivacyKey Input(Key key);
-	};
 	void reloadPrivacy(Privacy::Key key);
 	rpl::producer<Privacy> privacyValue(Privacy::Key key);
 
+	void reloadBlockedUsers();
+	rpl::producer<BlockedUsersSlice> blockedUsersSlice();
+
 	void reloadSelfDestruct();
 	rpl::producer<int> selfDestructValue() const;
 	void saveSelfDestruct(int days);
@@ -661,6 +690,7 @@ private:
 	using PeerRequests = QMap<PeerData*, mtpRequestId>;
 	PeerRequests _fullPeerRequests;
 	PeerRequests _peerRequests;
+	base::flat_set<not_null<PeerData*>> _requestedPeerSettings;
 
 	PeerRequests _participantsRequests;
 	PeerRequests _botsRequests;
@@ -835,6 +865,10 @@ private:
 	base::flat_map<Privacy::Key, Privacy> _privacyValues;
 	std::map<Privacy::Key, rpl::event_stream<Privacy>> _privacyChanges;
 
+	mtpRequestId _blockedUsersRequestId = 0;
+	std::optional<BlockedUsersSlice> _blockedUsersSlice;
+	rpl::event_stream<BlockedUsersSlice> _blockedUsersChanges;
+
 	mtpRequestId _selfDestructRequestId = 0;
 	std::optional<int> _selfDestructDays;
 	rpl::event_stream<int> _selfDestructChanges;
