@@ -10,6 +10,7 @@ https://github.com/telegramdesktop/tdesktop/blob/master/LEGAL
 #include "apiwrap.h"
 #include "core/application.h"
 #include "core/changelogs.h"
+#include "main/main_account.h"
 #include "storage/file_download.h"
 #include "storage/file_upload.h"
 #include "storage/localstorage.h"
@@ -94,6 +95,9 @@ QByteArray AuthSessionSettings::serialize() const {
 		stream << autoDownload;
 		stream << qint32(_variables.supportAllSearchResults.current() ? 1 : 0);
 		stream << qint32(_variables.archiveCollapsed.current() ? 1 : 0);
+		stream << qint32(_variables.notifyAboutPinned.current() ? 1 : 0);
+		stream << qint32(_variables.archiveInMainMenu.current() ? 1 : 0);
+		stream << qint32(_variables.skipArchiveInSearch.current() ? 1 : 0);
 	}
 	return result;
 }
@@ -131,6 +135,9 @@ void AuthSessionSettings::constructFromSerialized(const QByteArray &serialized)
 	QByteArray autoDownload;
 	qint32 supportAllSearchResults = _variables.supportAllSearchResults.current() ? 1 : 0;
 	qint32 archiveCollapsed = _variables.archiveCollapsed.current() ? 1 : 0;
+	qint32 notifyAboutPinned = _variables.notifyAboutPinned.current() ? 1 : 0;
+	qint32 archiveInMainMenu = _variables.archiveInMainMenu.current() ? 1 : 0;
+	qint32 skipArchiveInSearch = _variables.skipArchiveInSearch.current() ? 1 : 0;
 
 	stream >> selectorTab;
 	stream >> lastSeenWarningSeen;
@@ -213,6 +220,15 @@ void AuthSessionSettings::constructFromSerialized(const QByteArray &serialized)
 	if (!stream.atEnd()) {
 		stream >> archiveCollapsed;
 	}
+	if (!stream.atEnd()) {
+		stream >> notifyAboutPinned;
+	}
+	if (!stream.atEnd()) {
+		stream >> archiveInMainMenu;
+	}
+	if (!stream.atEnd()) {
+		stream >> skipArchiveInSearch;
+	}
 	if (stream.status() != QDataStream::Ok) {
 		LOG(("App Error: "
 			"Bad data for AuthSessionSettings::constructFromSerialized()"));
@@ -283,6 +299,9 @@ void AuthSessionSettings::constructFromSerialized(const QByteArray &serialized)
 	_variables.exeLaunchWarning = (exeLaunchWarning == 1);
 	_variables.supportAllSearchResults = (supportAllSearchResults == 1);
 	_variables.archiveCollapsed = (archiveCollapsed == 1);
+	_variables.notifyAboutPinned = (notifyAboutPinned == 1);
+	_variables.archiveInMainMenu = (archiveInMainMenu == 1);
+	_variables.skipArchiveInSearch = (skipArchiveInSearch == 1);
 }
 
 void AuthSessionSettings::setSupportChatsTimeSlice(int slice) {
@@ -389,25 +408,61 @@ rpl::producer<bool> AuthSessionSettings::archiveCollapsedChanges() const {
 	return _variables.archiveCollapsed.changes();
 }
 
-AuthSession &Auth() {
-	const auto result = Core::App().authSession();
-	Assert(result != nullptr);
-	return *result;
+void AuthSessionSettings::setArchiveInMainMenu(bool inMainMenu) {
+	_variables.archiveInMainMenu = inMainMenu;
 }
 
-AuthSession::AuthSession(const MTPUser &user)
-: _autoLockTimer([this] { checkAutoLock(); })
+bool AuthSessionSettings::archiveInMainMenu() const {
+	return _variables.archiveInMainMenu.current();
+}
+
+rpl::producer<bool> AuthSessionSettings::archiveInMainMenuChanges() const {
+	return _variables.archiveInMainMenu.changes();
+}
+
+void AuthSessionSettings::setNotifyAboutPinned(bool notify) {
+	_variables.notifyAboutPinned = notify;
+}
+
+bool AuthSessionSettings::notifyAboutPinned() const {
+	return _variables.notifyAboutPinned.current();
+}
+
+rpl::producer<bool> AuthSessionSettings::notifyAboutPinnedChanges() const {
+	return _variables.notifyAboutPinned.changes();
+}
+
+void AuthSessionSettings::setSkipArchiveInSearch(bool skip) {
+	_variables.skipArchiveInSearch = skip;
+}
+
+bool AuthSessionSettings::skipArchiveInSearch() const {
+	return _variables.skipArchiveInSearch.current();
+}
+
+rpl::producer<bool> AuthSessionSettings::skipArchiveInSearchChanges() const {
+	return _variables.skipArchiveInSearch.changes();
+}
+
+AuthSession &Auth() {
+	return Core::App().activeAccount().session();
+}
+
+AuthSession::AuthSession(
+	not_null<Main::Account*> account,
+	const MTPUser &user)
+: _account(account)
+, _autoLockTimer([=] { checkAutoLock(); })
 , _api(std::make_unique<ApiWrap>(this))
 , _calls(std::make_unique<Calls::Instance>())
-, _downloader(std::make_unique<Storage::Downloader>())
-, _uploader(std::make_unique<Storage::Uploader>())
+, _downloader(std::make_unique<Storage::Downloader>(_api.get()))
+, _uploader(std::make_unique<Storage::Uploader>(_api.get()))
 , _storage(std::make_unique<Storage::Facade>())
 , _notifications(std::make_unique<Window::Notifications::System>(this))
 , _data(std::make_unique<Data::Session>(this))
 , _user(_data->processUser(user))
 , _changelogs(Core::Changelogs::Create(this))
 , _supportHelper(Support::Helper::Create(this)) {
-
 	_saveDataTimer.setCallback([=] {
 		Local::writeUserSettings();
 	});
@@ -447,8 +502,18 @@ AuthSession::AuthSession(const MTPUser &user)
 	Window::Theme::Background()->start();
 }
 
+AuthSession::~AuthSession() {
+	ClickHandler::clearActive();
+	ClickHandler::unpressed();
+}
+
+Main::Account &AuthSession::account() const {
+	return *_account;
+}
+
 bool AuthSession::Exists() {
-	return Core::IsAppLaunched() && (Core::App().authSession() != nullptr);
+	return Core::IsAppLaunched()
+		&& Core::App().activeAccount().sessionExists();
 }
 
 base::Observable<void> &AuthSession::downloaderTaskFinished() {
@@ -467,7 +532,7 @@ bool AuthSession::validateSelf(const MTPUser &user) {
 	if (user.type() != mtpc_user || !user.c_user().is_self()) {
 		LOG(("API Error: bad self user received."));
 		return false;
-	} else if (user.c_user().vid.v != userId()) {
+	} else if (user.c_user().vid().v != userId()) {
 		LOG(("Auth Error: wrong self user received."));
 		crl::on_main(this, [] { Core::App().logOut(); });
 		return false;
@@ -548,8 +613,3 @@ Support::Helper &AuthSession::supportHelper() const {
 Support::Templates& AuthSession::supportTemplates() const {
 	return supportHelper().templates();
 }
-
-AuthSession::~AuthSession() {
-	ClickHandler::clearActive();
-	ClickHandler::unpressed();
-}
