@@ -28,6 +28,7 @@ https://github.com/telegramdesktop/tdesktop/blob/master/LEGAL
 #include "ui/toast/toast.h"
 #include "ui/special_buttons.h"
 #include "ui/text_options.h"
+#include "ui/unread_badge.h"
 #include "data/data_channel.h"
 #include "data/data_chat.h"
 #include "data/data_user.h"
@@ -51,6 +52,7 @@ bool IsValidPhone(QString phone) {
 		|| (phone.startsWith(qsl("42"))
 			&& (phone.length() == 2
 				|| phone.length() == 5
+				|| phone.length() == 6
 				|| phone == qsl("4242")));
 }
 
@@ -67,11 +69,11 @@ style::InputField CreateBioFieldStyle() {
 QString PeerFloodErrorText(PeerFloodType type) {
 	auto link = textcmdLink(
 		Core::App().createInternalLinkFull(qsl("spambot")),
-		lang(lng_cant_more_info));
+		tr::lng_cant_more_info(tr::now));
 	if (type == PeerFloodType::InviteGroup) {
-		return lng_cant_invite_not_contact(lt_more_info, link);
+		return tr::lng_cant_invite_not_contact(tr::now, lt_more_info, link);
 	}
-	return lng_cant_send_to_not_contact(lt_more_info, link);
+	return tr::lng_cant_send_to_not_contact(tr::now, lt_more_info, link);
 }
 
 void ShowAddParticipantsError(
@@ -107,9 +109,9 @@ void ShowAddParticipantsError(
 			};
 			Ui::show(
 				Box<ConfirmBox>(
-					lang(lng_cant_invite_offer_admin),
-					lang(lng_cant_invite_make_admin),
-					lang(lng_cancel),
+					tr::lng_cant_invite_offer_admin(tr::now),
+					tr::lng_cant_invite_make_admin(tr::now),
+					tr::lng_cancel(tr::now),
 					makeAdmin),
 				LayerOption::KeepOther);
 			return;
@@ -121,26 +123,32 @@ void ShowAddParticipantsError(
 	const auto hasBot = (bot != end(users));
 	const auto text = [&] {
 		if (error == qstr("USER_BOT")) {
-			return lang(lng_cant_invite_bot_to_channel);
+			return tr::lng_cant_invite_bot_to_channel(tr::now);
 		} else if (error == qstr("USER_LEFT_CHAT")) {
 			// Trying to return a user who has left.
 		} else if (error == qstr("USER_KICKED")) {
 			// Trying to return a user who was kicked by admin.
-			return lang(lng_cant_invite_banned);
+			return tr::lng_cant_invite_banned(tr::now);
 		} else if (error == qstr("USER_PRIVACY_RESTRICTED")) {
-			return lang(lng_cant_invite_privacy);
+			return tr::lng_cant_invite_privacy(tr::now);
 		} else if (error == qstr("USER_NOT_MUTUAL_CONTACT")) {
 			// Trying to return user who does not have me in contacts.
-			return lang(lng_failed_add_not_mutual);
+			return tr::lng_failed_add_not_mutual(tr::now);
 		} else if (error == qstr("USER_ALREADY_PARTICIPANT") && hasBot) {
-			return lang(lng_bot_already_in_group);
+			return tr::lng_bot_already_in_group(tr::now);
+		} else if (error == qstr("BOT_GROUPS_BLOCKED")) {
+			return tr::lng_error_cant_add_bot(tr::now);
 		} else if (error == qstr("PEER_FLOOD")) {
 			const auto isGroup = (chat->isChat() || chat->isMegagroup());
 			return PeerFloodErrorText(isGroup
 				? PeerFloodType::InviteGroup
 				: PeerFloodType::InviteChannel);
+		} else if (error == qstr("ADMINS_TOO_MUCH")) {
+			return ((chat->isChat() || chat->isMegagroup())
+				? tr::lng_error_admin_limit
+				: tr::lng_error_admin_limit_channel)(tr::now);
 		}
-		return lang(lng_failed_add_participant);
+		return tr::lng_failed_add_participant(tr::now);
 	}();
 	Ui::show(Box<InformBox>(text), LayerOption::KeepOther);
 }
@@ -161,7 +169,7 @@ private:
 		}
 
 		not_null<PeerData*> peer;
-		Text name, status;
+		Ui::Text::String name, status;
 	};
 	void paintChat(Painter &p, const ChatRow &row, bool selected) const;
 	void updateSelected();
@@ -182,9 +190,9 @@ private:
 };
 
 AddContactBox::AddContactBox(QWidget*, QString fname, QString lname, QString phone)
-: _first(this, st::defaultInputField, langFactory(lng_signup_firstname), fname)
-, _last(this, st::defaultInputField, langFactory(lng_signup_lastname), lname)
-, _phone(this, st::defaultInputField, langFactory(lng_contact_phone), phone)
+: _first(this, st::defaultInputField, tr::lng_signup_firstname(), fname)
+, _last(this, st::defaultInputField, tr::lng_signup_lastname(), lname)
+, _phone(this, st::defaultInputField, tr::lng_contact_phone(), phone)
 , _invertOrder(langFirstNameGoesSecond()) {
 	if (!phone.isEmpty()) {
 		_phone->setDisabled(true);
@@ -193,9 +201,9 @@ AddContactBox::AddContactBox(QWidget*, QString fname, QString lname, QString pho
 
 AddContactBox::AddContactBox(QWidget*, UserData *user)
 : _user(user)
-, _first(this, st::defaultInputField, langFactory(lng_signup_firstname), user->firstName)
-, _last(this, st::defaultInputField, langFactory(lng_signup_lastname), user->lastName)
-, _phone(this, st::defaultInputField, langFactory(lng_contact_phone), user->phone())
+, _first(this, st::defaultInputField, tr::lng_signup_firstname(), user->firstName)
+, _last(this, st::defaultInputField, tr::lng_signup_lastname(), user->lastName)
+, _phone(this, st::defaultInputField, tr::lng_contact_phone(), user->phone())
 , _invertOrder(langFirstNameGoesSecond()) {
 	_phone->setDisabled(true);
 }
@@ -205,10 +213,13 @@ void AddContactBox::prepare() {
 		setTabOrder(_last, _first);
 	}
 	if (_user) {
-		setTitle(langFactory(lng_edit_contact_title));
+		setTitle(tr::lng_edit_contact_title());
 	} else {
-		auto readyToAdd = !_phone->getLastText().isEmpty() && (!_first->getLastText().isEmpty() || !_last->getLastText().isEmpty());
-		setTitle(langFactory(readyToAdd ? lng_confirm_contact_data : lng_enter_contact_data));
+		const auto readyToAdd = !_phone->getLastText().isEmpty()
+			&& (!_first->getLastText().isEmpty() || !_last->getLastText().isEmpty());
+		setTitle(readyToAdd
+			? tr::lng_confirm_contact_data()
+			: tr::lng_enter_contact_data());
 	}
 	updateButtons();
 
@@ -236,7 +247,7 @@ void AddContactBox::paintEvent(QPaintEvent *e) {
 		p.setPen(st::boxTextFg);
 		p.setFont(st::boxTextFont);
 		auto textHeight = height() - st::contactPadding.top() - st::contactPadding.bottom() - st::boxPadding.bottom();
-		p.drawText(QRect(st::boxPadding.left(), st::contactPadding.top(), width() - st::boxPadding.left() - st::boxPadding.right(), textHeight), lng_contact_not_joined(lt_name, _sentName), style::al_topleft);
+		p.drawText(QRect(st::boxPadding.left(), st::contactPadding.top(), width() - st::boxPadding.left() - st::boxPadding.right(), textHeight), tr::lng_contact_not_joined(tr::now, lt_name, _sentName), style::al_topleft);
 	} else {
 		st::contactUserIcon.paint(
 			p,
@@ -342,21 +353,20 @@ void AddContactBox::onImportDone(const MTPcontacts_ImportedContacts &res) {
 	if (!isBoxShown() || !App::main()) return;
 
 	const auto &d = res.c_contacts_importedContacts();
-	Auth().data().processUsers(d.vusers);
+	Auth().data().processUsers(d.vusers());
 
-	const auto &v = d.vimported.v;
+	const auto &v = d.vimported().v;
 	const auto user = [&]() -> UserData* {
 		if (!v.isEmpty()) {
 			auto &c = v.front().c_importedContact();
-			if (c.vclient_id.v == _contactId) {
-				return Auth().data().userLoaded(c.vuser_id.v);
+			if (c.vclient_id().v == _contactId) {
+				return Auth().data().userLoaded(c.vuser_id().v);
 			}
 		}
 		return nullptr;
 	}();
 	if (user) {
-		if (user->contactStatus() == UserData::ContactStatus::Contact
-			|| user->session().supportMode()) {
+		if (user->isContact() || user->session().supportMode()) {
 			Ui::showPeerHistory(user, ShowAtTheEndMsgId);
 		}
 		Ui::hideLayer();
@@ -370,7 +380,7 @@ void AddContactBox::onImportDone(const MTPcontacts_ImportedContacts &res) {
 
 void AddContactBox::onSaveUserDone(const MTPcontacts_ImportedContacts &res) {
 	auto &d = res.c_contacts_importedContacts();
-	Auth().data().processUsers(d.vusers);
+	Auth().data().processUsers(d.vusers());
 	closeBox();
 }
 
@@ -391,16 +401,23 @@ void AddContactBox::retry() {
 void AddContactBox::updateButtons() {
 	clearButtons();
 	if (_retrying) {
-		addButton(langFactory(lng_try_other_contact), [this] { retry(); });
+		addButton(tr::lng_try_other_contact(), [=] { retry(); });
 	} else {
-		addButton(langFactory(_user ? lng_settings_save : lng_add_contact), [this] { save(); });
-		addButton(langFactory(lng_cancel), [this] { closeBox(); });
+		addButton(
+			_user ? tr::lng_settings_save() : tr::lng_add_contact(),
+			[=] { save(); });
+		addButton(tr::lng_cancel(), [=] { closeBox(); });
 	}
 }
 
-GroupInfoBox::GroupInfoBox(QWidget*, CreatingGroupType creating, bool fromTypeChoose)
-: _creating(creating)
-, _fromTypeChoose(fromTypeChoose) {
+GroupInfoBox::GroupInfoBox(
+	QWidget*,
+	Type type,
+	const QString &title,
+	Fn<void(not_null<ChannelData*>)> channelDone)
+: _type(type)
+, _initialTitle(title)
+, _channelDone(std::move(channelDone)) {
 }
 
 void GroupInfoBox::prepare() {
@@ -408,17 +425,18 @@ void GroupInfoBox::prepare() {
 
 	_photo.create(
 		this,
-		lang((_creating == CreatingGroupChannel)
-			? lng_create_channel_crop
-			: lng_create_group_crop),
+		((_type == Type::Channel)
+			? tr::lng_create_channel_crop
+			: tr::lng_create_group_crop)(tr::now),
 		Ui::UserpicButton::Role::ChangePhoto,
 		st::defaultUserpicButton);
 	_title.create(
 		this,
 		st::defaultInputField,
-		langFactory(_creating == CreatingGroupChannel
-			? lng_dlg_new_channel_name
-			: lng_dlg_new_group_name));
+		(_type == Type::Channel
+			? tr::lng_dlg_new_channel_name
+			: tr::lng_dlg_new_group_name)(),
+		_initialTitle);
 	_title->setMaxLength(kMaxGroupChannelTitle);
 	_title->setInstantReplaces(Ui::InstantReplaces::Default());
 	_title ->setInstantReplacesEnabled(Global::ReplaceEmojiValue());
@@ -426,12 +444,12 @@ void GroupInfoBox::prepare() {
 		getDelegate()->outerContainer(),
 		_title);
 
-	if (_creating == CreatingGroupChannel) {
+	if (_type != Type::Group) {
 		_description.create(
 			this,
 			st::newGroupDescription,
 			Ui::InputField::Mode::MultiLine,
-			langFactory(lng_create_group_description));
+			tr::lng_create_group_description());
 		_description->show();
 		_description->setMaxLength(kMaxChannelDescription);
 		_description->setInstantReplaces(Ui::InstantReplaces::Default());
@@ -449,8 +467,12 @@ void GroupInfoBox::prepare() {
 
 	connect(_title, &Ui::InputField::submitted, [=] { submitName(); });
 
-	addButton(langFactory(_creating == CreatingGroupChannel ? lng_create_group_create : lng_create_group_next), [this] { submit(); });
-	addButton(langFactory(_fromTypeChoose ? lng_create_group_back : lng_cancel), [this] { closeBox(); });
+	addButton(
+		(_type != Type::Group
+			? tr::lng_create_group_create()
+			: tr::lng_create_group_next()),
+		[=] { submit(); });
+	addButton(tr::lng_cancel(), [this] { closeBox(); });
 
 	updateMaxHeight();
 }
@@ -522,9 +544,9 @@ void GroupInfoBox::createGroup(
 			| [](auto updates) -> std::optional<const QVector<MTPChat>*> {
 				switch (updates->type()) {
 				case mtpc_updates:
-					return &updates->c_updates().vchats.v;
+					return &updates->c_updates().vchats().v;
 				case mtpc_updatesCombined:
-					return &updates->c_updatesCombined().vchats.v;
+					return &updates->c_updatesCombined().vchats().v;
 				}
 				LOG(("API Error: unexpected update cons %1 "
 					"(GroupInfoBox::creationDone)").arg(updates->type()));
@@ -537,7 +559,7 @@ void GroupInfoBox::createGroup(
 					: std::nullopt;
 			}
 			| [](auto chats) {
-				return Auth().data().chat(chats->front().c_chat().vid.v);
+				return Auth().data().chat(chats->front().c_chat().vid().v);
 			}
 			| [&](not_null<ChatData*> chat) {
 				if (!image.isNull()) {
@@ -561,7 +583,7 @@ void GroupInfoBox::createGroup(
 			}
 		} else if (error.type() == qstr("USERS_TOO_FEW")) {
 			Ui::show(
-				Box<InformBox>(lang(lng_cant_invite_privacy)),
+				Box<InformBox>(tr::lng_cant_invite_privacy(tr::now)),
 				LayerOption::KeepOther);
 		} else if (error.type() == qstr("PEER_FLOOD")) {
 			Ui::show(
@@ -570,7 +592,7 @@ void GroupInfoBox::createGroup(
 				LayerOption::KeepOther);
 		} else if (error.type() == qstr("USER_RESTRICTED")) {
 			Ui::show(
-				Box<InformBox>(lang(lng_cant_do_this)),
+				Box<InformBox>(tr::lng_cant_do_this(tr::now)),
 				LayerOption::KeepOther);
 		}
 	}).send();
@@ -590,7 +612,7 @@ void GroupInfoBox::submit() {
 		_title->showError();
 		return;
 	}
-	if (_creating != CreatingGroupGroup) {
+	if (_type != Type::Group) {
 		createChannel(title, description);
 	} else {
 		auto initBox = [title, weak = make_weak(this)](
@@ -603,12 +625,8 @@ void GroupInfoBox::submit() {
 					}
 				}
 			};
-			box->addButton(
-				langFactory(lng_create_group_create),
-				std::move(create));
-			box->addButton(
-				langFactory(lng_cancel),
-				[box] { box->closeBox(); });
+			box->addButton(tr::lng_create_group_create(), std::move(create));
+			box->addButton(tr::lng_cancel(), [=] { box->closeBox(); });
 		};
 		Ui::show(
 			Box<PeerListBox>(
@@ -619,22 +637,25 @@ void GroupInfoBox::submit() {
 }
 
 void GroupInfoBox::createChannel(const QString &title, const QString &description) {
-	bool mega = false;
-	auto flags = mega ? MTPchannels_CreateChannel::Flag::f_megagroup : MTPchannels_CreateChannel::Flag::f_broadcast;
+	const auto flags = (_type == Type::Megagroup)
+		? MTPchannels_CreateChannel::Flag::f_megagroup
+		: MTPchannels_CreateChannel::Flag::f_broadcast;
 	_creationRequestId = request(MTPchannels_CreateChannel(
 		MTP_flags(flags),
 		MTP_string(title),
-		MTP_string(description)
+		MTP_string(description),
+		MTPInputGeoPoint(), // geo_point
+		MTPstring() // address
 	)).done([=](const MTPUpdates &result) {
 		Auth().api().applyUpdates(result);
 
-		auto success = base::make_optional(&result)
+		const auto success = base::make_optional(&result)
 			| [](auto updates) -> std::optional<const QVector<MTPChat>*> {
 				switch (updates->type()) {
 				case mtpc_updates:
-					return &updates->c_updates().vchats.v;
+					return &updates->c_updates().vchats().v;
 				case mtpc_updatesCombined:
-					return &updates->c_updatesCombined().vchats.v;
+					return &updates->c_updatesCombined().vchats().v;
 				}
 				LOG(("API Error: unexpected update cons %1 (GroupInfoBox::createChannel)").arg(updates->type()));
 				return std::nullopt;
@@ -645,7 +666,7 @@ void GroupInfoBox::createChannel(const QString &title, const QString &descriptio
 					: std::nullopt;
 			}
 			| [](auto chats) {
-				return Auth().data().channel(chats->front().c_channel().vid.v);
+				return Auth().data().channel(chats->front().c_channel().vid().v);
 			}
 			| [&](not_null<ChannelData*> channel) {
 				auto image = _photo->takeResultImage();
@@ -660,10 +681,17 @@ void GroupInfoBox::createChannel(const QString &title, const QString &descriptio
 				)).done([=](const MTPExportedChatInvite &result) {
 					_creationRequestId = 0;
 					if (result.type() == mtpc_chatInviteExported) {
-						auto link = qs(result.c_chatInviteExported().vlink);
+						auto link = qs(result.c_chatInviteExported().vlink());
 						_createdChannel->setInviteLink(link);
 					}
-					Ui::show(Box<SetupChannelBox>(_createdChannel));
+					if (_channelDone) {
+						const auto callback = _channelDone;
+						const auto argument = _createdChannel;
+						closeBox();
+						callback(argument);
+					} else {
+						Ui::show(Box<SetupChannelBox>(_createdChannel));
+					}
 				}).send();
 			};
 		if (!success) {
@@ -676,9 +704,9 @@ void GroupInfoBox::createChannel(const QString &title, const QString &descriptio
 			_title->setFocus();
 			_title->showError();
 		} else if (error.type() == qstr("USER_RESTRICTED")) {
-			Ui::show(Box<InformBox>(lang(lng_cant_do_this)));
+			Ui::show(Box<InformBox>(tr::lng_cant_do_this(tr::now)));
 		} else if (error.type() == qstr("CHANNELS_TOO_MUCH")) {
-			Ui::show(Box<InformBox>(lang(lng_cant_do_this))); // TODO
+			Ui::show(Box<InformBox>(tr::lng_cant_do_this(tr::now))); // TODO
 		}
 	}).send();
 }
@@ -706,12 +734,38 @@ SetupChannelBox::SetupChannelBox(QWidget*, ChannelData *channel, bool existing)
 : _channel(channel)
 , _existing(existing)
 , _privacyGroup(std::make_shared<Ui::RadioenumGroup<Privacy>>(Privacy::Public))
-, _public(this, _privacyGroup, Privacy::Public, lang(channel->isMegagroup() ? lng_create_public_group_title : lng_create_public_channel_title), st::defaultBoxCheckbox)
-, _private(this, _privacyGroup, Privacy::Private, lang(channel->isMegagroup() ? lng_create_private_group_title : lng_create_private_channel_title), st::defaultBoxCheckbox)
+, _public(
+	this,
+	_privacyGroup,
+	Privacy::Public,
+	(channel->isMegagroup()
+		? tr::lng_create_public_group_title
+		: tr::lng_create_public_channel_title)(tr::now),
+	st::defaultBoxCheckbox)
+, _private(
+	this,
+	_privacyGroup,
+	Privacy::Private,
+	(channel->isMegagroup()
+		? tr::lng_create_private_group_title
+		: tr::lng_create_private_channel_title)(tr::now),
+	st::defaultBoxCheckbox)
 , _aboutPublicWidth(st::boxWideWidth - st::boxPadding.left() - st::boxButtonPadding.right() - st::newGroupPadding.left() - st::defaultRadio.diameter - st::defaultBoxCheckbox.textPosition.x())
-, _aboutPublic(st::defaultTextStyle, lang(channel->isMegagroup() ? lng_create_public_group_about : lng_create_public_channel_about), _defaultOptions, _aboutPublicWidth)
-, _aboutPrivate(st::defaultTextStyle, lang(channel->isMegagroup() ? lng_create_private_group_about : lng_create_private_channel_about), _defaultOptions, _aboutPublicWidth)
-, _link(this, st::setupChannelLink, Fn<QString()>(), channel->username, true) {
+, _aboutPublic(
+	st::defaultTextStyle,
+	(channel->isMegagroup()
+		? tr::lng_create_public_group_about
+		: tr::lng_create_public_channel_about)(tr::now),
+	_defaultOptions,
+	_aboutPublicWidth)
+, _aboutPrivate(
+	st::defaultTextStyle,
+	(channel->isMegagroup()
+		? tr::lng_create_private_group_about
+		: tr::lng_create_private_channel_about)(tr::now),
+	_defaultOptions,
+	_aboutPublicWidth)
+, _link(this, st::setupChannelLink, nullptr, channel->username, true) {
 }
 
 void SetupChannelBox::prepare() {
@@ -719,10 +773,17 @@ void SetupChannelBox::prepare() {
 
 	setMouseTracking(true);
 
-	_checkRequestId = MTP::send(MTPchannels_CheckUsername(_channel->inputChannel, MTP_string("preston")), RPCDoneHandlerPtr(), rpcFail(&SetupChannelBox::onFirstCheckFail));
+	_checkRequestId = MTP::send(
+		MTPchannels_CheckUsername(
+			_channel->inputChannel,
+			MTP_string("preston")),
+		RPCDoneHandlerPtr(),
+		rpcFail(&SetupChannelBox::onFirstCheckFail));
 
-	addButton(langFactory(lng_settings_save), [=] { save(); });
-	addButton(langFactory(_existing ? lng_cancel : lng_create_group_skip), [=] { closeBox(); });
+	addButton(tr::lng_settings_save(), [=] { save(); });
+	addButton(
+		_existing ? tr::lng_cancel() : tr::lng_create_group_skip(),
+		[=] { closeBox(); });
 
 	connect(_link, &Ui::MaskedInputField::changed, [=] { handleChange(); });
 	_link->setVisible(_privacyGroup->value() == Privacy::Public);
@@ -792,7 +853,13 @@ void SetupChannelBox::paintEvent(QPaintEvent *e) {
 	if (!_channel->isMegagroup() || !_link->isHidden()) {
 		p.setPen(st::boxTextFg);
 		p.setFont(st::newGroupLinkFont);
-		p.drawTextLeft(st::boxPadding.left() + st::newGroupPadding.left() + st::defaultInputField.textMargins.left(), _link->y() - st::newGroupLinkPadding.top() + st::newGroupLinkTop, width(), lang(_link->isHidden() ? lng_create_group_invite_link : lng_create_group_link));
+		p.drawTextLeft(
+			st::boxPadding.left() + st::newGroupPadding.left() + st::defaultInputField.textMargins.left(),
+			_link->y() - st::newGroupLinkPadding.top() + st::newGroupLinkTop,
+			width(),
+			(_link->isHidden()
+				? tr::lng_create_group_invite_link
+				: tr::lng_create_group_link)(tr::now));
 	}
 
 	if (_link->isHidden()) {
@@ -801,7 +868,7 @@ void SetupChannelBox::paintEvent(QPaintEvent *e) {
 			option.setWrapMode(QTextOption::WrapAnywhere);
 			p.setFont(_linkOver ? st::boxTextFont->underline() : st::boxTextFont);
 			p.setPen(st::defaultLinkButton.color);
-			auto inviteLinkText = _channel->inviteLink().isEmpty() ? lang(lng_group_invite_create) : _channel->inviteLink();
+			auto inviteLinkText = _channel->inviteLink().isEmpty() ? tr::lng_group_invite_create(tr::now) : _channel->inviteLink();
 			p.drawText(_invitationLink, inviteLinkText, option);
 		}
 	} else {
@@ -838,7 +905,7 @@ void SetupChannelBox::mousePressEvent(QMouseEvent *e) {
 			_channel->session().api().exportInviteLink(_channel);
 		} else {
 			QGuiApplication::clipboard()->setText(_channel->inviteLink());
-			Ui::Toast::Show(lang(lng_create_channel_link_copied));
+			Ui::Toast::Show(tr::lng_create_channel_link_copied(tr::now));
 		}
 	}
 }
@@ -893,8 +960,8 @@ void SetupChannelBox::handleChange() {
 		for (int32 i = 0; i < len; ++i) {
 			QChar ch = name.at(i);
 			if ((ch < 'A' || ch > 'Z') && (ch < 'a' || ch > 'z') && (ch < '0' || ch > '9') && ch != '_') {
-				if (_errorText != lang(lng_create_channel_link_bad_symbols)) {
-					_errorText = lang(lng_create_channel_link_bad_symbols);
+				if (_errorText != tr::lng_create_channel_link_bad_symbols(tr::now)) {
+					_errorText = tr::lng_create_channel_link_bad_symbols(tr::now);
 					update();
 				}
 				_checkTimer.stop();
@@ -902,8 +969,8 @@ void SetupChannelBox::handleChange() {
 			}
 		}
 		if (name.size() < kMinUsernameLength) {
-			if (_errorText != lang(lng_create_channel_link_too_short)) {
-				_errorText = lang(lng_create_channel_link_too_short);
+			if (_errorText != tr::lng_create_channel_link_too_short(tr::now)) {
+				_errorText = tr::lng_create_channel_link_too_short(tr::now);
 				update();
 			}
 			_checkTimer.stop();
@@ -974,13 +1041,13 @@ bool SetupChannelBox::onUpdateFail(const RPCError &error) {
 	} else if (err == "USERNAME_INVALID") {
 		_link->setFocus();
 		_link->showError();
-		_errorText = lang(lng_create_channel_link_invalid);
+		_errorText = tr::lng_create_channel_link_invalid(tr::now);
 		update();
 		return true;
 	} else if (err == "USERNAME_OCCUPIED" || err == "USERNAMES_UNAVAILABLE") {
 		_link->setFocus();
 		_link->showError();
-		_errorText = lang(lng_create_channel_link_occupied);
+		_errorText = tr::lng_create_channel_link_occupied(tr::now);
 		update();
 		return true;
 	}
@@ -990,8 +1057,8 @@ bool SetupChannelBox::onUpdateFail(const RPCError &error) {
 
 void SetupChannelBox::onCheckDone(const MTPBool &result) {
 	_checkRequestId = 0;
-	QString newError = (mtpIsTrue(result) || _checkUsername == _channel->username) ? QString() : lang(lng_create_channel_link_occupied);
-	QString newGood = newError.isEmpty() ? lang(lng_create_channel_link_available) : QString();
+	QString newError = (mtpIsTrue(result) || _checkUsername == _channel->username) ? QString() : tr::lng_create_channel_link_occupied(tr::now);
+	QString newGood = newError.isEmpty() ? tr::lng_create_channel_link_available(tr::now) : QString();
 	if (_errorText != newError || _goodText != newGood) {
 		_errorText = newError;
 		_goodText = newGood;
@@ -1016,11 +1083,11 @@ bool SetupChannelBox::onCheckFail(const RPCError &error) {
 		}
 		return true;
 	} else if (err == qstr("USERNAME_INVALID")) {
-		_errorText = lang(lng_create_channel_link_invalid);
+		_errorText = tr::lng_create_channel_link_invalid(tr::now);
 		update();
 		return true;
 	} else if (err == qstr("USERNAME_OCCUPIED") && _checkUsername != _channel->username) {
-		_errorText = lang(lng_create_channel_link_occupied);
+		_errorText = tr::lng_create_channel_link_occupied(tr::now);
 		update();
 		return true;
 	}
@@ -1062,22 +1129,22 @@ bool SetupChannelBox::onFirstCheckFail(const RPCError &error) {
 
 EditNameBox::EditNameBox(QWidget*, not_null<UserData*> user)
 : _user(user)
-, _first(this, st::defaultInputField, langFactory(lng_signup_firstname), _user->firstName)
-, _last(this, st::defaultInputField, langFactory(lng_signup_lastname), _user->lastName)
+, _first(this, st::defaultInputField, tr::lng_signup_firstname(), _user->firstName)
+, _last(this, st::defaultInputField, tr::lng_signup_lastname(), _user->lastName)
 , _invertOrder(langFirstNameGoesSecond()) {
 }
 
 void EditNameBox::prepare() {
 	auto newHeight = st::contactPadding.top() + _first->height();
 
-	setTitle(langFactory(lng_edit_self_title));
+	setTitle(tr::lng_edit_self_title());
 	newHeight += st::contactSkip + _last->height();
 
 	newHeight += st::boxPadding.bottom() + st::contactPadding.bottom();
 	setDimensions(st::boxWideWidth, newHeight);
 
-	addButton(langFactory(lng_settings_save), [=] { save(); });
-	addButton(langFactory(lng_cancel), [=] { closeBox(); });
+	addButton(tr::lng_settings_save(), [=] { save(); });
+	addButton(tr::lng_cancel(), [=] { closeBox(); });
 	if (_invertOrder) {
 		setTabOrder(_last, _first);
 	}
@@ -1184,16 +1251,17 @@ bool EditNameBox::saveSelfFail(const RPCError &error) {
 
 RevokePublicLinkBox::Inner::Inner(QWidget *parent, Fn<void()> revokeCallback) : TWidget(parent)
 , _rowHeight(st::contactsPadding.top() + st::contactsPhotoSize + st::contactsPadding.bottom())
-, _revokeWidth(st::normalFont->width(lang(lng_channels_too_much_public_revoke)))
+, _revokeWidth(st::normalFont->width(tr::lng_channels_too_much_public_revoke(tr::now)))
 , _revokeCallback(std::move(revokeCallback)) {
 	setMouseTracking(true);
 
 	resize(width(), 5 * _rowHeight);
 
 	request(MTPchannels_GetAdminedPublicChannels(
+		MTP_flags(0)
 	)).done([=](const MTPmessages_Chats &result) {
 		const auto &chats = result.match([](const auto &data) {
-			return data.vchats.v;
+			return data.vchats().v;
 		});
 		for (const auto &chat : chats) {
 			if (const auto peer = Auth().data().processChat(chat)) {
@@ -1225,8 +1293,7 @@ RevokePublicLinkBox::RevokePublicLinkBox(
 	Fn<void()> revokeCallback)
 : _aboutRevoke(
 	this,
-	lang(lng_channels_too_much_public_about),
-	Ui::FlatLabel::InitType::Simple,
+	tr::lng_channels_too_much_public_about(tr::now),
 	st::aboutRevokePublicLabel)
 , _revokeCallback(std::move(revokeCallback)) {
 }
@@ -1241,7 +1308,7 @@ void RevokePublicLinkBox::prepare() {
 		}
 	}), st::boxLayerScroll, _innerTop);
 
-	addButton(langFactory(lng_cancel), [=] { closeBox(); });
+	addButton(tr::lng_cancel(), [=] { closeBox(); });
 
 	subscribe(Auth().downloaderTaskFinished(), [=] { update(); });
 
@@ -1283,14 +1350,21 @@ void RevokePublicLinkBox::Inner::mouseReleaseEvent(QMouseEvent *e) {
 	auto pressed = base::take(_pressed);
 	setCursor((_selected || _pressed) ? style::cur_pointer : style::cur_default);
 	if (pressed && pressed == _selected) {
-		auto text_method = pressed->isMegagroup() ? lng_channels_too_much_public_revoke_confirm_group : lng_channels_too_much_public_revoke_confirm_channel;
-		auto text = text_method(lt_link, Core::App().createInternalLink(pressed->userName()), lt_group, pressed->name);
-		auto confirmText = lang(lng_channels_too_much_public_revoke);
+		auto text_method = pressed->isMegagroup()
+			? tr::lng_channels_too_much_public_revoke_confirm_group
+			: tr::lng_channels_too_much_public_revoke_confirm_channel;
+		auto text = text_method(
+			tr::now,
+			lt_link,
+			Core::App().createInternalLink(pressed->userName()),
+			lt_group,
+			pressed->name);
+		auto confirmText = tr::lng_channels_too_much_public_revoke(tr::now);
 		_weakRevokeConfirmBox = Ui::show(Box<ConfirmBox>(text, confirmText, crl::guard(this, [this, pressed]() {
 			if (_revokeRequestId) return;
 			_revokeRequestId = request(MTPchannels_UpdateUsername(
 				pressed->asChannel()->inputChannel,
-				MTP_string("")
+				MTP_string()
 			)).done([=](const MTPBool &result) {
 				const auto callback = _revokeCallback;
 				if (_weakRevokeConfirmBox) {
@@ -1327,16 +1401,26 @@ void RevokePublicLinkBox::Inner::paintChat(Painter &p, const ChatRow &row, bool
 
 	int32 namex = st::contactsPadding.left() + st::contactsPhotoSize + st::contactsPadding.left();
 	int32 namew = width() - namex - st::contactsPadding.right() - (_revokeWidth + st::contactsCheckPosition.x() * 2);
-	if (peer->isVerified()) {
-		auto icon = &st::dialogsVerifiedIcon;
-		namew -= icon->width();
-		icon->paint(p, namex + qMin(row.name.maxWidth(), namew), st::contactsPadding.top() + st::contactsNameTop, width());
-	}
+
+	const auto badgeStyle = Ui::PeerBadgeStyle{
+		&st::dialogsVerifiedIcon,
+		&st::attentionButtonFg };
+	namew -= Ui::DrawPeerBadgeGetWidth(
+		peer,
+		p,
+		QRect(
+			namex,
+			st::contactsPadding.top() + st::contactsNameTop,
+			row.name.maxWidth(),
+			st::contactsNameStyle.font->height),
+		namew,
+		width(),
+		badgeStyle);
 	row.name.drawLeftElided(p, namex, st::contactsPadding.top() + st::contactsNameTop, namew, width());
 
 	p.setFont(selected ? st::linkOverFont : st::linkFont);
 	p.setPen(selected ? st::defaultLinkButton.overColor : st::defaultLinkButton.color);
-	p.drawTextRight(st::contactsPadding.right() + st::contactsCheckPosition.x(), st::contactsPadding.top() + (st::contactsPhotoSize - st::normalFont->height) / 2, width(), lang(lng_channels_too_much_public_revoke), _revokeWidth);
+	p.drawTextRight(st::contactsPadding.right() + st::contactsCheckPosition.x(), st::contactsPadding.top() + (st::contactsPhotoSize - st::normalFont->height) / 2, width(), tr::lng_channels_too_much_public_revoke(tr::now), _revokeWidth);
 
 	p.setPen(st::contactsStatusFg);
 	p.setTextPalette(st::revokePublicLinkStatusPalette);
