@@ -11,8 +11,9 @@ https://github.com/telegramdesktop/tdesktop/blob/master/LEGAL
 #include "ui/widgets/labels.h"
 #include "ui/widgets/input_fields.h"
 #include "ui/wrap/fade_wrap.h"
-#include "boxes/confirm_phone_box.h"
 #include "ui/toast/toast.h"
+#include "ui/text/text_utilities.h"
+#include "boxes/confirm_phone_box.h"
 #include "boxes/confirm_box.h"
 #include "auth_session.h"
 #include "data/data_session.h"
@@ -48,7 +49,6 @@ void createErrorLabel(
 			object_ptr<Ui::FlatLabel>(
 				parent,
 				text,
-				Ui::FlatLabel::InitType::Simple,
 				st::changePhoneError));
 		label->hide(anim::type::instant);
 		label->moveToLeft(x, y);
@@ -120,23 +120,23 @@ private:
 };
 
 void ChangePhoneBox::EnterPhone::prepare() {
-	setTitle(langFactory(lng_change_phone_title));
+	setTitle(tr::lng_change_phone_title());
 
 	auto phoneValue = QString();
-	_phone.create(this, st::defaultInputField, langFactory(lng_change_phone_new_title), phoneValue);
+	_phone.create(this, st::defaultInputField, tr::lng_change_phone_new_title(), phoneValue);
 
 	_phone->resize(st::boxWidth - 2 * st::boxPadding.left(), _phone->height());
 	_phone->moveToLeft(st::boxPadding.left(), st::boxLittleSkip);
 	connect(_phone, &Ui::PhoneInput::submitted, [=] { submit(); });
 
-	auto description = object_ptr<Ui::FlatLabel>(this, lang(lng_change_phone_new_description), Ui::FlatLabel::InitType::Simple, st::changePhoneLabel);
+	auto description = object_ptr<Ui::FlatLabel>(this, tr::lng_change_phone_new_description(tr::now), st::changePhoneLabel);
 	auto errorSkip = st::boxLittleSkip + st::changePhoneError.style.font->height;
 	description->moveToLeft(st::boxPadding.left(), _phone->y() + _phone->height() + errorSkip + st::boxLittleSkip);
 
 	setDimensions(st::boxWidth, description->bottomNoMargins() + st::boxLittleSkip);
 
-	addButton(langFactory(lng_change_phone_new_submit), [this] { submit(); });
-	addButton(langFactory(lng_cancel), [this] { closeBox(); });
+	addButton(tr::lng_change_phone_new_submit(), [this] { submit(); });
+	addButton(tr::lng_cancel(), [this] { closeBox(); });
 }
 
 void ChangePhoneBox::EnterPhone::submit() {
@@ -167,22 +167,24 @@ void ChangePhoneBox::EnterPhone::sendPhoneDone(const QString &phoneNumber, const
 
 	auto codeLength = 0;
 	auto &data = result.c_auth_sentCode();
-	switch (data.vtype.type()) {
+	switch (data.vtype().type()) {
 	case mtpc_auth_sentCodeTypeApp:
 		LOG(("Error: should not be in-app code!"));
 		showError(Lang::Hard::ServerError());
 		return;
-	case mtpc_auth_sentCodeTypeSms: codeLength = data.vtype.c_auth_sentCodeTypeSms().vlength.v; break;
-	case mtpc_auth_sentCodeTypeCall: codeLength = data.vtype.c_auth_sentCodeTypeCall().vlength.v; break;
+	case mtpc_auth_sentCodeTypeSms: codeLength = data.vtype().c_auth_sentCodeTypeSms().vlength().v; break;
+	case mtpc_auth_sentCodeTypeCall: codeLength = data.vtype().c_auth_sentCodeTypeCall().vlength().v; break;
 	case mtpc_auth_sentCodeTypeFlashCall:
 		LOG(("Error: should not be flashcall!"));
 		showError(Lang::Hard::ServerError());
 		return;
 	}
-	auto phoneCodeHash = qs(data.vphone_code_hash);
+	auto phoneCodeHash = qs(data.vphone_code_hash());
 	auto callTimeout = 0;
-	if (data.has_next_type() && data.vnext_type.type() == mtpc_auth_codeTypeCall) {
-		callTimeout = data.has_timeout() ? data.vtimeout.v : 60;
+	if (const auto nextType = data.vnext_type()) {
+		if (nextType->type() == mtpc_auth_codeTypeCall) {
+			callTimeout = data.vtimeout().value_or(60);
+		}
 	}
 	Ui::show(
 		Box<EnterCode>(
@@ -196,13 +198,22 @@ void ChangePhoneBox::EnterPhone::sendPhoneDone(const QString &phoneNumber, const
 bool ChangePhoneBox::EnterPhone::sendPhoneFail(const QString &phoneNumber, const RPCError &error) {
 	auto errorText = Lang::Hard::ServerError();
 	if (MTP::isFloodError(error)) {
-		errorText = lang(lng_flood_error);
+		errorText = tr::lng_flood_error(tr::now);
 	} else if (MTP::isDefaultHandledError(error)) {
 		return false;
 	} else if (error.type() == qstr("PHONE_NUMBER_INVALID")) {
-		errorText = lang(lng_bad_phone);
+		errorText = tr::lng_bad_phone(tr::now);
+	} else if (error.type() == qstr("PHONE_NUMBER_BANNED")) {
+		ShowPhoneBannedError(phoneNumber);
+		_requestId = 0;
+		return true;
 	} else if (error.type() == qstr("PHONE_NUMBER_OCCUPIED")) {
-		Ui::show(Box<InformBox>(lng_change_phone_occupied(lt_phone, App::formatPhone(phoneNumber)), lang(lng_box_ok)));
+		Ui::show(Box<InformBox>(
+			tr::lng_change_phone_occupied(
+				tr::now,
+				lt_phone,
+				App::formatPhone(phoneNumber)),
+			tr::lng_box_ok(tr::now)));
 		_requestId = 0;
 		return true;
 	}
@@ -227,14 +238,18 @@ ChangePhoneBox::EnterCode::EnterCode(QWidget*, const QString &phone, const QStri
 }
 
 void ChangePhoneBox::EnterCode::prepare() {
-	setTitle(langFactory(lng_change_phone_title));
+	setTitle(tr::lng_change_phone_title());
 
-	auto descriptionText = lng_change_phone_code_description(lt_phone, textcmdStartSemibold() + App::formatPhone(_phone) + textcmdStopSemibold());
-	auto description = object_ptr<Ui::FlatLabel>(this, descriptionText, Ui::FlatLabel::InitType::Rich, st::changePhoneLabel);
+	auto descriptionText = tr::lng_change_phone_code_description(
+		tr::now,
+		lt_phone,
+		Ui::Text::Bold(App::formatPhone(_phone)),
+		Ui::Text::WithEntities);
+	auto description = object_ptr<Ui::FlatLabel>(this, rpl::single(descriptionText), st::changePhoneLabel);
 	description->moveToLeft(st::boxPadding.left(), 0);
 
 	auto phoneValue = QString();
-	_code.create(this, st::defaultInputField, langFactory(lng_change_phone_code_title), phoneValue);
+	_code.create(this, st::defaultInputField, tr::lng_change_phone_code_title(), phoneValue);
 	_code->setAutoSubmit(_codeLength, [=] { submit(); });
 	_code->setChangedCallback([=] { hideError(); });
 
@@ -249,8 +264,8 @@ void ChangePhoneBox::EnterCode::prepare() {
 		updateCall();
 	}
 
-	addButton(langFactory(lng_change_phone_new_submit), [=] { submit(); });
-	addButton(langFactory(lng_cancel), [=] { closeBox(); });
+	addButton(tr::lng_change_phone_new_submit(), [=] { submit(); });
+	addButton(tr::lng_cancel(), [=] { closeBox(); });
 }
 
 int ChangePhoneBox::EnterCode::countHeight() {
@@ -274,7 +289,7 @@ void ChangePhoneBox::EnterCode::submit() {
 		if (weak) {
 			Ui::hideLayer();
 		}
-		Ui::Toast::Show(lang(lng_change_phone_success));
+		Ui::Toast::Show(tr::lng_change_phone_success(tr::now));
 	}), rpcFail(crl::guard(this, [this](const RPCError &error) {
 		return sendCodeFail(error);
 	})));
@@ -291,7 +306,7 @@ void ChangePhoneBox::EnterCode::updateCall() {
 	if (text.isEmpty()) {
 		_callLabel.destroy();
 	} else if (!_callLabel) {
-		_callLabel.create(this, text, Ui::FlatLabel::InitType::Simple, st::changePhoneLabel);
+		_callLabel.create(this, text, st::changePhoneLabel);
 		_callLabel->moveToLeft(st::boxPadding.left(), countHeight() - _callLabel->height());
 		_callLabel->show();
 	} else {
@@ -309,17 +324,18 @@ void ChangePhoneBox::EnterCode::showError(const QString &text) {
 bool ChangePhoneBox::EnterCode::sendCodeFail(const RPCError &error) {
 	auto errorText = Lang::Hard::ServerError();
 	if (MTP::isFloodError(error)) {
-		errorText = lang(lng_flood_error);
+		errorText = tr::lng_flood_error(tr::now);
 	} else if (MTP::isDefaultHandledError(error)) {
 		return false;
 	} else if (error.type() == qstr("PHONE_CODE_EMPTY") || error.type() == qstr("PHONE_CODE_INVALID")) {
-		errorText = lang(lng_bad_code);
-	} else if (error.type() == qstr("PHONE_CODE_EXPIRED")) {
+		errorText = tr::lng_bad_code(tr::now);
+	} else if (error.type() == qstr("PHONE_CODE_EXPIRED")
+		|| error.type() == qstr("PHONE_NUMBER_BANNED")) {
 		closeBox(); // Go back to phone input.
 		_requestId = 0;
 		return true;
 	} else if (error.type() == qstr("PHONE_NUMBER_INVALID")) {
-		errorText = lang(lng_bad_phone);
+		errorText = tr::lng_bad_phone(tr::now);
 	}
 	_requestId = 0;
 	showError(errorText);
@@ -327,17 +343,20 @@ bool ChangePhoneBox::EnterCode::sendCodeFail(const RPCError &error) {
 }
 
 void ChangePhoneBox::prepare() {
-	setTitle(langFactory(lng_change_phone_title));
-	addButton(langFactory(lng_change_phone_button), [] {
-		Ui::show(Box<ConfirmBox>(lang(lng_change_phone_warning), [] {
+	setTitle(tr::lng_change_phone_title());
+	addButton(tr::lng_change_phone_button(), [] {
+		Ui::show(Box<ConfirmBox>(tr::lng_change_phone_warning(tr::now), [] {
 			Ui::show(Box<EnterPhone>());
 		}));
 	});
-	addButton(langFactory(lng_cancel), [this] {
+	addButton(tr::lng_cancel(), [this] {
 		closeBox();
 	});
 
-	auto label = object_ptr<Ui::FlatLabel>(this, lang(lng_change_phone_description), Ui::FlatLabel::InitType::Rich, st::changePhoneDescription);
+	const auto label = Ui::CreateChild<Ui::FlatLabel>(
+		this,
+		tr::lng_change_phone_about(Ui::Text::RichLangValue),
+		st::changePhoneDescription);
 	label->moveToLeft((st::boxWideWidth - label->width()) / 2, st::changePhoneDescriptionTop);
 
 	setDimensions(st::boxWideWidth, label->bottomNoMargins() + st::boxLittleSkip);
