@@ -16,12 +16,15 @@ https://github.com/telegramdesktop/tdesktop/blob/master/LEGAL
 #include "boxes/peer_list_controllers.h"
 #include "info/profile/info_profile_button.h"
 #include "settings/settings_common.h"
+#include "settings/settings_privacy_security.h"
 #include "calls/calls_instance.h"
 #include "base/binary_guard.h"
 #include "lang/lang_keys.h"
 #include "apiwrap.h"
 #include "auth_session.h"
 #include "data/data_user.h"
+#include "data/data_chat.h"
+#include "data/data_channel.h"
 #include "styles/style_settings.h"
 #include "styles/style_boxes.h"
 
@@ -29,73 +32,88 @@ namespace {
 
 class PrivacyExceptionsBoxController : public ChatsListBoxController {
 public:
-	PrivacyExceptionsBoxController(Fn<QString()> titleFactory, const std::vector<not_null<UserData*>> &selected);
+	PrivacyExceptionsBoxController(
+		rpl::producer<QString> title,
+		const std::vector<not_null<PeerData*>> &selected);
 	void rowClicked(not_null<PeerListRow*> row) override;
 
-	std::vector<not_null<UserData*>> getResult() const;
+	std::vector<not_null<PeerData*>> getResult() const;
 
 protected:
 	void prepareViewHook() override;
 	std::unique_ptr<Row> createRow(not_null<History*> history) override;
 
 private:
-	Fn<QString()> _titleFactory;
-	std::vector<not_null<UserData*>> _selected;
+	rpl::producer<QString> _title;
+	std::vector<not_null<PeerData*>> _selected;
 
 };
 
-PrivacyExceptionsBoxController::PrivacyExceptionsBoxController(Fn<QString()> titleFactory, const std::vector<not_null<UserData*>> &selected)
-: _titleFactory(std::move(titleFactory))
+PrivacyExceptionsBoxController::PrivacyExceptionsBoxController(
+	rpl::producer<QString> title,
+	const std::vector<not_null<PeerData*>> &selected)
+: _title(std::move(title))
 , _selected(selected) {
 }
 
 void PrivacyExceptionsBoxController::prepareViewHook() {
-	delegate()->peerListSetTitle(_titleFactory);
+	delegate()->peerListSetTitle(std::move(_title));
 	delegate()->peerListAddSelectedRows(_selected);
 }
 
-std::vector<not_null<UserData*>> PrivacyExceptionsBoxController::getResult() const {
-	auto peers = delegate()->peerListCollectSelectedRows();
-	auto users = std::vector<not_null<UserData*>>();
-	if (!peers.empty()) {
-		users.reserve(peers.size());
-		for_const (auto peer, peers) {
-			auto user = peer->asUser();
-			Assert(user != nullptr);
-			users.push_back(user);
-		}
-	}
-	return users;
+std::vector<not_null<PeerData*>> PrivacyExceptionsBoxController::getResult() const {
+	return delegate()->peerListCollectSelectedRows();
 }
 
 void PrivacyExceptionsBoxController::rowClicked(not_null<PeerListRow*> row) {
 	delegate()->peerListSetRowChecked(row, !row->checked());
+	if (const auto channel = row->peer()->asChannel()) {
+		if (!channel->membersCountKnown()) {
+			channel->updateFull();
+		}
+	}
 }
 
 std::unique_ptr<PrivacyExceptionsBoxController::Row> PrivacyExceptionsBoxController::createRow(not_null<History*> history) {
 	if (history->peer->isSelf()) {
 		return nullptr;
+	} else if (!history->peer->isUser()
+		&& !history->peer->isChat()
+		&& !history->peer->isMegagroup()) {
+		return nullptr;
 	}
-	if (auto user = history->peer->asUser()) {
-		return std::make_unique<Row>(history);
+	auto result = std::make_unique<Row>(history);
+	const auto count = [&] {
+		if (const auto chat = history->peer->asChat()) {
+			return chat->count;
+		} else if (const auto channel = history->peer->asChannel()) {
+			return channel->membersCountKnown()
+				? channel->membersCount()
+				: 0;
+		}
+		return 0;
+	}();
+	if (count > 0) {
+		result->setCustomStatus(
+			tr::lng_chat_status_members(tr::now, lt_count_decimal, count));
 	}
-	return nullptr;
+	return result;
 }
 
 } // namespace
 
-LangKey EditPrivacyBox::Controller::optionLabelKey(Option option) {
+QString EditPrivacyController::optionLabel(Option option) {
 	switch (option) {
-	case Option::Everyone: return lng_edit_privacy_everyone;
-	case Option::Contacts: return lng_edit_privacy_contacts;
-	case Option::Nobody: return lng_edit_privacy_nobody;
+	case Option::Everyone: return tr::lng_edit_privacy_everyone(tr::now);
+	case Option::Contacts: return tr::lng_edit_privacy_contacts(tr::now);
+	case Option::Nobody: return tr::lng_edit_privacy_nobody(tr::now);
 	}
 	Unexpected("Option value in optionsLabelKey.");
 }
 
 EditPrivacyBox::EditPrivacyBox(
 	QWidget*,
-	std::unique_ptr<Controller> controller,
+	std::unique_ptr<EditPrivacyController> controller,
 	const Value &value)
 : _controller(std::move(controller))
 , _value(value) {
@@ -107,34 +125,33 @@ void EditPrivacyBox::prepare() {
 	setupContent();
 }
 
-void EditPrivacyBox::editExceptionUsers(
+void EditPrivacyBox::editExceptions(
 		Exception exception,
 		Fn<void()> done) {
 	auto controller = std::make_unique<PrivacyExceptionsBoxController>(
-		crl::guard(this, [=] {
-			return _controller->exceptionBoxTitle(exception);
-		}),
-		exceptionUsers(exception));
+		_controller->exceptionBoxTitle(exception),
+		exceptions(exception));
 	auto initBox = [=, controller = controller.get()](
 			not_null<PeerListBox*> box) {
-		box->addButton(langFactory(lng_settings_save), crl::guard(this, [=] {
-			exceptionUsers(exception) = controller->getResult();
-			const auto removeFrom = ([=] {
+		box->addButton(tr::lng_settings_save(), crl::guard(this, [=] {
+			exceptions(exception) = controller->getResult();
+			const auto type = [&] {
 				switch (exception) {
 				case Exception::Always: return Exception::Never;
 				case Exception::Never: return Exception::Always;
 				}
 				Unexpected("Invalid exception value.");
-			})();
-			auto &removeFromUsers = exceptionUsers(removeFrom);
-			for (const auto user : exceptionUsers(exception)) {
-				const auto from = ranges::remove(removeFromUsers, user);
-				removeFromUsers.erase(from, end(removeFromUsers));
+			}();
+			auto &removeFrom = exceptions(type);
+			for (const auto peer : exceptions(exception)) {
+				removeFrom.erase(
+					ranges::remove(removeFrom, peer),
+					end(removeFrom));
 			}
 			done();
 			box->closeBox();
 		}));
-		box->addButton(langFactory(lng_cancel), [=] { box->closeBox(); });
+		box->addButton(tr::lng_cancel(), [=] { box->closeBox(); });
 	};
 	Ui::show(
 		Box<PeerListBox>(std::move(controller), std::move(initBox)),
@@ -142,11 +159,23 @@ void EditPrivacyBox::editExceptionUsers(
 }
 
 QVector<MTPInputPrivacyRule> EditPrivacyBox::collectResult() {
-	auto collectInputUsers = [](auto &users) {
+	const auto collectInputUsers = [](const auto &peers) {
 		auto result = QVector<MTPInputUser>();
-		result.reserve(users.size());
-		for (auto user : users) {
-			result.push_back(user->inputUser);
+		result.reserve(peers.size());
+		for (const auto peer : peers) {
+			if (const auto user = peer->asUser()) {
+				result.push_back(user->inputUser);
+			}
+		}
+		return result;
+	};
+	const auto collectInputChats = [](const auto &peers) {
+		auto result = QVector<MTPint>();
+		result.reserve(peers.size());
+		for (const auto peer : peers) {
+			if (!peer->isUser()) {
+				result.push_back(MTP_int(peer->bareId()));
+			}
 		}
 		return result;
 	};
@@ -154,11 +183,25 @@ QVector<MTPInputPrivacyRule> EditPrivacyBox::collectResult() {
 	constexpr auto kMaxRules = 3; // allow users, disallow users, option
 	auto result = QVector<MTPInputPrivacyRule>();
 	result.reserve(kMaxRules);
-	if (showExceptionLink(Exception::Always) && !_value.always.empty()) {
-		result.push_back(MTP_inputPrivacyValueAllowUsers(MTP_vector<MTPInputUser>(collectInputUsers(_value.always))));
+	if (showExceptionLink(Exception::Always)) {
+		const auto users = collectInputUsers(_value.always);
+		const auto chats = collectInputChats(_value.always);
+		if (!users.empty()) {
+			result.push_back(MTP_inputPrivacyValueAllowUsers(MTP_vector<MTPInputUser>(users)));
+		}
+		if (!chats.empty()) {
+			result.push_back(MTP_inputPrivacyValueAllowChatParticipants(MTP_vector<MTPint>(chats)));
+		}
 	}
-	if (showExceptionLink(Exception::Never) && !_value.never.empty()) {
-		result.push_back(MTP_inputPrivacyValueDisallowUsers(MTP_vector<MTPInputUser>(collectInputUsers(_value.never))));
+	if (showExceptionLink(Exception::Never)) {
+		const auto users = collectInputUsers(_value.never);
+		const auto chats = collectInputChats(_value.never);
+		if (!users.empty()) {
+			result.push_back(MTP_inputPrivacyValueDisallowUsers(MTP_vector<MTPInputUser>(users)));
+		}
+		if (!chats.empty()) {
+			result.push_back(MTP_inputPrivacyValueDisallowChatParticipants(MTP_vector<MTPint>(chats)));
+		}
 	}
 	result.push_back([&] {
 		switch (_value.option) {
@@ -172,7 +215,7 @@ QVector<MTPInputPrivacyRule> EditPrivacyBox::collectResult() {
 	return result;
 }
 
-std::vector<not_null<UserData*>> &EditPrivacyBox::exceptionUsers(Exception exception) {
+std::vector<not_null<PeerData*>> &EditPrivacyBox::exceptions(Exception exception) {
 	switch (exception) {
 	case Exception::Always: return _value.always;
 	case Exception::Never: return _value.never;
@@ -201,7 +244,7 @@ Ui::Radioenum<EditPrivacyBox::Option> *EditPrivacyBox::addOption(
 			container,
 			group,
 			option,
-			lang(_controller->optionLabelKey(option)),
+			_controller->optionLabel(option),
 			st::settingsSendType),
 		st::settingsSendTypePadding);
 }
@@ -229,7 +272,7 @@ Ui::FlatLabel *EditPrivacyBox::addLabel(
 void EditPrivacyBox::setupContent() {
 	using namespace Settings;
 
-	setTitle([=] { return _controller->title(); });
+	setTitle(_controller->title());
 
 	auto wrap = object_ptr<Ui::VerticalLayout>(this);
 	const auto content = wrap.data();
@@ -256,11 +299,11 @@ void EditPrivacyBox::setupContent() {
 		auto label = update->events_starting_with(
 			rpl::empty_value()
 		) | rpl::map([=] {
-			return exceptionUsers(exception).size();
+			return Settings::ExceptionUsersCount(exceptions(exception));
 		}) | rpl::map([](int count) {
 			return count
-				? lng_edit_privacy_exceptions_count(lt_count, count)
-				: lang(lng_edit_privacy_exceptions_add);
+				? tr::lng_edit_privacy_exceptions_count(tr::now, lt_count, count)
+				: tr::lng_edit_privacy_exceptions_add(tr::now);
 		});
 		auto text = _controller->exceptionButtonTextKey(exception);
 		const auto button = content->add(
@@ -268,19 +311,19 @@ void EditPrivacyBox::setupContent() {
 				content,
 				object_ptr<Button>(
 					content,
-					Lang::Viewer(text),
+					rpl::duplicate(text),
 					st::settingsButton)));
 		CreateRightLabel(
 			button->entity(),
 			std::move(label),
 			st::settingsButton,
-			text);
+			std::move(text));
 		button->toggleOn(rpl::duplicate(
 			optionValue
 		) | rpl::map([=] {
 			return showExceptionLink(exception);
 		}))->entity()->addClickHandler([=] {
-			editExceptionUsers(exception, [=] { update->fire({}); });
+			editExceptions(exception, [=] { update->fire({}); });
 		});
 		return button;
 	};
@@ -301,13 +344,17 @@ void EditPrivacyBox::setupContent() {
 
 	AddDivider(content);
 	AddSkip(content);
-	AddSubsectionTitle(content, lng_edit_privacy_exceptions);
+	AddSubsectionTitle(content, tr::lng_edit_privacy_exceptions());
 	const auto always = addExceptionLink(Exception::Always);
 	const auto never = addExceptionLink(Exception::Never);
 	addLabel(content, _controller->exceptionsDescription());
 	AddSkip(content);
 
-	addButton(langFactory(lng_settings_save), [=] {
+	if (auto below = _controller->setupBelowWidget(content)) {
+		content->add(std::move(below));
+	}
+
+	addButton(tr::lng_settings_save(), [=] {
 		const auto someAreDisallowed = (_value.option != Option::Everyone)
 			|| !_value.never.empty();
 		_controller->confirmSave(someAreDisallowed, crl::guard(this, [=] {
@@ -317,7 +364,7 @@ void EditPrivacyBox::setupContent() {
 			closeBox();
 		}));
 	});
-	addButton(langFactory(lng_cancel), [this] { closeBox(); });
+	addButton(tr::lng_cancel(), [this] { closeBox(); });
 
 	const auto linkHeight = st::settingsButton.padding.top()
 		+ st::settingsButton.height
