@@ -24,7 +24,7 @@ https://github.com/telegramdesktop/tdesktop/blob/master/LEGAL
 #include "data/data_chat.h"
 #include "data/data_user.h"
 #include "ui/widgets/popup_menu.h"
-#include "window/window_controller.h"
+#include "window/window_session_controller.h"
 #include "history/history.h"
 
 namespace {
@@ -131,24 +131,7 @@ void SaveChannelAdmin(
 			onDone();
 		}
 	}).fail([=](const RPCError &error) {
-		if (error.type() == qstr("USER_NOT_MUTUAL_CONTACT")) {
-			Ui::show(
-				Box<InformBox>(PeerFloodErrorText(
-					channel->isMegagroup()
-					? PeerFloodType::InviteGroup
-					: PeerFloodType::InviteChannel)),
-				LayerOption::KeepOther);
-		} else if (error.type() == qstr("BOT_GROUPS_BLOCKED")) {
-			Ui::show(
-				Box<InformBox>(lang(lng_error_cant_add_bot)),
-				LayerOption::KeepOther);
-		} else if (error.type() == qstr("ADMINS_TOO_MUCH")) {
-			Ui::show(
-				Box<InformBox>(lang(channel->isMegagroup()
-					? lng_error_admin_limit
-					: lng_error_admin_limit_channel)),
-				LayerOption::KeepOther);
-		}
+		ShowAddParticipantsError(error.type(), channel, { 1, user });
 		if (onFail) {
 			onFail();
 		}
@@ -227,7 +210,7 @@ Fn<void(
 			};
 			const auto flags = newRights.match([](
 					const MTPDchatAdminRights &data) {
-				return data.vflags.v;
+				return data.vflags().v;
 			});
 			if (flags == ChatData::DefaultAdminRights()) {
 				saveChatAdmin(true);
@@ -267,7 +250,7 @@ Fn<void(
 		if (const auto chat = peer->asChatNotMigrated()) {
 			const auto flags = newRights.match([](
 					const MTPDchatBannedRights &data) {
-				return data.vflags.v;
+				return data.vflags().v;
 			});
 			if (flags & MTPDchatBannedRights::Flag::f_view_messages) {
 				SaveChatParticipantKick(chat, user, done, onFail);
@@ -527,13 +510,13 @@ UserData *ParticipantsAdditionalData::applyParticipant(
 			&& overrideRole != Role::Members) {
 			return logBad();
 		}
-		return applyRegular(data.vuser_id);
+		return applyRegular(data.vuser_id());
 	}, [&](const MTPDchannelParticipant &data) {
 		if (overrideRole != Role::Profile
 			&& overrideRole != Role::Members) {
 			return logBad();
 		}
-		return applyRegular(data.vuser_id);
+		return applyRegular(data.vuser_id());
 	}, [&](const MTPDchannelParticipantBanned &data) {
 		if (overrideRole != Role::Profile
 			&& overrideRole != Role::Members
@@ -547,7 +530,7 @@ UserData *ParticipantsAdditionalData::applyParticipant(
 
 UserData *ParticipantsAdditionalData::applyCreator(
 		const MTPDchannelParticipantCreator &data) {
-	if (const auto user = applyRegular(data.vuser_id)) {
+	if (const auto user = applyRegular(data.vuser_id())) {
 		_creator = user;
 		return user;
 	}
@@ -556,7 +539,7 @@ UserData *ParticipantsAdditionalData::applyCreator(
 
 UserData *ParticipantsAdditionalData::applyAdmin(
 		const MTPDchannelParticipantAdmin &data) {
-	const auto user = _peer->owner().userLoaded(data.vuser_id.v);
+	const auto user = _peer->owner().userLoaded(data.vuser_id().v);
 	if (!user) {
 		return nullptr;
 	} else if (const auto chat = _peer->asChat()) {
@@ -569,13 +552,13 @@ UserData *ParticipantsAdditionalData::applyAdmin(
 	_restrictedRights.erase(user);
 	_kicked.erase(user);
 	_restrictedBy.erase(user);
-	_adminRights[user] = data.vadmin_rights;
+	_adminRights[user] = data.vadmin_rights();
 	if (data.is_can_edit()) {
 		_adminCanEdit.emplace(user);
 	} else {
 		_adminCanEdit.erase(user);
 	}
-	if (const auto by = _peer->owner().userLoaded(data.vpromoted_by.v)) {
+	if (const auto by = _peer->owner().userLoaded(data.vpromoted_by().v)) {
 		const auto i = _adminPromotedBy.find(user);
 		if (i == _adminPromotedBy.end()) {
 			_adminPromotedBy.emplace(user, by);
@@ -584,7 +567,7 @@ UserData *ParticipantsAdditionalData::applyAdmin(
 		}
 	} else {
 		LOG(("API Error: No user %1 for admin promoted by."
-			).arg(data.vpromoted_by.v));
+			).arg(data.vpromoted_by().v));
 	}
 	return user;
 }
@@ -611,7 +594,7 @@ UserData *ParticipantsAdditionalData::applyRegular(MTPint userId) {
 
 UserData *ParticipantsAdditionalData::applyBanned(
 		const MTPDchannelParticipantBanned &data) {
-	const auto user = _peer->owner().userLoaded(data.vuser_id.v);
+	const auto user = _peer->owner().userLoaded(data.vuser_id().v);
 	if (!user) {
 		return nullptr;
 	}
@@ -625,8 +608,8 @@ UserData *ParticipantsAdditionalData::applyBanned(
 	} else {
 		_kicked.erase(user);
 	}
-	_restrictedRights[user] = data.vbanned_rights;
-	if (const auto by = _peer->owner().userLoaded(data.vkicked_by.v)) {
+	_restrictedRights[user] = data.vbanned_rights();
+	if (const auto by = _peer->owner().userLoaded(data.vkicked_by().v)) {
 		const auto i = _restrictedBy.find(user);
 		if (i == _restrictedBy.end()) {
 			_restrictedBy.emplace(user, by);
@@ -723,7 +706,7 @@ ParticipantsBoxController::SavedState::SavedState(
 }
 
 ParticipantsBoxController::ParticipantsBoxController(
-	not_null<Window::Navigation*> navigation,
+	not_null<Window::SessionNavigation*> navigation,
 	not_null<PeerData*> peer,
 	Role role)
 : PeerListController(CreateSearchController(peer, role, &_additional))
@@ -735,6 +718,9 @@ ParticipantsBoxController::ParticipantsBoxController(
 	if (_role == Role::Profile) {
 		setupListChangeViewers();
 	}
+	if (const auto channel = _peer->asChannel()) {
+		subscribeToCreatorChange(channel);
+	}
 }
 
 void ParticipantsBoxController::setupListChangeViewers() {
@@ -792,7 +778,7 @@ auto ParticipantsBoxController::CreateSearchController(
 }
 
 void ParticipantsBoxController::Start(
-		not_null<Window::Navigation*> navigation,
+		not_null<Window::SessionNavigation*> navigation,
 		not_null<PeerData*> peer,
 		Role role) {
 	auto controller = std::make_unique<ParticipantsBoxController>(
@@ -801,7 +787,7 @@ void ParticipantsBoxController::Start(
 		role);
 	auto initBox = [=, controller = controller.get()](
 			not_null<PeerListBox*> box) {
-		box->addButton(langFactory(lng_close), [=] { box->closeBox(); });
+		box->addButton(tr::lng_close(), [=] { box->closeBox(); });
 
 		const auto chat = peer->asChat();
 		const auto channel = peer->asChannel();
@@ -828,23 +814,23 @@ void ParticipantsBoxController::Start(
 
 			Unexpected("Role value in ParticipantsBoxController::Start()");
 		}();
-		const auto addNewItemText = [&] {
+		auto addNewItemText = [&] {
 			switch (role) {
 			case Role::Members:
-				return langFactory((chat || channel->isMegagroup())
-					? lng_channel_add_members
-					: lng_channel_add_users);
+				return (chat || channel->isMegagroup())
+					? tr::lng_channel_add_members()
+					: tr::lng_channel_add_users();
 			case Role::Admins:
-				return langFactory(lng_channel_add_admin);
+				return tr::lng_channel_add_admin();
 			case Role::Restricted:
-				return langFactory(lng_channel_add_exception);
+				return tr::lng_channel_add_exception();
 			case Role::Kicked:
-				return langFactory(lng_channel_add_removed);
+				return tr::lng_channel_add_removed();
 			}
 			Unexpected("Role value in ParticipantsBoxController::Start()");
 		}();
 		if (canAddNewItem) {
-			box->addLeftButton(addNewItemText, [=] {
+			box->addLeftButton(std::move(addNewItemText), [=] {
 				controller->addNewItem();
 			});
 		}
@@ -872,7 +858,7 @@ void ParticipantsBoxController::addNewItem() {
 		editRestrictedDone(user, rights);
 	});
 	const auto initBox = [](not_null<PeerListBox*> box) {
-		box->addButton(langFactory(lng_cancel), [=] { box->closeBox(); });
+		box->addButton(tr::lng_cancel(), [=] { box->closeBox(); });
 	};
 
 	_addBox = Ui::show(
@@ -1026,20 +1012,20 @@ rpl::producer<int> ParticipantsBoxController::onlineCountValue() const {
 }
 
 void ParticipantsBoxController::prepare() {
-	const auto titleKey = [&] {
+	auto title = [&] {
 		switch (_role) {
-		case Role::Admins: return lng_channel_admins;
+		case Role::Admins: return tr::lng_channel_admins();
 		case Role::Profile:
-		case Role::Members: return lng_profile_participants_section;
-		case Role::Restricted: return lng_exceptions_list_title;
-		case Role::Kicked: return lng_removed_list_title;
+		case Role::Members: return tr::lng_profile_participants_section();
+		case Role::Restricted: return tr::lng_exceptions_list_title();
+		case Role::Kicked: return tr::lng_removed_list_title();
 		}
 		Unexpected("Role in ParticipantsBoxController::prepare()");
 	}();
 	delegate()->peerListSetSearchMode(PeerListSearchMode::Enabled);
-	delegate()->peerListSetTitle(langFactory(titleKey));
-	setDescriptionText(lang(lng_contacts_loading));
-	setSearchNoResultsText(lang(lng_blocked_list_not_found));
+	delegate()->peerListSetTitle(std::move(title));
+	setDescriptionText(tr::lng_contacts_loading(tr::now));
+	setSearchNoResultsText(tr::lng_blocked_list_not_found(tr::now));
 
 	if (const auto chat = _peer->asChat()) {
 		prepareChatRows(chat);
@@ -1215,9 +1201,9 @@ void ParticipantsBoxController::loadMoreRows() {
 		} else if (_role == Role::Admins) {
 			return MTP_channelParticipantsAdmins();
 		} else if (_role == Role::Restricted) {
-			return MTP_channelParticipantsBanned(MTP_string(QString()));
+			return MTP_channelParticipantsBanned(MTP_string());
 		}
-		return MTP_channelParticipantsKicked(MTP_string(QString()));
+		return MTP_channelParticipantsKicked(MTP_string());
 	}();
 
 	// First query is small and fast, next loads a lot of rows.
@@ -1282,12 +1268,12 @@ void ParticipantsBoxController::loadMoreRows() {
 
 void ParticipantsBoxController::refreshDescription() {
 	setDescriptionText((_role == Role::Kicked)
-		? lang((_peer->isChat() || _peer->isMegagroup())
-			? lng_group_removed_list_about
-			: lng_channel_removed_list_about)
+		? ((_peer->isChat() || _peer->isMegagroup())
+			? tr::lng_group_removed_list_about
+			: tr::lng_channel_removed_list_about)(tr::now)
 		: (delegate()->peerListFullRowsCount() > 0)
 		? QString()
-		: lang(lng_blocked_list_not_found));
+		: tr::lng_blocked_list_not_found(tr::now));
 }
 
 bool ParticipantsBoxController::feedMegagroupLastParticipants() {
@@ -1339,7 +1325,7 @@ void ParticipantsBoxController::rowClicked(not_null<PeerListRow*> row) {
 	const auto user = row->peer()->asUser();
 	if (_role == Role::Admins) {
 		showAdmin(user);
-	} else if ((_role == Role::Restricted || _role == Role::Kicked)
+	} else if (_role == Role::Restricted
 		&& (_peer->isChat() || _peer->isMegagroup())) {
 		showRestricted(user);
 	} else {
@@ -1371,18 +1357,18 @@ base::unique_qptr<Ui::PopupMenu> ParticipantsBoxController::rowContextMenu(
 	const auto user = row->peer()->asUser();
 	auto result = base::make_unique_q<Ui::PopupMenu>(parent);
 	result->addAction(
-		lang(lng_context_view_profile),
+		tr::lng_context_view_profile(tr::now),
 		crl::guard(this, [=] { _navigation->showPeerInfo(user); }));
 	if (_role == Role::Kicked) {
 		if (_peer->isMegagroup()
 			&& _additional.canRestrictUser(user)) {
 			if (channel->canAddMembers()) {
 				result->addAction(
-					lang(lng_context_add_to_group),
+					tr::lng_context_add_to_group(tr::now),
 					crl::guard(this, [=] { unkickMember(user); }));
 			}
 			result->addAction(
-				lang(lng_profile_delete_removed),
+				tr::lng_profile_delete_removed(tr::now),
 				crl::guard(this, [=] { removeKickedWithRow(user); }));
 		}
 		return result;
@@ -1390,11 +1376,10 @@ base::unique_qptr<Ui::PopupMenu> ParticipantsBoxController::rowContextMenu(
 	if (_additional.canAddOrEditAdmin(user)) {
 		const auto isAdmin = _additional.isCreator(user)
 			|| _additional.adminRights(user).has_value();
-		const auto labelKey = isAdmin
-			? lng_context_edit_permissions
-			: lng_context_promote_admin;
 		result->addAction(
-			lang(labelKey),
+			(isAdmin
+				? tr::lng_context_edit_permissions
+				: tr::lng_context_promote_admin)(tr::now),
 			crl::guard(this, [=] { showAdmin(user); }));
 	}
 	if (_additional.canRestrictUser(user)) {
@@ -1406,15 +1391,15 @@ base::unique_qptr<Ui::PopupMenu> ParticipantsBoxController::rowContextMenu(
 		}();
 		if (canRestrictWithoutKick) {
 			result->addAction(
-				lang(lng_context_restrict_user),
+				tr::lng_context_restrict_user(tr::now),
 				crl::guard(this, [=] { showRestricted(user); }));
 		}
 		if (!_additional.isKicked(user)) {
 			const auto isGroup = _peer->isChat() || _peer->isMegagroup();
 			result->addAction(
-				lang(isGroup
-					? lng_context_remove_from_group
-					: lng_profile_kick),
+				(isGroup
+					? tr::lng_context_remove_from_group
+					: tr::lng_profile_kick)(tr::now),
 				crl::guard(this, [=] { kickMember(user); }));
 		}
 	}
@@ -1439,21 +1424,25 @@ void ParticipantsBoxController::showAdmin(not_null<UserData*> user) {
 			editAdminDone(user, newRights);
 		});
 		const auto fail = crl::guard(this, [=] {
-			_editBox = nullptr;
+			if (_editParticipantBox) {
+				_editParticipantBox->closeBox();
+			}
 		});
 		box->setSaveCallback(SaveAdminCallback(_peer, user, done, fail));
 	}
-	_editBox = Ui::show(std::move(box), LayerOption::KeepOther);
+	_editParticipantBox = Ui::show(std::move(box), LayerOption::KeepOther);
 }
 
 void ParticipantsBoxController::editAdminDone(
 		not_null<UserData*> user,
 		const MTPChatAdminRights &rights) {
 	_addBox = nullptr;
-	_editBox = nullptr;
+	if (_editParticipantBox) {
+		_editParticipantBox->closeBox();
+	}
 
 	const auto date = unixtime(); // Incorrect, but ignored.
-	if (rights.c_chatAdminRights().vflags.v == 0) {
+	if (rights.c_chatAdminRights().vflags().v == 0) {
 		_additional.applyParticipant(MTP_channelParticipant(
 			MTP_int(user->bareId()),
 			MTP_int(date)));
@@ -1502,22 +1491,26 @@ void ParticipantsBoxController::showRestricted(not_null<UserData*> user) {
 			editRestrictedDone(user, newRights);
 		});
 		const auto fail = crl::guard(this, [=] {
-			_editBox = nullptr;
+			if (_editParticipantBox) {
+				_editParticipantBox->closeBox();
+			}
 		});
 		box->setSaveCallback(
 			SaveRestrictedCallback(_peer, user, done, fail));
 	}
-	_editBox = Ui::show(std::move(box), LayerOption::KeepOther);
+	_editParticipantBox = Ui::show(std::move(box), LayerOption::KeepOther);
 }
 
 void ParticipantsBoxController::editRestrictedDone(
 		not_null<UserData*> user,
 		const MTPChatBannedRights &rights) {
 	_addBox = nullptr;
-	_editBox = nullptr;
+	if (_editParticipantBox) {
+		_editParticipantBox->closeBox();
+	}
 
 	const auto date = unixtime(); // Incorrect, but ignored.
-	if (rights.c_chatBannedRights().vflags.v == 0) {
+	if (rights.c_chatBannedRights().vflags().v == 0) {
 		_additional.applyParticipant(MTP_channelParticipant(
 			MTP_int(user->bareId()),
 			MTP_int(date)));
@@ -1561,12 +1554,15 @@ void ParticipantsBoxController::editRestrictedDone(
 
 void ParticipantsBoxController::kickMember(not_null<UserData*> user) {
 	const auto text = ((_peer->isChat() || _peer->isMegagroup())
-		? lng_profile_sure_kick
-		: lng_profile_sure_kick_channel)(lt_user, user->firstName);
+		? tr::lng_profile_sure_kick
+		: tr::lng_profile_sure_kick_channel)(
+			tr::now,
+			lt_user,
+			user->firstName);
 	_editBox = Ui::show(
 		Box<ConfirmBox>(
 			text,
-			lang(lng_box_remove),
+			tr::lng_box_remove(tr::now),
 			crl::guard(this, [=] { kickMemberSure(user); })),
 		LayerOption::KeepOther);
 }
@@ -1605,10 +1601,11 @@ void ParticipantsBoxController::kickMemberSure(not_null<UserData*> user) {
 void ParticipantsBoxController::removeAdmin(not_null<UserData*> user) {
 	_editBox = Ui::show(
 		Box<ConfirmBox>(
-			lng_profile_sure_remove_admin(
+			tr::lng_profile_sure_remove_admin(
+				tr::now,
 				lt_user,
 				user->firstName),
-			lang(lng_box_remove),
+			tr::lng_box_remove(tr::now),
 			crl::guard(this, [=] { removeAdminSure(user); })),
 		LayerOption::KeepOther);
 }
@@ -1651,7 +1648,7 @@ void ParticipantsBoxController::removeKicked(
 	delegate()->peerListRemoveRow(row);
 	if (_role != Role::Kicked
 		&& !delegate()->peerListFullRowsCount()) {
-		setDescriptionText(lang(lng_blocked_list_not_found));
+		setDescriptionText(tr::lng_blocked_list_not_found(tr::now));
 	}
 	delegate()->peerListRefreshRows();
 	removeKicked(user);
@@ -1690,14 +1687,14 @@ bool ParticipantsBoxController::removeRow(not_null<UserData*> user) {
 	if (auto row = delegate()->peerListFindRow(user->id)) {
 		if (_role == Role::Admins) {
 			// Perhaps we are removing an admin from search results.
-			row->setCustomStatus(lang(lng_channel_admin_status_not_admin));
+			row->setCustomStatus(tr::lng_channel_admin_status_not_admin(tr::now));
 			delegate()->peerListConvertRowToSearchResult(row);
 		} else {
 			delegate()->peerListRemoveRow(row);
 		}
 		if (_role != Role::Kicked
 			&& !delegate()->peerListFullRowsCount()) {
-			setDescriptionText(lang(lng_blocked_list_not_found));
+			setDescriptionText(tr::lng_blocked_list_not_found(tr::now));
 		}
 		return true;
 	}
@@ -1716,17 +1713,17 @@ std::unique_ptr<PeerListRow> ParticipantsBoxController::createRow(
 	if (_role == Role::Admins
 		&& _additional.adminRights(user).has_value()
 		&& _additional.canEditAdmin(user)) {
-		row->setActionLink(lang(lng_profile_kick));
+		row->setActionLink(tr::lng_profile_kick(tr::now));
 	} else if (_role == Role::Kicked || _role == Role::Restricted) {
 		if (_additional.canRestrictUser(user)) {
-			row->setActionLink(lang(lng_profile_delete_removed));
+			row->setActionLink(tr::lng_profile_delete_removed(tr::now));
 		}
 	} else if (_role == Role::Members) {
 		if ((chat ? chat->canBanMembers() : channel->canBanMembers())
 			&& !_additional.isCreator(user)
 			&& (!_additional.adminRights(user)
 				|| _additional.canEditAdmin(user))) {
-			row->setActionLink(lang(lng_profile_kick));
+			row->setActionLink(tr::lng_profile_kick(tr::now));
 		}
 	}
 	return std::move(row);
@@ -1759,25 +1756,27 @@ void ParticipantsBoxController::refreshCustomStatus(
 	const auto user = row->peer()->asUser();
 	if (_role == Role::Admins) {
 		if (const auto by = _additional.adminPromotedBy(user)) {
-			row->setCustomStatus(lng_channel_admin_status_promoted_by(
+			row->setCustomStatus(tr::lng_channel_admin_status_promoted_by(
+				tr::now,
 				lt_user,
 				App::peerName(by)));
 		} else {
 			if (_additional.isCreator(user)) {
 				row->setCustomStatus(
-					lang(lng_channel_admin_status_creator));
+					tr::lng_channel_admin_status_creator(tr::now));
 			} else {
 				row->setCustomStatus(
-					lang(lng_channel_admin_status_not_admin));
+					tr::lng_channel_admin_status_not_admin(tr::now));
 			}
 		}
 	} else if (_role == Role::Kicked || _role == Role::Restricted) {
 		const auto by = _additional.restrictedBy(user);
 		row->setCustomStatus((_role == Role::Kicked
-			? lng_channel_banned_status_removed_by
-			: lng_channel_banned_status_restricted_by)(
-			lt_user,
-			by ? App::peerName(by) : "Unknown"));
+			? tr::lng_channel_banned_status_removed_by
+			: tr::lng_channel_banned_status_restricted_by)(
+				tr::now,
+				lt_user,
+				by ? App::peerName(by) : "Unknown"));
 	}
 }
 
@@ -1791,6 +1790,54 @@ void ParticipantsBoxController::subscribeToMigration() {
 void ParticipantsBoxController::migrate(not_null<ChannelData*> channel) {
 	_peer = channel;
 	_additional.migrate(channel);
+	subscribeToCreatorChange(channel);
+}
+
+void ParticipantsBoxController::subscribeToCreatorChange(
+		not_null<ChannelData*> channel) {
+	const auto isCreator = channel->amCreator();
+	channel->flagsValue(
+	) | rpl::filter([](const ChannelData::Flags::Change &change) {
+		return (change.diff & MTPDchannel::Flag::f_creator);
+	}) | rpl::filter([=] {
+		return (isCreator != channel->amCreator());
+	}) | rpl::start_with_next([=] {
+		if (channel->isBroadcast()) {
+			fullListRefresh();
+			return;
+		}
+		const auto weak = base::make_weak(this);
+		const auto api = &channel->session().api();
+		api->request(MTPchannels_GetParticipants(
+			channel->inputChannel,
+			MTP_channelParticipantsRecent(),
+			MTP_int(0),
+			MTP_int(Global::ChatSizeMax()),
+			MTP_int(0)
+		)).done([=](const MTPchannels_ChannelParticipants &result) {
+			channel->mgInfo->creator = channel->amCreator()
+				? channel->session().user().get()
+				: nullptr;
+			channel->mgInfo->lastAdmins.clear();
+			channel->mgInfo->lastRestricted.clear();
+			channel->mgInfo->lastParticipants.clear();
+			api->parseRecentChannelParticipants(channel, result);
+			if (weak) {
+				fullListRefresh();
+			}
+		}).send();
+	}, lifetime());
+}
+
+void ParticipantsBoxController::fullListRefresh() {
+	_additional = ParticipantsAdditionalData(_peer, _role);
+
+	while (const auto count = delegate()->peerListFullRowsCount()) {
+		delegate()->peerListRemoveRow(
+			delegate()->peerListRowAt(count - 1));
+	}
+	loadMoreRows();
+	delegate()->peerListRefreshRows();
 }
 
 ParticipantsBoxSearchController::ParticipantsBoxSearchController(
@@ -1949,7 +1996,7 @@ void ParticipantsBoxSearchController::searchDone(
 
 	_requestId = 0;
 	result.match([&](const MTPDchannels_channelParticipants &data) {
-		const auto &list = data.vparticipants.v;
+		const auto &list = data.vparticipants().v;
 		if (list.size() < requestedCount) {
 			// We want cache to have full information about a query with
 			// small results count (that we don't need the second request).
