@@ -8,6 +8,24 @@ https://github.com/telegramdesktop/tdesktop/blob/master/LEGAL
 #pragma once
 
 #include "mtproto/sender.h"
+#include "ui/image/image_source.h"
+
+class DocumentData;
+class AuthSession;
+
+namespace Storage {
+namespace Cache {
+struct Key;
+} // namespace Cache
+} // namespace Storage
+
+namespace Lottie {
+class SinglePlayer;
+class MultiPlayer;
+class FrameRenderer;
+class Animation;
+enum class Quality : char;
+} // namespace Lottie
 
 namespace Stickers {
 
@@ -103,4 +121,57 @@ QString GetSetTitle(const MTPDstickerSet &s);
 
 RecentStickerPack &GetRecentPack();
 
+enum class LottieSize : uchar {
+	MessageHistory,
+	StickerSet,
+	StickersPanel,
+	StickersFooter,
+	SetsListThumbnail,
+	InlineResults,
+};
+
+[[nodiscard]] std::unique_ptr<Lottie::SinglePlayer> LottiePlayerFromDocument(
+	not_null<DocumentData*> document,
+	LottieSize sizeTag,
+	QSize box,
+	Lottie::Quality quality = Lottie::Quality(),
+	std::shared_ptr<Lottie::FrameRenderer> renderer = nullptr);
+[[nodiscard]] not_null<Lottie::Animation*> LottieAnimationFromDocument(
+	not_null<Lottie::MultiPlayer*> player,
+	not_null<DocumentData*> document,
+	LottieSize sizeTag,
+	QSize box);
+
+[[nodiscard]] bool HasLottieThumbnail(
+	ImagePtr thumbnail,
+	not_null<DocumentData*> sticker);
+[[nodiscard]] std::unique_ptr<Lottie::SinglePlayer> LottieThumbnail(
+	ImagePtr thumbnail,
+	not_null<DocumentData*> sticker,
+	LottieSize sizeTag,
+	QSize box,
+	std::shared_ptr<Lottie::FrameRenderer> renderer = nullptr);
+
+class ThumbnailSource : public Images::StorageSource {
+public:
+	ThumbnailSource(
+		const StorageImageLocation &location,
+		int size);
+
+	QImage takeLoaded() override;
+
+	QByteArray bytesForCache() override;
+
+protected:
+	std::unique_ptr<FileLoader> createLoader(
+		Data::FileOrigin origin,
+		LoadFromCloudSetting fromCloud,
+		bool autoLoading) override;
+
+private:
+	QPointer<FileLoader> _loader;
+	QByteArray _bytesForAnimated;
+
+};
+
 } // namespace Stickers
