@@ -20,7 +20,7 @@ https://github.com/telegramdesktop/tdesktop/blob/master/LEGAL
 #include "boxes/connection_box.h"
 #include "boxes/sticker_set_box.h"
 #include "passport/passport_form_controller.h"
-#include "window/window_controller.h"
+#include "window/window_session_controller.h"
 #include "data/data_session.h"
 #include "data/data_channel.h"
 #include "mainwindow.h"
@@ -45,8 +45,8 @@ bool JoinGroupByHash(const Match &match, const QVariant &context) {
 				Auth().api().importChatInvite(hash);
 			}));
 		}, [=](const MTPDchatInviteAlready &data) {
-			if (const auto chat = Auth().data().processChat(data.vchat)) {
-				App::wnd()->controller()->showPeerHistory(
+			if (const auto chat = Auth().data().processChat(data.vchat())) {
+				App::wnd()->sessionController()->showPeerHistory(
 					chat,
 					Window::SectionShow::Way::Forward);
 			}
@@ -56,7 +56,7 @@ bool JoinGroupByHash(const Match &match, const QVariant &context) {
 			return;
 		}
 		Core::App().hideMediaView();
-		Ui::show(Box<InformBox>(lang(lng_group_invite_bad_link)));
+		Ui::show(Box<InformBox>(tr::lng_group_invite_bad_link(tr::now)));
 	});
 	return true;
 }
@@ -67,6 +67,7 @@ bool ShowStickerSet(const Match &match, const QVariant &context) {
 	}
 	Core::App().hideMediaView();
 	Ui::show(Box<StickerSetBox>(
+		App::wnd()->sessionController(),
 		MTP_inputStickerSetShortName(MTP_string(match->captured(1)))));
 	return true;
 }
@@ -145,7 +146,7 @@ bool ShowPassportForm(const QMap<QString, QString> &params) {
 		QString());
 	const auto errors = params.value("errors", QString());
 	if (const auto window = App::wnd()) {
-		if (const auto controller = window->controller()) {
+		if (const auto controller = window->sessionController()) {
 			controller->showPassportForm(Passport::FormRequest(
 				botId,
 				scope,
@@ -240,13 +241,13 @@ bool ResolvePrivatePost(const Match &match, const QVariant &context) {
 		return false;
 	}
 	const auto done = [=](not_null<PeerData*> peer) {
-		App::wnd()->controller()->showPeerHistory(
+		App::wnd()->sessionController()->showPeerHistory(
 			peer->id,
 			Window::SectionShow::Way::Forward,
 			msgId);
 	};
 	const auto fail = [=] {
-		Ui::show(Box<InformBox>(lang(lng_error_post_link_invalid)));
+		Ui::show(Box<InformBox>(tr::lng_error_post_link_invalid(tr::now)));
 	};
 	const auto auth = &Auth();
 	if (const auto channel = auth->data().channelLoaded(channelId)) {
@@ -259,7 +260,7 @@ bool ResolvePrivatePost(const Match &match, const QVariant &context) {
 			MTP_inputChannel(MTP_int(channelId), MTP_long(0)))
 	)).done([=](const MTPmessages_Chats &result) {
 		result.match([&](const auto &data) {
-			const auto peer = auth->data().processChats(data.vchats);
+			const auto peer = auth->data().processChats(data.vchats());
 			if (peer && peer->id == peerFromChannel(channelId)) {
 				done(peer);
 			} else {
@@ -279,10 +280,9 @@ bool HandleUnknown(const Match &match, const QVariant &context) {
 	const auto request = match->captured(1);
 	const auto callback = [=](const MTPDhelp_deepLinkInfo &result) {
 		const auto text = TextWithEntities{
-			qs(result.vmessage),
-			(result.has_entities()
-				? TextUtilities::EntitiesFromMTP(result.ventities.v)
-				: EntitiesInText())
+			qs(result.vmessage()),
+			TextUtilities::EntitiesFromMTP(
+				result.ventities().value_or_empty())
 		};
 		if (result.is_update_app()) {
 			const auto box = std::make_shared<QPointer<BoxContent>>();
@@ -292,7 +292,7 @@ bool HandleUnknown(const Match &match, const QVariant &context) {
 			};
 			*box = Ui::show(Box<ConfirmBox>(
 				text,
-				lang(lng_menu_update),
+				tr::lng_menu_update(tr::now),
 				callback));
 		} else {
 			Ui::show(Box<InformBox>(text));
