@@ -7,6 +7,7 @@ https://github.com/telegramdesktop/tdesktop/blob/master/LEGAL
 */
 #pragma once
 
+#include "base/flags.h"
 #include "data/data_types.h"
 #include "ui/image/image.h"
 
@@ -52,6 +53,7 @@ struct StickerData : public DocumentAdditionalData {
 	Data::FileOrigin setOrigin() const;
 
 	std::unique_ptr<Image> image;
+	bool animated = false;
 	QString alt;
 	MTPInputStickerSet set = MTP_inputStickerSetEmpty();
 	StorageImageLocation loc; // doc thumb location
@@ -111,8 +113,10 @@ public:
 	void cancel();
 	[[nodiscard]] bool cancelled() const;
 	[[nodiscard]] float64 progress() const;
-	[[nodiscard]] int32 loadOffset() const;
+	[[nodiscard]] int loadOffset() const;
 	[[nodiscard]] bool uploading() const;
+	[[nodiscard]] bool loadedInMediaCache() const;
+	void setLoadedInMediaCache(bool loaded);
 
 	void setWaitingForAlbum();
 	[[nodiscard]] bool waitingForAlbum() const;
@@ -177,6 +181,9 @@ public:
 	void refreshGoodThumbnail();
 	void replaceGoodThumbnail(std::unique_ptr<Images::Source> &&source);
 
+	[[nodiscard]] auto bigFileBaseCacheKey() const
+	-> std::optional<Storage::Cache::Key>;
+
 	void setRemoteLocation(
 		int32 dc,
 		uint64 access,
@@ -214,8 +221,10 @@ public:
 
 	[[nodiscard]] bool canBePlayed() const;
 	[[nodiscard]] bool canBeStreamed() const;
-	[[nodiscard]] auto createStreamingLoader(Data::FileOrigin origin) const
-		-> std::unique_ptr<Media::Streaming::Loader>;
+	[[nodiscard]] auto createStreamingLoader(
+		Data::FileOrigin origin,
+		bool forceRemoteLoader) const
+	-> std::unique_ptr<Media::Streaming::Loader>;
 
 	void setInappPlaybackFailed();
 	[[nodiscard]] bool inappPlaybackFailed() const;
@@ -233,20 +242,40 @@ public:
 	std::unique_ptr<Data::UploadState> uploadingData;
 
 private:
-	enum class SupportsStreaming : uchar {
-		Unknown,
-		MaybeYes,
-		MaybeNo,
-		No,
+	enum class Flag : uchar {
+		StreamingMaybeYes = 0x01,
+		StreamingMaybeNo = 0x02,
+		StreamingPlaybackFailed = 0x04,
+		ImageType = 0x08,
+		DownloadCancelled = 0x10,
+		LoadedInMediaCache = 0x20,
 	};
+	using Flags = base::flags<Flag>;
+	friend constexpr bool is_flag_type(Flag) { return true; };
+
+	static constexpr Flags kStreamingSupportedMask = Flags()
+		| Flag::StreamingMaybeYes
+		| Flag::StreamingMaybeNo;
+	static constexpr Flags kStreamingSupportedUnknown = Flags()
+		| Flag::StreamingMaybeYes
+		| Flag::StreamingMaybeNo;
+	static constexpr Flags kStreamingSupportedMaybeYes = Flags()
+		| Flag::StreamingMaybeYes;
+	static constexpr Flags kStreamingSupportedMaybeNo = Flags()
+		| Flag::StreamingMaybeNo;
+	static constexpr Flags kStreamingSupportedNo = Flags();
+
 	friend class Serialize::Document;
 
 	LocationType locationType() const;
+	void validateLottieSticker();
 	void validateGoodThumbnail();
 	void setMaybeSupportsStreaming(bool supports);
+	void setLoadedInMediaCacheLocation();
 
-	void destroyLoader(mtpFileLoader *newValue = nullptr) const;
+	void destroyLoader() const;
 
+	[[nodiscard]] bool useStreamingLoader() const;
 	[[nodiscard]] bool thumbnailEnoughForSticker() const;
 
 	// Two types of location: from MTProto by dc+access or from web by url
@@ -269,11 +298,8 @@ private:
 	QByteArray _data;
 	std::unique_ptr<DocumentAdditionalData> _additional;
 	int32 _duration = -1;
-	bool _isImage = false;
-	SupportsStreaming _supportsStreaming = SupportsStreaming::Unknown;
-	bool _inappPlaybackFailed = false;
-
-	mutable FileLoader *_loader = nullptr;
+	mutable Flags _flags = kStreamingSupportedUnknown;
+	mutable std::unique_ptr<FileLoader> _loader;
 
 };
 
