@@ -46,8 +46,8 @@ namespace {
 Call ComputeCallData(const MTPDmessageActionPhoneCall &call) {
 	auto result = Call();
 	result.finishReason = [&] {
-		if (call.has_reason()) {
-			switch (call.vreason.type()) {
+		if (const auto reason = call.vreason()) {
+			switch (reason->type()) {
 			case mtpc_phoneCallDiscardReasonBusy:
 				return CallFinishReason::Busy;
 			case mtpc_phoneCallDiscardReasonDisconnect:
@@ -61,7 +61,7 @@ Call ComputeCallData(const MTPDmessageActionPhoneCall &call) {
 		}
 		return CallFinishReason::Hangup;
 	}();
-	result.duration = call.has_duration() ? call.vduration.v : 0;;
+	result.duration = call.vduration().value_or_empty();
 	return result;
 }
 
@@ -70,15 +70,13 @@ Invoice ComputeInvoiceData(
 		const MTPDmessageMediaInvoice &data) {
 	auto result = Invoice();
 	result.isTest = data.is_test();
-	result.amount = data.vtotal_amount.v;
-	result.currency = qs(data.vcurrency);
-	result.description = qs(data.vdescription);
-	result.title = TextUtilities::SingleLine(qs(data.vtitle));
-	if (data.has_receipt_msg_id()) {
-		result.receiptMsgId = data.vreceipt_msg_id.v;
-	}
-	if (data.has_photo()) {
-		result.photo = item->history()->owner().photoFromWeb(data.vphoto);
+	result.amount = data.vtotal_amount().v;
+	result.currency = qs(data.vcurrency());
+	result.description = qs(data.vdescription());
+	result.title = TextUtilities::SingleLine(qs(data.vtitle()));
+	result.receiptMsgId = data.vreceipt_msg_id().value_or_empty();
+	if (const auto photo = data.vphoto()) {
+		result.photo = item->history()->owner().photoFromWeb(*photo);
 	}
 	return result;
 }
@@ -90,15 +88,15 @@ QString WithCaptionDialogsText(
 		return textcmdLink(1, TextUtilities::Clean(attachType));
 	}
 
-	auto captionText = TextUtilities::Clean(caption);
-	auto attachTypeWrapped = textcmdLink(1, lng_dialogs_text_media_wrapped(
-		lt_media,
-		TextUtilities::Clean(attachType)));
-	return lng_dialogs_text_media(
+	return tr::lng_dialogs_text_media(
+		tr::now,
 		lt_media_part,
-		attachTypeWrapped,
+		textcmdLink(1, tr::lng_dialogs_text_media_wrapped(
+			tr::now,
+			lt_media,
+			TextUtilities::Clean(attachType))),
 		lt_caption,
-		captionText);
+		TextUtilities::Clean(caption));
 }
 
 QString WithCaptionNotificationText(
@@ -108,12 +106,13 @@ QString WithCaptionNotificationText(
 		return attachType;
 	}
 
-	auto attachTypeWrapped = lng_dialogs_text_media_wrapped(
-		lt_media,
-		attachType);
-	return lng_dialogs_text_media(
+	return tr::lng_dialogs_text_media(
+		tr::now,
 		lt_media_part,
-		attachTypeWrapped,
+		tr::lng_dialogs_text_media_wrapped(
+			tr::now,
+			lt_media,
+			attachType),
 		lt_caption,
 		caption);
 }
@@ -167,7 +166,7 @@ const Invoice *Media::invoice() const {
 	return nullptr;
 }
 
-LocationData *Media::location() const {
+LocationThumbnail *Media::location() const {
 	return nullptr;
 }
 
@@ -306,23 +305,23 @@ Image *MediaPhoto::replyPreview() const {
 
 QString MediaPhoto::notificationText() const {
 	return WithCaptionNotificationText(
-		lang(lng_in_dlg_photo),
+		tr::lng_in_dlg_photo(tr::now),
 		parent()->originalText().text);
 }
 
 QString MediaPhoto::chatListText() const {
 	return WithCaptionDialogsText(
-		lang(lng_in_dlg_photo),
+		tr::lng_in_dlg_photo(tr::now),
 		parent()->originalText().text);
 }
 
 QString MediaPhoto::pinnedTextSubstring() const {
-	return lang(lng_action_pinned_media_photo);
+	return tr::lng_action_pinned_media_photo(tr::now);
 }
 
 TextForMimeData MediaPhoto::clipboardText() const {
 	return WithCaptionClipboardText(
-		lang(lng_in_dlg_photo),
+		tr::lng_in_dlg_photo(tr::now),
 		parent()->clipboardText());
 }
 
@@ -335,20 +334,21 @@ bool MediaPhoto::allowsEditMedia() const {
 }
 
 QString MediaPhoto::errorTextForForward(not_null<PeerData*> peer) const {
-	const auto errorKey = Data::RestrictionErrorKey(
+	return Data::RestrictionError(
 		peer,
-		ChatRestriction::f_send_media);
-	return errorKey ? lang(*errorKey) : QString();
+		ChatRestriction::f_send_media
+	).value_or(QString());
 }
 
 bool MediaPhoto::updateInlineResultMedia(const MTPMessageMedia &media) {
 	if (media.type() != mtpc_messageMediaPhoto) {
 		return false;
 	}
-	auto &data = media.c_messageMediaPhoto();
-	if (data.has_photo() && !data.has_ttl_seconds()) {
+	const auto &data = media.c_messageMediaPhoto();
+	const auto content = data.vphoto();
+	if (content && !data.vttl_seconds()) {
 		const auto photo = parent()->history()->owner().processPhoto(
-			data.vphoto);
+			*content);
 		if (photo == _photo) {
 			return true;
 		} else {
@@ -366,22 +366,23 @@ bool MediaPhoto::updateSentMedia(const MTPMessageMedia &media) {
 	if (media.type() != mtpc_messageMediaPhoto) {
 		return false;
 	}
-	auto &mediaPhoto = media.c_messageMediaPhoto();
-	if (!mediaPhoto.has_photo() || mediaPhoto.has_ttl_seconds()) {
+	const auto &mediaPhoto = media.c_messageMediaPhoto();
+	const auto content = mediaPhoto.vphoto();
+	if (!content || mediaPhoto.vttl_seconds()) {
 		LOG(("Api Error: "
 			"Got MTPMessageMediaPhoto without photo "
 			"or with ttl_seconds in updateSentMedia()"));
 		return false;
 	}
-	parent()->history()->owner().photoConvert(_photo, mediaPhoto.vphoto);
+	parent()->history()->owner().photoConvert(_photo, *content);
 
-	if (mediaPhoto.vphoto.type() != mtpc_photo) {
+	if (content->type() != mtpc_photo) {
 		return false;
 	}
-	const auto &photo = mediaPhoto.vphoto.c_photo();
+	const auto &photo = content->c_photo();
 
 	struct SizeData {
-		MTPstring type = MTP_string(QString());
+		MTPstring type = MTP_string();
 		int width = 0;
 		int height = 0;
 		QByteArray bytes;
@@ -393,12 +394,12 @@ bool MediaPhoto::updateSentMedia(const MTPMessageMedia &media) {
 
 		const auto key = StorageImageLocation(
 			StorageFileLocation(
-				photo.vdc_id.v,
+				photo.vdc_id().v,
 				_photo->session().userId(),
 				MTP_inputPhotoFileLocation(
-					photo.vid,
-					photo.vaccess_hash,
-					photo.vfile_reference,
+					photo.vid(),
+					photo.vaccess_hash(),
+					photo.vfile_reference(),
 					size.type)),
 			size.width,
 			size.height);
@@ -421,23 +422,23 @@ bool MediaPhoto::updateSentMedia(const MTPMessageMedia &media) {
 		image->replaceSource(
 			std::make_unique<Images::StorageSource>(key, length));
 	};
-	auto &sizes = photo.vsizes.v;
+	auto &sizes = photo.vsizes().v;
 	auto max = 0;
 	auto maxSize = SizeData();
 	for (const auto &data : sizes) {
 		const auto size = data.match([](const MTPDphotoSize &data) {
 			return SizeData{
-				data.vtype,
-				data.vw.v,
-				data.vh.v,
+				data.vtype(),
+				data.vw().v,
+				data.vh().v,
 				QByteArray()
 			};
 		}, [](const MTPDphotoCachedSize &data) {
 			return SizeData{
-				data.vtype,
-				data.vw.v,
-				data.vh.v,
-				qba(data.vbytes)
+				data.vtype(),
+				data.vw().v,
+				data.vh().v,
+				qba(data.vbytes())
 			};
 		}, [](const MTPDphotoSizeEmpty &) {
 			return SizeData();
@@ -564,20 +565,20 @@ QString MediaFile::chatListText() const {
 	}
 	const auto type = [&] {
 		if (_document->isVideoMessage()) {
-			return lang(lng_in_dlg_video_message);
+			return tr::lng_in_dlg_video_message(tr::now);
 		} else if (_document->isAnimation()) {
 			return qsl("GIF");
 		} else if (_document->isVideoFile()) {
-			return lang(lng_in_dlg_video);
+			return tr::lng_in_dlg_video(tr::now);
 		} else if (_document->isVoiceMessage()) {
-			return lang(lng_in_dlg_audio);
+			return tr::lng_in_dlg_audio(tr::now);
 		} else if (const auto name = _document->composeNameString();
 				!name.isEmpty()) {
 			return name;
 		} else if (_document->isAudioFile()) {
-			return lang(lng_in_dlg_audio_file);
+			return tr::lng_in_dlg_audio_file(tr::now);
 		}
-		return lang(lng_in_dlg_file);
+		return tr::lng_in_dlg_file(tr::now);
 	}();
 	return WithCaptionDialogsText(type, parent()->originalText().text);
 }
@@ -585,24 +586,24 @@ QString MediaFile::chatListText() const {
 QString MediaFile::notificationText() const {
 	if (const auto sticker = _document->sticker()) {
 		return _emoji.isEmpty()
-			? lang(lng_in_dlg_sticker)
-			: lng_in_dlg_sticker_emoji(lt_emoji, _emoji);
+			? tr::lng_in_dlg_sticker(tr::now)
+			: tr::lng_in_dlg_sticker_emoji(tr::now, lt_emoji, _emoji);
 	}
 	const auto type = [&] {
 		if (_document->isVideoMessage()) {
-			return lang(lng_in_dlg_video_message);
+			return tr::lng_in_dlg_video_message(tr::now);
 		} else if (_document->isAnimation()) {
 			return qsl("GIF");
 		} else if (_document->isVideoFile()) {
-			return lang(lng_in_dlg_video);
+			return tr::lng_in_dlg_video(tr::now);
 		} else if (_document->isVoiceMessage()) {
-			return lang(lng_in_dlg_audio);
+			return tr::lng_in_dlg_audio(tr::now);
 		} else if (!_document->filename().isEmpty()) {
 			return _document->filename();
 		} else if (_document->isAudioFile()) {
-			return lang(lng_in_dlg_audio_file);
+			return tr::lng_in_dlg_audio_file(tr::now);
 		}
-		return lang(lng_in_dlg_file);
+		return tr::lng_in_dlg_file(tr::now);
 	}();
 	return WithCaptionNotificationText(type, parent()->originalText().text);
 }
@@ -610,22 +611,22 @@ QString MediaFile::notificationText() const {
 QString MediaFile::pinnedTextSubstring() const {
 	if (const auto sticker = _document->sticker()) {
 		if (!_emoji.isEmpty()) {
-			return lng_action_pinned_media_emoji_sticker(lt_emoji, _emoji);
+			return tr::lng_action_pinned_media_emoji_sticker(tr::now, lt_emoji, _emoji);
 		}
-		return lang(lng_action_pinned_media_sticker);
+		return tr::lng_action_pinned_media_sticker(tr::now);
 	} else if (_document->isAnimation()) {
 		if (_document->isVideoMessage()) {
-			return lang(lng_action_pinned_media_video_message);
+			return tr::lng_action_pinned_media_video_message(tr::now);
 		}
-		return lang(lng_action_pinned_media_gif);
+		return tr::lng_action_pinned_media_gif(tr::now);
 	} else if (_document->isVideoFile()) {
-		return lang(lng_action_pinned_media_video);
+		return tr::lng_action_pinned_media_video(tr::now);
 	} else if (_document->isVoiceMessage()) {
-		return lang(lng_action_pinned_media_voice);
+		return tr::lng_action_pinned_media_voice(tr::now);
 	} else if (_document->isSong()) {
-		return lang(lng_action_pinned_media_audio);
+		return tr::lng_action_pinned_media_audio(tr::now);
 	}
-	return lang(lng_action_pinned_media_file);
+	return tr::lng_action_pinned_media_file(tr::now);
 }
 
 TextForMimeData MediaFile::clipboardText() const {
@@ -636,22 +637,22 @@ TextForMimeData MediaFile::clipboardText() const {
 			: QString();
 		if (const auto sticker = _document->sticker()) {
 			if (!_emoji.isEmpty()) {
-				return lng_in_dlg_sticker_emoji(lt_emoji, _emoji);
+				return tr::lng_in_dlg_sticker_emoji(tr::now, lt_emoji, _emoji);
 			}
-			return lang(lng_in_dlg_sticker);
+			return tr::lng_in_dlg_sticker(tr::now);
 		} else if (_document->isAnimation()) {
 			if (_document->isVideoMessage()) {
-				return lang(lng_in_dlg_video_message);
+				return tr::lng_in_dlg_video_message(tr::now);
 			}
 			return qsl("GIF");
 		} else if (_document->isVideoFile()) {
-			return lang(lng_in_dlg_video);
+			return tr::lng_in_dlg_video(tr::now);
 		} else if (_document->isVoiceMessage()) {
-			return lang(lng_in_dlg_audio) + addName;
+			return tr::lng_in_dlg_audio(tr::now) + addName;
 		} else if (_document->isSong()) {
-			return lang(lng_in_dlg_audio_file) + addName;
+			return tr::lng_in_dlg_audio_file(tr::now) + addName;
 		}
-		return lang(lng_in_dlg_file) + addName;
+		return tr::lng_in_dlg_file(tr::now) + addName;
 	}();
 	return WithCaptionClipboardText(
 		attachType,
@@ -676,29 +677,29 @@ bool MediaFile::forwardedBecomesUnread() const {
 
 QString MediaFile::errorTextForForward(not_null<PeerData*> peer) const {
 	if (const auto sticker = _document->sticker()) {
-		if (const auto key = Data::RestrictionErrorKey(
+		if (const auto error = Data::RestrictionError(
 				peer,
 				ChatRestriction::f_send_stickers)) {
-			return lang(*key);
+			return *error;
 		}
 	} else if (_document->isAnimation()) {
 		if (_document->isVideoMessage()) {
-			if (const auto key = Data::RestrictionErrorKey(
+			if (const auto error = Data::RestrictionError(
 					peer,
 					ChatRestriction::f_send_media)) {
-				return lang(*key);
+				return *error;
 			}
 		} else {
-			if (const auto key = Data::RestrictionErrorKey(
+			if (const auto error = Data::RestrictionError(
 					peer,
 					ChatRestriction::f_send_gifs)) {
-				return lang(*key);
+				return *error;
 			}
 		}
-	} else if (const auto key = Data::RestrictionErrorKey(
+	} else if (const auto error = Data::RestrictionError(
 			peer,
 			ChatRestriction::f_send_media)) {
-		return lang(*key);
+		return *error;
 	}
 	return QString();
 }
@@ -707,10 +708,11 @@ bool MediaFile::updateInlineResultMedia(const MTPMessageMedia &media) {
 	if (media.type() != mtpc_messageMediaDocument) {
 		return false;
 	}
-	auto &data = media.c_messageMediaDocument();
-	if (data.has_document() && !data.has_ttl_seconds()) {
+	const auto &data = media.c_messageMediaDocument();
+	const auto content = data.vdocument();
+	if (content && !data.vttl_seconds()) {
 		const auto document = parent()->history()->owner().processDocument(
-			data.vdocument);
+			*content);
 		if (document == _document) {
 			return false;
 		} else {
@@ -728,14 +730,15 @@ bool MediaFile::updateSentMedia(const MTPMessageMedia &media) {
 	if (media.type() != mtpc_messageMediaDocument) {
 		return false;
 	}
-	auto &data = media.c_messageMediaDocument();
-	if (!data.has_document() || data.has_ttl_seconds()) {
+	const auto &data = media.c_messageMediaDocument();
+	const auto content = data.vdocument();
+	if (!content || data.vttl_seconds()) {
 		LOG(("Api Error: "
 			"Got MTPMessageMediaDocument without document "
 			"or with ttl_seconds in updateSentMedia()"));
 		return false;
 	}
-	parent()->history()->owner().documentConvert(_document, data.vdocument);
+	parent()->history()->owner().documentConvert(_document, *content);
 
 	if (const auto good = _document->goodThumbnail()) {
 		auto bytes = good->bytesForCache();
@@ -807,16 +810,17 @@ const SharedContact *MediaContact::sharedContact() const {
 }
 
 QString MediaContact::notificationText() const {
-	return lang(lng_in_dlg_contact);
+	return tr::lng_in_dlg_contact(tr::now);
 }
 
 QString MediaContact::pinnedTextSubstring() const {
-	return lang(lng_action_pinned_media_contact);
+	return tr::lng_action_pinned_media_contact(tr::now);
 }
 
 TextForMimeData MediaContact::clipboardText() const {
-	const auto text = qsl("[ ") + lang(lng_in_dlg_contact) + qsl(" ]\n")
-		+ lng_full_name(
+	const auto text = qsl("[ ") + tr::lng_in_dlg_contact(tr::now) + qsl(" ]\n")
+		+ tr::lng_full_name(
+			tr::now,
 			lt_first_name,
 			_contact.firstName,
 			lt_last_name,
@@ -834,11 +838,11 @@ bool MediaContact::updateSentMedia(const MTPMessageMedia &media) {
 	if (media.type() != mtpc_messageMediaContact) {
 		return false;
 	}
-	if (_contact.userId != media.c_messageMediaContact().vuser_id.v) {
+	if (_contact.userId != media.c_messageMediaContact().vuser_id().v) {
 		parent()->history()->owner().unregisterContactItem(
 			_contact.userId,
 			parent());
-		_contact.userId = media.c_messageMediaContact().vuser_id.v;
+		_contact.userId = media.c_messageMediaContact().vuser_id().v;
 		parent()->history()->owner().registerContactItem(
 			_contact.userId,
 			parent());
@@ -859,17 +863,17 @@ std::unique_ptr<HistoryMedia> MediaContact::createView(
 
 MediaLocation::MediaLocation(
 	not_null<HistoryItem*> parent,
-	const LocationCoords &coords)
-: MediaLocation(parent, coords, QString(), QString()) {
+	const LocationPoint &point)
+: MediaLocation(parent, point, QString(), QString()) {
 }
 
 MediaLocation::MediaLocation(
 	not_null<HistoryItem*> parent,
-	const LocationCoords &coords,
+	const LocationPoint &point,
 	const QString &title,
 	const QString &description)
 : Media(parent)
-, _location(parent->history()->owner().location(coords))
+, _location(parent->history()->owner().location(point))
 , _title(title)
 , _description(description) {
 }
@@ -877,30 +881,30 @@ MediaLocation::MediaLocation(
 std::unique_ptr<Media> MediaLocation::clone(not_null<HistoryItem*> parent) {
 	return std::make_unique<MediaLocation>(
 		parent,
-		_location->coords,
+		_location->point,
 		_title,
 		_description);
 }
 
-LocationData *MediaLocation::location() const {
+LocationThumbnail *MediaLocation::location() const {
 	return _location;
 }
 
 QString MediaLocation::chatListText() const {
-	return WithCaptionDialogsText(lang(lng_maps_point), _title);
+	return WithCaptionDialogsText(tr::lng_maps_point(tr::now), _title);
 }
 
 QString MediaLocation::notificationText() const {
-	return WithCaptionNotificationText(lang(lng_maps_point), _title);
+	return WithCaptionNotificationText(tr::lng_maps_point(tr::now), _title);
 }
 
 QString MediaLocation::pinnedTextSubstring() const {
-	return lang(lng_action_pinned_media_location);
+	return tr::lng_action_pinned_media_location(tr::now);
 }
 
 TextForMimeData MediaLocation::clipboardText() const {
 	auto result = TextForMimeData::Simple(
-		qstr("[ ") + lang(lng_maps_point) + qstr(" ]\n"));
+		qstr("[ ") + tr::lng_maps_point(tr::now) + qstr(" ]\n"));
 	auto titleResult = TextUtilities::ParseEntities(
 		TextUtilities::Clean(_title),
 		Ui::WebpageTextTitleOptions().flags);
@@ -913,7 +917,7 @@ TextForMimeData MediaLocation::clipboardText() const {
 	if (!descriptionResult.text.isEmpty()) {
 		result.append(std::move(descriptionResult));
 	}
-	result.append(LocationClickHandler(_location->coords).dragText());
+	result.append(LocationClickHandler(_location->point).dragText());
 	return result;
 }
 
@@ -953,7 +957,8 @@ const Call *MediaCall::call() const {
 QString MediaCall::notificationText() const {
 	auto result = Text(parent(), _call.finishReason);
 	if (_call.duration > 0) {
-		result = lng_call_type_and_duration(
+		result = tr::lng_call_type_and_duration(
+			tr::now,
 			lt_type,
 			result,
 			lt_duration,
@@ -993,15 +998,15 @@ QString MediaCall::Text(
 		not_null<HistoryItem*> item,
 		CallFinishReason reason) {
 	if (item->out()) {
-		return lang(reason == CallFinishReason::Missed
-			? lng_call_cancelled
-			: lng_call_outgoing);
+		return (reason == CallFinishReason::Missed)
+			? tr::lng_call_cancelled(tr::now)
+			: tr::lng_call_outgoing(tr::now);
 	} else if (reason == CallFinishReason::Missed) {
-		return lang(lng_call_missed);
+		return tr::lng_call_missed(tr::now);
 	} else if (reason == CallFinishReason::Busy) {
-		return lang(lng_call_declined);
+		return tr::lng_call_declined(tr::now);
 	}
-	return lang(lng_call_incoming);
+	return tr::lng_call_incoming(tr::now);
 }
 
 MediaWebPage::MediaWebPage(
@@ -1133,7 +1138,7 @@ GameData *MediaGame::game() const {
 
 QString MediaGame::pinnedTextSubstring() const {
 	const auto title = _game->title;
-	return lng_action_pinned_media_game(lt_game, title);
+	return tr::lng_action_pinned_media_game(tr::now, lt_game, title);
 }
 
 TextForMimeData MediaGame::clipboardText() const {
@@ -1141,10 +1146,10 @@ TextForMimeData MediaGame::clipboardText() const {
 }
 
 QString MediaGame::errorTextForForward(not_null<PeerData*> peer) const {
-	const auto errorKey = Data::RestrictionErrorKey(
+	return Data::RestrictionError(
 		peer,
-		ChatRestriction::f_send_games);
-	return errorKey ? lang(*errorKey) : QString();
+		ChatRestriction::f_send_games
+	).value_or(QString());
 }
 
 bool MediaGame::consumeMessageText(const TextWithEntities &text) {
@@ -1165,7 +1170,7 @@ bool MediaGame::updateSentMedia(const MTPMessageMedia &media) {
 		return false;
 	}
 	parent()->history()->owner().gameConvert(
-		_game, media.c_messageMediaGame().vgame);
+		_game, media.c_messageMediaGame().vgame());
 	return true;
 }
 
@@ -1265,7 +1270,7 @@ QString MediaPoll::pinnedTextSubstring() const {
 
 TextForMimeData MediaPoll::clipboardText() const {
 	const auto text = qstr("[ ")
-		+ lang(lng_in_dlg_poll)
+		+ tr::lng_in_dlg_poll(tr::now)
 		+ qstr(" : ")
 		+ _poll->question
 		+ qstr(" ]")
@@ -1280,10 +1285,10 @@ TextForMimeData MediaPoll::clipboardText() const {
 }
 
 QString MediaPoll::errorTextForForward(not_null<PeerData*> peer) const {
-	const auto errorKey = Data::RestrictionErrorKey(
+	return Data::RestrictionError(
 		peer,
-		ChatRestriction::f_send_polls);
-	return errorKey ? lang(*errorKey) : QString();
+		ChatRestriction::f_send_polls
+	).value_or(QString());
 }
 
 bool MediaPoll::updateInlineResultMedia(const MTPMessageMedia &media) {
