@@ -374,27 +374,27 @@ bool History::updateSendActionNeedsAnimating(
 		emplaceAction(
 			Type::UploadVideo,
 			kStatusShowClientsideUploadVideo,
-			data.vprogress.v);
+			data.vprogress().v);
 	}, [&](const MTPDsendMessageUploadAudioAction &data) {
 		emplaceAction(
 			Type::UploadVoice,
 			kStatusShowClientsideUploadVoice,
-			data.vprogress.v);
+			data.vprogress().v);
 	}, [&](const MTPDsendMessageUploadRoundAction &data) {
 		emplaceAction(
 			Type::UploadRound,
 			kStatusShowClientsideUploadRound,
-			data.vprogress.v);
+			data.vprogress().v);
 	}, [&](const MTPDsendMessageUploadPhotoAction &data) {
 		emplaceAction(
 			Type::UploadPhoto,
 			kStatusShowClientsideUploadPhoto,
-			data.vprogress.v);
+			data.vprogress().v);
 	}, [&](const MTPDsendMessageUploadDocumentAction &data) {
 		emplaceAction(
 			Type::UploadFile,
 			kStatusShowClientsideUploadFile,
-			data.vprogress.v);
+			data.vprogress().v);
 	}, [&](const MTPDsendMessageGamePlayAction &) {
 		const auto i = _sendActions.find(user);
 		if ((i == end(_sendActions))
@@ -479,17 +479,19 @@ bool History::updateSendActionNeedsAnimating(crl::time now, bool force) {
 		QString newTypingString;
 		auto typingCount = _typing.size();
 		if (typingCount > 2) {
-			newTypingString = lng_many_typing(lt_count, typingCount);
+			newTypingString = tr::lng_many_typing(tr::now, lt_count, typingCount);
 		} else if (typingCount > 1) {
-			newTypingString = lng_users_typing(
+			newTypingString = tr::lng_users_typing(
+				tr::now,
 				lt_user,
 				begin(_typing)->first->firstName,
 				lt_second_user,
 				(end(_typing) - 1)->first->firstName);
 		} else if (typingCount) {
 			newTypingString = peer->isUser()
-				? lang(lng_typing)
-				: lng_user_typing(
+				? tr::lng_typing(tr::now)
+				: tr::lng_user_typing(
+					tr::now,
 					lt_user,
 					begin(_typing)->first->firstName);
 		} else if (!_sendActions.empty()) {
@@ -497,16 +499,16 @@ bool History::updateSendActionNeedsAnimating(crl::time now, bool force) {
 			using Type = SendAction::Type;
 			auto sendActionString = [](Type type, const QString &name) -> QString {
 				switch (type) {
-				case Type::RecordVideo: return name.isEmpty() ? lang(lng_send_action_record_video) : lng_user_action_record_video(lt_user, name);
-				case Type::UploadVideo: return name.isEmpty() ? lang(lng_send_action_upload_video) : lng_user_action_upload_video(lt_user, name);
-				case Type::RecordVoice: return name.isEmpty() ? lang(lng_send_action_record_audio) : lng_user_action_record_audio(lt_user, name);
-				case Type::UploadVoice: return name.isEmpty() ? lang(lng_send_action_upload_audio) : lng_user_action_upload_audio(lt_user, name);
-				case Type::RecordRound: return name.isEmpty() ? lang(lng_send_action_record_round) : lng_user_action_record_round(lt_user, name);
-				case Type::UploadRound: return name.isEmpty() ? lang(lng_send_action_upload_round) : lng_user_action_upload_round(lt_user, name);
-				case Type::UploadPhoto: return name.isEmpty() ? lang(lng_send_action_upload_photo) : lng_user_action_upload_photo(lt_user, name);
-				case Type::UploadFile: return name.isEmpty() ? lang(lng_send_action_upload_file) : lng_user_action_upload_file(lt_user, name);
+				case Type::RecordVideo: return name.isEmpty() ? tr::lng_send_action_record_video(tr::now) : tr::lng_user_action_record_video(tr::now, lt_user, name);
+				case Type::UploadVideo: return name.isEmpty() ? tr::lng_send_action_upload_video(tr::now) : tr::lng_user_action_upload_video(tr::now, lt_user, name);
+				case Type::RecordVoice: return name.isEmpty() ? tr::lng_send_action_record_audio(tr::now) : tr::lng_user_action_record_audio(tr::now, lt_user, name);
+				case Type::UploadVoice: return name.isEmpty() ? tr::lng_send_action_upload_audio(tr::now) : tr::lng_user_action_upload_audio(tr::now, lt_user, name);
+				case Type::RecordRound: return name.isEmpty() ? tr::lng_send_action_record_round(tr::now) : tr::lng_user_action_record_round(tr::now, lt_user, name);
+				case Type::UploadRound: return name.isEmpty() ? tr::lng_send_action_upload_round(tr::now) : tr::lng_user_action_upload_round(tr::now, lt_user, name);
+				case Type::UploadPhoto: return name.isEmpty() ? tr::lng_send_action_upload_photo(tr::now) : tr::lng_user_action_upload_photo(tr::now, lt_user, name);
+				case Type::UploadFile: return name.isEmpty() ? tr::lng_send_action_upload_file(tr::now) : tr::lng_user_action_upload_file(tr::now, lt_user, name);
 				case Type::ChooseLocation:
-				case Type::ChooseContact: return name.isEmpty() ? lang(lng_typing) : lng_user_typing(lt_user, name);
+				case Type::ChooseContact: return name.isEmpty() ? tr::lng_typing(tr::now) : tr::lng_user_typing(tr::now, lt_user, name);
 				default: break;
 				};
 				return QString();
@@ -525,19 +527,22 @@ bool History::updateSendActionNeedsAnimating(crl::time now, bool force) {
 			if (newTypingString.isEmpty()) {
 				int playingCount = _sendActions.size();
 				if (playingCount > 2) {
-					newTypingString = lng_many_playing_game(
+					newTypingString = tr::lng_many_playing_game(
+						tr::now,
 						lt_count,
 						playingCount);
 				} else if (playingCount > 1) {
-					newTypingString = lng_users_playing_game(
+					newTypingString = tr::lng_users_playing_game(
+						tr::now,
 						lt_user,
 						begin(_sendActions)->first->firstName,
 						lt_second_user,
 						(end(_sendActions) - 1)->first->firstName);
 				} else {
 					newTypingString = peer->isUser()
-						? lang(lng_playing_game)
-						: lng_user_playing_game(
+						? tr::lng_playing_game(tr::now)
+						: tr::lng_user_playing_game(
+							tr::now,
 							lt_user,
 							begin(_sendActions)->first->firstName);
 				}
@@ -582,10 +587,7 @@ HistoryItem *History::createItem(
 			result->removeMainView();
 		}
 		if (message.type() == mtpc_message) {
-			const auto media = message.c_message().has_media()
-				? &message.c_message().vmedia
-				: nullptr;
-			result->updateSentMedia(media);
+			result->updateSentMedia(message.c_message().vmedia());
 		}
 		return result;
 	}
@@ -833,9 +835,9 @@ void History::addUnreadMentionsSlice(const MTPmessages_Messages &result) {
 	auto count = 0;
 	auto messages = (const QVector<MTPMessage>*)nullptr;
 	auto getMessages = [&](auto &list) {
-		owner().processUsers(list.vusers);
-		owner().processChats(list.vchats);
-		return &list.vmessages.v;
+		owner().processUsers(list.vusers());
+		owner().processChats(list.vchats());
+		return &list.vmessages().v;
 	};
 	switch (result.type()) {
 	case mtpc_messages_messages: {
@@ -847,14 +849,14 @@ void History::addUnreadMentionsSlice(const MTPmessages_Messages &result) {
 	case mtpc_messages_messagesSlice: {
 		auto &d = result.c_messages_messagesSlice();
 		messages = getMessages(d);
-		count = d.vcount.v;
+		count = d.vcount().v;
 	} break;
 
 	case mtpc_messages_channelMessages: {
 		LOG(("API Error: unexpected messages.channelMessages! (History::addUnreadMentionsSlice)"));
 		auto &d = result.c_messages_channelMessages();
 		messages = getMessages(d);
-		count = d.vcount.v;
+		count = d.vcount().v;
 	} break;
 
 	case mtpc_messages_messagesNotModified: {
@@ -1014,14 +1016,14 @@ void History::applyMessageChanges(
 void History::applyServiceChanges(
 		not_null<HistoryItem*> item,
 		const MTPDmessageService &data) {
-	auto &action = data.vaction;
+	auto &action = data.vaction();
 	switch (action.type()) {
 	case mtpc_messageActionChatAddUser: {
 		auto &d = action.c_messageActionChatAddUser();
 		if (const auto megagroup = peer->asMegagroup()) {
 			const auto mgInfo = megagroup->mgInfo.get();
 			Assert(mgInfo != nullptr);
-			for (const auto &userId : d.vusers.v) {
+			for (const auto &userId : d.vusers().v) {
 				if (const auto user = owner().userLoaded(userId.v)) {
 					if (!base::contains(mgInfo->lastParticipants, user)) {
 						mgInfo->lastParticipants.push_front(user);
@@ -1068,7 +1070,7 @@ void History::applyServiceChanges(
 
 	case mtpc_messageActionChatDeleteUser: {
 		auto &d = action.c_messageActionChatDeleteUser();
-		auto uid = d.vuser_id.v;
+		auto uid = d.vuser_id().v;
 		if (lastKeyboardFrom == peerFromUser(uid)) {
 			clearLastKeyboard();
 		}
@@ -1109,8 +1111,8 @@ void History::applyServiceChanges(
 
 	case mtpc_messageActionChatEditPhoto: {
 		auto &d = action.c_messageActionChatEditPhoto();
-		d.vphoto.match([&](const MTPDphoto &data) {
-			const auto &sizes = data.vsizes.v;
+		d.vphoto().match([&](const MTPDphoto &data) {
+			const auto &sizes = data.vsizes().v;
 			if (!sizes.isEmpty()) {
 				auto photo = owner().processPhoto(data);
 				photo->peer = peer;
@@ -1119,18 +1121,18 @@ void History::applyServiceChanges(
 				const MTPFileLocation *smallLoc = nullptr;
 				const MTPFileLocation *bigLoc = nullptr;
 				switch (smallSize.type()) {
-				case mtpc_photoSize: smallLoc = &smallSize.c_photoSize().vlocation; break;
-				case mtpc_photoCachedSize: smallLoc = &smallSize.c_photoCachedSize().vlocation; break;
+				case mtpc_photoSize: smallLoc = &smallSize.c_photoSize().vlocation(); break;
+				case mtpc_photoCachedSize: smallLoc = &smallSize.c_photoCachedSize().vlocation(); break;
 				}
 				switch (bigSize.type()) {
-				case mtpc_photoSize: bigLoc = &bigSize.c_photoSize().vlocation; break;
-				case mtpc_photoCachedSize: bigLoc = &bigSize.c_photoCachedSize().vlocation; break;
+				case mtpc_photoSize: bigLoc = &bigSize.c_photoSize().vlocation(); break;
+				case mtpc_photoCachedSize: bigLoc = &bigSize.c_photoCachedSize().vlocation(); break;
 				}
 				if (smallLoc && bigLoc) {
 					if (const auto chat = peer->asChat()) {
-						chat->setPhoto(photo->id, MTP_chatPhoto(*smallLoc, *bigLoc, data.vdc_id));
+						chat->setPhoto(photo->id, MTP_chatPhoto(*smallLoc, *bigLoc, data.vdc_id()));
 					} else if (const auto channel = peer->asChannel()) {
-						channel->setPhoto(photo->id, MTP_chatPhoto(*smallLoc, *bigLoc, data.vdc_id));
+						channel->setPhoto(photo->id, MTP_chatPhoto(*smallLoc, *bigLoc, data.vdc_id()));
 					}
 					peer->loadUserpic();
 				}
@@ -1147,7 +1149,7 @@ void History::applyServiceChanges(
 	case mtpc_messageActionChatEditTitle: {
 		auto &d = action.c_messageActionChatEditTitle();
 		if (auto chat = peer->asChat()) {
-			chat->setName(qs(d.vtitle));
+			chat->setName(qs(d.vtitle()));
 		}
 	} break;
 
@@ -1155,7 +1157,7 @@ void History::applyServiceChanges(
 		if (const auto chat = peer->asChat()) {
 			chat->addFlags(MTPDchat::Flag::f_deactivated);
 			const auto &d = action.c_messageActionChatMigrateTo();
-			if (const auto channel = owner().channelLoaded(d.vchannel_id.v)) {
+			if (const auto channel = owner().channelLoaded(d.vchannel_id().v)) {
 				Data::ApplyMigration(chat, channel);
 			}
 		}
@@ -1165,15 +1167,17 @@ void History::applyServiceChanges(
 		if (const auto channel = peer->asChannel()) {
 			channel->addFlags(MTPDchannel::Flag::f_megagroup);
 			const auto &d = action.c_messageActionChannelMigrateFrom();
-			if (const auto chat = owner().chatLoaded(d.vchat_id.v)) {
+			if (const auto chat = owner().chatLoaded(d.vchat_id().v)) {
 				Data::ApplyMigration(chat, channel);
 			}
 		}
 	} break;
 
 	case mtpc_messageActionPinMessage: {
-		if (data.has_reply_to_msg_id() && item) {
-			item->history()->peer->setPinnedMessageId(data.vreply_to_msg_id.v);
+		if (const auto replyToMsgId = data.vreply_to_msg_id()) {
+			if (item) {
+				item->history()->peer->setPinnedMessageId(replyToMsgId->v);
+			}
 		}
 	} break;
 
@@ -1843,7 +1847,7 @@ void History::setFolderPointer(Data::Folder *folder) {
 }
 
 void History::applyPinnedUpdate(const MTPDupdateDialogPinned &data) {
-	const auto folderId = data.has_folder_id() ? data.vfolder_id.v : 0;
+	const auto folderId = data.vfolder_id().value_or_empty();
 	if (!folderKnown()) {
 		if (folderId) {
 			setFolder(owner().folder(folderId));
@@ -2396,9 +2400,9 @@ void History::setFakeChatListMessageFrom(const MTPmessages_Messages &data) {
 			const MTPDmessages_messagesNotModified &) {
 		return static_cast<const MTPMessage*>(nullptr);
 	}, [&](const auto &data) {
-		for (const auto &message : data.vmessages.v) {
+		for (const auto &message : data.vmessages().v) {
 			const auto id = message.match([](const auto &data) {
-				return data.vid.v;
+				return data.vid().v;
 			});
 			if (id != last->id) {
 				return &message;
@@ -2495,27 +2499,28 @@ bool History::isServerSideUnread(not_null<const HistoryItem*> item) const {
 void History::applyDialog(
 		Data::Folder *requestFolder,
 		const MTPDdialog &data) {
-	const auto folder = data.has_folder_id()
-		? (data.vfolder_id.v
-			? owner().folder(data.vfolder_id.v).get()
-			: nullptr)
-		: requestFolder;
+	const auto folderId = data.vfolder_id();
+	const auto folder = !folderId
+		? requestFolder
+		: folderId->v
+		? owner().folder(folderId->v).get()
+		: nullptr;
 	applyDialogFields(
 		folder,
-		data.vunread_count.v,
-		data.vread_inbox_max_id.v,
-		data.vread_outbox_max_id.v);
-	applyDialogTopMessage(data.vtop_message.v);
+		data.vunread_count().v,
+		data.vread_inbox_max_id().v,
+		data.vread_outbox_max_id().v);
+	applyDialogTopMessage(data.vtop_message().v);
 	setUnreadMark(data.is_unread_mark());
-	setUnreadMentionsCount(data.vunread_mentions_count.v);
+	setUnreadMentionsCount(data.vunread_mentions_count().v);
 	if (const auto channel = peer->asChannel()) {
-		if (data.has_pts()) {
-			channel->ptsReceived(data.vpts.v);
+		if (const auto pts = data.vpts()) {
+			channel->ptsReceived(pts->v);
 		}
 		if (!channel->amCreator()) {
 			const auto topMessageId = FullMsgId(
 				peerToChannel(channel->id),
-				data.vtop_message.v);
+				data.vtop_message().v);
 			if (const auto item = owner().message(topMessageId)) {
 				if (item->date() <= channel->date) {
 					session().api().requestSelfParticipant(channel);
@@ -2524,11 +2529,12 @@ void History::applyDialog(
 		}
 	}
 	owner().applyNotifySetting(
-		MTP_notifyPeer(data.vpeer),
-		data.vnotify_settings);
+		MTP_notifyPeer(data.vpeer()),
+		data.vnotify_settings());
 
-	if (data.has_draft() && data.vdraft.type() == mtpc_draftMessage) {
-		Data::applyPeerCloudDraft(peer->id, data.vdraft.c_draftMessage());
+	const auto draft = data.vdraft();
+	if (draft && draft->type() == mtpc_draftMessage) {
+		Data::applyPeerCloudDraft(peer->id, draft->c_draftMessage());
 	}
 	session().api().dialogEntryApplied(this);
 }
