@@ -25,7 +25,7 @@ https://github.com/telegramdesktop/tdesktop/blob/master/LEGAL
 #include "core/application.h"
 #include "layout.h"
 #include "window/notifications_manager.h"
-#include "window/window_controller.h"
+#include "window/window_session_controller.h"
 #include "observer_peer.h"
 #include "storage/storage_shared_media.h"
 #include "data/data_session.h"
@@ -71,6 +71,31 @@ MTPDmessage::Flags NewForwardedFlags(
 	return result;
 }
 
+bool CopyMarkupToForward(not_null<const HistoryItem*> item) {
+	auto mediaOriginal = item->media();
+	if (mediaOriginal && mediaOriginal->game()) {
+		// Copy inline keyboard when forwarding messages with a game.
+		return true;
+	}
+	const auto markup = item->inlineReplyMarkup();
+	if (!markup) {
+		return false;
+	}
+	using Type = HistoryMessageMarkupButton::Type;
+	for (const auto &row : markup->rows) {
+		for (const auto &button : row) {
+			const auto switchInline = (button.type == Type::SwitchInline)
+				|| (button.type == Type::SwitchInlineSame);
+			const auto url = (button.type == Type::Url)
+				|| (button.type == Type::Auth);
+			if ((!switchInline || !item->viaBot()) && !url) {
+				return false;
+			}
+		}
+	}
+	return true;
+}
+
 bool HasInlineItems(const HistoryItemsList &items) {
 	for (const auto item : items) {
 		if (item->viaBot()) {
@@ -116,7 +141,7 @@ void FastShareMessage(not_null<HistoryItem*> item) {
 
 						QApplication::clipboard()->setText(link);
 
-						Ui::Toast::Show(lang(lng_share_game_link_copied));
+						Ui::Toast::Show(tr::lng_share_game_link_copied(tr::now));
 					}
 				}
 			}
@@ -158,7 +183,7 @@ void FastShareMessage(not_null<HistoryItem*> item) {
 			history->session().api().applyUpdates(updates);
 			data->requests.remove(requestId);
 			if (data->requests.empty()) {
-				Ui::Toast::Show(lang(lng_share_done));
+				Ui::Toast::Show(tr::lng_share_done(tr::now));
 				Ui::hideLayer();
 			}
 		};
@@ -251,7 +276,7 @@ QString GetErrorTextForForward(
 		not_null<PeerData*> peer,
 		const HistoryItemsList &items) {
 	if (!peer->canWrite()) {
-		return lang(lng_forward_cant);
+		return tr::lng_forward_cant(tr::now);
 	}
 
 	for (const auto item : items) {
@@ -262,12 +287,10 @@ QString GetErrorTextForForward(
 			}
 		}
 	}
-	const auto errorKey = Data::RestrictionErrorKey(
+	const auto error = Data::RestrictionError(
 		peer,
 		ChatRestriction::f_send_inline);
-	return (errorKey && HasInlineItems(items))
-		? lang(*errorKey)
-		: QString();
+	return (error && HasInlineItems(items)) ? *error : QString();
 }
 
 struct HistoryMessage::CreateConfig {
@@ -294,18 +317,20 @@ struct HistoryMessage::CreateConfig {
 void HistoryMessage::FillForwardedInfo(
 		CreateConfig &config,
 		const MTPDmessageFwdHeader &data) {
-	config.originalDate = data.vdate.v;
-	if (data.has_from_id() || data.has_channel_id()) {
-		config.senderOriginal = data.has_channel_id()
-			? peerFromChannel(data.vchannel_id)
-			: peerFromUser(data.vfrom_id);
+	if (const auto channelId = data.vchannel_id()) {
+		config.senderOriginal = peerFromChannel(*channelId);
+	} else if (const auto fromId = data.vfrom_id()) {
+		config.senderOriginal = peerFromUser(*fromId);
 	}
-	if (data.has_from_name()) config.senderNameOriginal = qs(data.vfrom_name);
-	if (data.has_channel_post()) config.originalId = data.vchannel_post.v;
-	if (data.has_post_author()) config.authorOriginal = qs(data.vpost_author);
-	if (data.has_saved_from_peer() && data.has_saved_from_msg_id()) {
-		config.savedFromPeer = peerFromMTP(data.vsaved_from_peer);
-		config.savedFromMsgId = data.vsaved_from_msg_id.v;
+	config.originalDate = data.vdate().v;
+	config.senderNameOriginal = qs(data.vfrom_name().value_or_empty());
+	config.originalId = data.vchannel_post().value_or_empty();
+	config.authorOriginal = qs(data.vpost_author().value_or_empty());
+	const auto savedFromPeer = data.vsaved_from_peer();
+	const auto savedFromMsgId = data.vsaved_from_msg_id();
+	if (savedFromPeer && savedFromMsgId) {
+		config.savedFromPeer = peerFromMTP(*savedFromPeer);
+		config.savedFromMsgId = savedFromMsgId->v;
 	}
 }
 
@@ -314,38 +339,35 @@ HistoryMessage::HistoryMessage(
 	const MTPDmessage &data)
 : HistoryItem(
 		history,
-		data.vid.v,
-		data.vflags.v,
-		data.vdate.v,
-		data.has_from_id() ? data.vfrom_id.v : UserId(0)) {
+		data.vid().v,
+		data.vflags().v,
+		data.vdate().v,
+		data.vfrom_id().value_or_empty()) {
 	auto config = CreateConfig();
-
-	if (data.has_fwd_from()) {
-		data.vfwd_from.match([&](const MTPDmessageFwdHeader &data) {
+	if (const auto forwarded = data.vfwd_from()) {
+		forwarded->match([&](const MTPDmessageFwdHeader &data) {
 			FillForwardedInfo(config, data);
 		});
 	}
-	if (data.has_reply_to_msg_id()) config.replyTo = data.vreply_to_msg_id.v;
-	if (data.has_via_bot_id()) config.viaBotId = data.vvia_bot_id.v;
-	if (data.has_views()) config.viewsCount = data.vviews.v;
-	if (data.has_reply_markup()) config.mtpMarkup = &data.vreply_markup;
-	if (data.has_edit_date()) config.editDate = data.vedit_date.v;
-	if (data.has_post_author()) config.author = qs(data.vpost_author);
+	config.replyTo = data.vreply_to_msg_id().value_or_empty();
+	config.viaBotId = data.vvia_bot_id().value_or_empty();
+	config.viewsCount = data.vviews().value_or(-1);
+	config.mtpMarkup = data.vreply_markup();
+	config.editDate = data.vedit_date().value_or_empty();
+	config.author = qs(data.vpost_author().value_or_empty());
 
 	createComponents(config);
 
-	if (data.has_media()) {
-		setMedia(data.vmedia);
+	if (const auto media = data.vmedia()) {
+		setMedia(*media);
 	}
-
-	auto text = TextUtilities::Clean(qs(data.vmessage));
-	auto entities = data.has_entities()
-		? TextUtilities::EntitiesFromMTP(data.ventities.v)
-		: EntitiesInText();
-	setText({ text, entities });
-
-	if (data.has_grouped_id()) {
-		setGroupId(MessageGroupId::FromRaw(data.vgrouped_id.v));
+	setText({
+		TextUtilities::Clean(qs(data.vmessage())),
+		TextUtilities::EntitiesFromMTP(data.ventities().value_or_empty())
+	});
+	if (const auto groupedId = data.vgrouped_id()) {
+		setGroupId(
+			MessageGroupId::FromRaw(history->peer->id, groupedId->v));
 	}
 }
 
@@ -354,21 +376,21 @@ HistoryMessage::HistoryMessage(
 	const MTPDmessageService &data)
 : HistoryItem(
 		history,
-		data.vid.v,
-		mtpCastFlags(data.vflags.v),
-		data.vdate.v,
-		data.has_from_id() ? data.vfrom_id.v : UserId(0)) {
+		data.vid().v,
+		mtpCastFlags(data.vflags().v),
+		data.vdate().v,
+		data.vfrom_id().value_or_empty()) {
 	auto config = CreateConfig();
 
-	if (data.has_reply_to_msg_id()) config.replyTo = data.vreply_to_msg_id.v;
+	config.replyTo = data.vreply_to_msg_id().value_or_empty();
 
 	createComponents(config);
 
-	switch (data.vaction.type()) {
+	switch (data.vaction().type()) {
 	case mtpc_messageActionPhoneCall: {
 		_media = std::make_unique<Data::MediaCall>(
 			this,
-			data.vaction.c_messageActionPhoneCall());
+			data.vaction().c_messageActionPhoneCall());
 	} break;
 
 	default: Unexpected("Service message action type in HistoryMessage.");
@@ -426,18 +448,20 @@ HistoryMessage::HistoryMessage(
 	if (flags & MTPDmessage::Flag::f_post_author) {
 		config.author = postAuthor;
 	}
-	auto fwdViaBot = original->viaBot();
-	if (fwdViaBot) config.viaBotId = peerToUser(fwdViaBot->id);
-	int fwdViewsCount = original->viewsCount();
+	if (const auto fwdViaBot = original->viaBot()) {
+		config.viaBotId = peerToUser(fwdViaBot->id);
+	}
+	const auto fwdViewsCount = original->viewsCount();
 	if (fwdViewsCount > 0) {
 		config.viewsCount = fwdViewsCount;
-	} else if (isPost()) {
+	} else if (isPost()
+		|| (original->senderOriginal()
+			&& original->senderOriginal()->isChannel())) {
 		config.viewsCount = 1;
 	}
 
-	// Copy inline keyboard when forwarding messages with a game.
-	auto mediaOriginal = original->media();
-	if (mediaOriginal && mediaOriginal->game()) {
+	const auto mediaOriginal = original->media();
+	if (CopyMarkupToForward(original)) {
 		config.inlineMarkup = original->inlineReplyMarkup();
 	}
 
@@ -467,8 +491,8 @@ HistoryMessage::HistoryMessage(
 	UserId from,
 	const QString &postAuthor,
 	const TextWithEntities &textWithEntities)
-: HistoryItem(history, id, flags, date, (flags & MTPDmessage::Flag::f_from_id) ? from : 0) {
-	createComponentsHelper(flags, replyTo, viaBotId, postAuthor, MTPReplyMarkup());
+: HistoryItem(history, id, flags & ~MTPDmessage::Flag::f_reply_markup, date, (flags & MTPDmessage::Flag::f_from_id) ? from : 0) {
+	createComponentsHelper(flags & ~MTPDmessage::Flag::f_reply_markup, replyTo, viaBotId, postAuthor, MTPReplyMarkup());
 
 	setText(textWithEntities);
 }
@@ -553,12 +577,6 @@ int HistoryMessage::viewsCount() const {
 	return HistoryItem::viewsCount();
 }
 
-PeerData *HistoryMessage::displayFrom() const {
-	return history()->peer->isSelf()
-		? senderOriginal()
-		: author().get();
-}
-
 bool HistoryMessage::updateDependencyItem() {
 	if (const auto reply = Get<HistoryMessageReply>()) {
 		return reply->updateData(this, true);
@@ -602,6 +620,10 @@ bool HistoryMessage::allowsForward() const {
 	return !_media || _media->allowsForward();
 }
 
+bool HistoryMessage::hasMessageBadge() const {
+	return hasAdminBadge() || isDiscussionPost();
+}
+
 bool HistoryMessage::isTooOldForEdit(TimeId now) const {
 	const auto peer = _history->peer;
 	if (peer->isSelf()) {
@@ -618,7 +640,7 @@ bool HistoryMessage::allowsEdit(TimeId now) const {
 	return canStopPoll()
 		&& !isTooOldForEdit(now)
 		&& (!_media || _media->allowsEdit())
-		&& !isUnsupportedMessage()
+		&& !isLegacyMessage()
 		&& !isEditingMedia();
 }
 
@@ -640,13 +662,6 @@ void HistoryMessage::createComponents(const CreateConfig &config) {
 	if (!config.author.isEmpty()) {
 		mask |= HistoryMessageSigned::Bit();
 	}
-	auto hasViaBot = (config.viaBotId != 0);
-	auto hasInlineMarkup = [&config] {
-		if (config.mtpMarkup) {
-			return (config.mtpMarkup->type() == mtpc_replyInlineMarkup);
-		}
-		return (config.inlineMarkup != nullptr);
-	};
 	if (config.editDate != TimeId(0)) {
 		mask |= HistoryMessageEdited::Bit();
 	}
@@ -656,7 +671,7 @@ void HistoryMessage::createComponents(const CreateConfig &config) {
 	if (config.mtpMarkup) {
 		// optimization: don't create markup component for the case
 		// MTPDreplyKeyboardHide with flags = 0, assume it has f_zero flag
-		if (config.mtpMarkup->type() != mtpc_replyKeyboardHide || config.mtpMarkup->c_replyKeyboardHide().vflags.v != 0) {
+		if (config.mtpMarkup->type() != mtpc_replyKeyboardHide || config.mtpMarkup->c_replyKeyboardHide().vflags().v != 0) {
 			mask |= HistoryMessageReplyMarkup::Bit();
 		}
 	} else if (config.inlineMarkup) {
@@ -721,25 +736,6 @@ void HistoryMessage::setupForwardedComponent(const CreateConfig &config) {
 	forwarded->savedFromMsgId = config.savedFromMsgId;
 }
 
-QString FormatViewsCount(int views) {
-	if (views > 999999) {
-		views /= 100000;
-		if (views % 10) {
-			return QString::number(views / 10) + '.' + QString::number(views % 10) + 'M';
-		}
-		return QString::number(views / 10) + 'M';
-	} else if (views > 9999) {
-		views /= 100;
-		if (views % 10) {
-			return QString::number(views / 10) + '.' + QString::number(views % 10) + 'K';
-		}
-		return QString::number(views / 10) + 'K';
-	} else if (views > 0) {
-		return QString::number(views);
-	}
-	return qsl("1");
-}
-
 void HistoryMessage::refreshMedia(const MTPMessageMedia *media) {
 	_media = nullptr;
 	if (media) {
@@ -786,49 +782,50 @@ std::unique_ptr<Data::Media> HistoryMessage::CreateMedia(
 	return media.match([&](const MTPDmessageMediaContact &media) -> Result {
 		return std::make_unique<Data::MediaContact>(
 			item,
-			media.vuser_id.v,
-			qs(media.vfirst_name),
-			qs(media.vlast_name),
-			qs(media.vphone_number));
+			media.vuser_id().v,
+			qs(media.vfirst_name()),
+			qs(media.vlast_name()),
+			qs(media.vphone_number()));
 	}, [&](const MTPDmessageMediaGeo &media) -> Result {
-		return media.vgeo.match([&](const MTPDgeoPoint &point) -> Result {
+		return media.vgeo().match([&](const MTPDgeoPoint &point) -> Result {
 			return std::make_unique<Data::MediaLocation>(
 				item,
-				LocationCoords(point));
+				Data::LocationPoint(point));
 		}, [](const MTPDgeoPointEmpty &) -> Result {
 			return nullptr;
 		});
 	}, [&](const MTPDmessageMediaGeoLive &media) -> Result {
-		return media.vgeo.match([&](const MTPDgeoPoint &point) -> Result {
+		return media.vgeo().match([&](const MTPDgeoPoint &point) -> Result {
 			return std::make_unique<Data::MediaLocation>(
 				item,
-				LocationCoords(point));
+				Data::LocationPoint(point));
 		}, [](const MTPDgeoPointEmpty &) -> Result {
 			return nullptr;
 		});
 	}, [&](const MTPDmessageMediaVenue &media) -> Result {
-		return media.vgeo.match([&](const MTPDgeoPoint &point) -> Result {
+		return media.vgeo().match([&](const MTPDgeoPoint &point) -> Result {
 			return std::make_unique<Data::MediaLocation>(
 				item,
-				LocationCoords(point),
-				qs(media.vtitle),
-				qs(media.vaddress));
+				Data::LocationPoint(point),
+				qs(media.vtitle()),
+				qs(media.vaddress()));
 		}, [](const MTPDgeoPointEmpty &data) -> Result {
 			return nullptr;
 		});
 	}, [&](const MTPDmessageMediaPhoto &media) -> Result {
-		if (media.has_ttl_seconds()) {
+		const auto photo = media.vphoto();
+		if (media.vttl_seconds()) {
 			LOG(("App Error: "
 				"Unexpected MTPMessageMediaPhoto "
 				"with ttl_seconds in HistoryMessage."));
 			return nullptr;
-		} else if (!media.has_photo()) {
+		} else if (!photo) {
 			LOG(("API Error: "
 				"Got MTPMessageMediaPhoto "
 				"without photo and without ttl_seconds."));
 			return nullptr;
 		}
-		return media.vphoto.match([&](const MTPDphoto &photo) -> Result {
+		return photo->match([&](const MTPDphoto &photo) -> Result {
 			return std::make_unique<Data::MediaPhoto>(
 				item,
 				item->history()->owner().processPhoto(photo));
@@ -836,19 +833,19 @@ std::unique_ptr<Data::Media> HistoryMessage::CreateMedia(
 			return nullptr;
 		});
 	}, [&](const MTPDmessageMediaDocument &media) -> Result {
-		if (media.has_ttl_seconds()) {
+		const auto document = media.vdocument();
+		if (media.vttl_seconds()) {
 			LOG(("App Error: "
 				"Unexpected MTPMessageMediaDocument "
 				"with ttl_seconds in HistoryMessage."));
 			return nullptr;
-		} else if (!media.has_document()) {
+		} else if (!document) {
 			LOG(("API Error: "
 				"Got MTPMessageMediaDocument "
 				"without document and without ttl_seconds."));
 			return nullptr;
 		}
-		const auto &document = media.vdocument;
-		return document.match([&](const MTPDdocument &document) -> Result {
+		return document->match([&](const MTPDdocument &document) -> Result {
 			return std::make_unique<Data::MediaFile>(
 				item,
 				item->history()->owner().processDocument(document));
@@ -856,7 +853,7 @@ std::unique_ptr<Data::Media> HistoryMessage::CreateMedia(
 			return nullptr;
 		});
 	}, [&](const MTPDmessageMediaWebPage &media) {
-		return media.vwebpage.match([](const MTPDwebPageEmpty &) -> Result {
+		return media.vwebpage().match([](const MTPDwebPageEmpty &) -> Result {
 			return nullptr;
 		}, [&](const MTPDwebPagePending &webpage) -> Result {
 			return std::make_unique<Data::MediaWebPage>(
@@ -872,7 +869,7 @@ std::unique_ptr<Data::Media> HistoryMessage::CreateMedia(
 			return nullptr;
 		});
 	}, [&](const MTPDmessageMediaGame &media) -> Result {
-		return media.vgame.match([&](const MTPDgame &game) {
+		return media.vgame().match([&](const MTPDgame &game) {
 			return std::make_unique<Data::MediaGame>(
 				item,
 				item->history()->owner().processGame(game));
@@ -896,7 +893,7 @@ void HistoryMessage::replaceBuyWithReceiptInMarkup() {
 		for (auto &row : markup->rows) {
 			for (auto &button : row) {
 				if (button.type == HistoryMessageMarkupButton::Type::Buy) {
-					button.text = lang(lng_payments_receipt_button);
+					button.text = tr::lng_payments_receipt_button(tr::now);
 				}
 			}
 		}
@@ -912,31 +909,31 @@ void HistoryMessage::applyEdition(const MTPDmessage &message) {
 	//	}
 	//}
 
-	if (message.has_edit_date()) {
+	if (const auto editDate = message.vedit_date()) {
 		_flags |= MTPDmessage::Flag::f_edit_date;
 		if (!Has<HistoryMessageEdited>()) {
 			AddComponents(HistoryMessageEdited::Bit());
 		}
 		auto edited = Get<HistoryMessageEdited>();
-		edited->date = message.vedit_date.v;
+		edited->date = editDate->v;
 	}
 
-	TextWithEntities textWithEntities = { qs(message.vmessage), EntitiesInText() };
-	if (message.has_entities()) {
-		textWithEntities.entities = TextUtilities::EntitiesFromMTP(message.ventities.v);
-	}
-	setReplyMarkup(message.has_reply_markup() ? (&message.vreply_markup) : nullptr);
+	const auto textWithEntities = TextWithEntities{
+		qs(message.vmessage()),
+		TextUtilities::EntitiesFromMTP(message.ventities().value_or_empty())
+	};
+	setReplyMarkup(message.vreply_markup());
 	if (!isLocalUpdateMedia()) {
-		refreshMedia(message.has_media() ? (&message.vmedia) : nullptr);
+		refreshMedia(message.vmedia());
 	}
-	setViewsCount(message.has_views() ? message.vviews.v : -1);
+	setViewsCount(message.vviews().value_or(-1));
 	setText(textWithEntities);
 
 	finishEdition(keyboardTop);
 }
 
 void HistoryMessage::applyEdition(const MTPDmessageService &message) {
-	if (message.vaction.type() == mtpc_messageActionHistoryClear) {
+	if (message.vaction().type() == mtpc_messageActionHistoryClear) {
 		setReplyMarkup(nullptr);
 		refreshMedia(nullptr);
 		setEmptyText();
@@ -1062,7 +1059,7 @@ void HistoryMessage::setReplyMarkup(const MTPReplyMarkup *markup) {
 
 	// optimization: don't create markup component for the case
 	// MTPDreplyKeyboardHide with flags = 0, assume it has f_zero flag
-	if (markup->type() == mtpc_replyKeyboardHide && markup->c_replyKeyboardHide().vflags.v == 0) {
+	if (markup->type() == mtpc_replyKeyboardHide && markup->c_replyKeyboardHide().vflags().v == 0) {
 		bool changed = false;
 		if (Has<HistoryMessageReplyMarkup>()) {
 			RemoveComponents(HistoryMessageReplyMarkup::Bit());
@@ -1117,9 +1114,9 @@ void HistoryMessage::setViewsCount(int32 count) {
 
 	const auto was = views->_viewsWidth;
 	views->_views = count;
-	views->_viewsText = (views->_views >= 0)
-		? FormatViewsCount(views->_views)
-		: QString();
+	views->_viewsText = (views->_views > 0)
+		? Lang::FormatCountToShort(views->_views).string
+		: QString("1");
 	views->_viewsWidth = views->_viewsText.isEmpty()
 		? 0
 		: st::msgDateFont->width(views->_viewsText);
