@@ -93,8 +93,8 @@ QSize HistoryWebPage::countOptimalSize() {
 		_openl = nullptr;
 		_attach = nullptr;
 		_collage = PrepareCollageMedia(_parent->data(), _data->collage);
-		_title = Text(st::msgMinWidth - st::webPageLeft);
-		_description = Text(st::msgMinWidth - st::webPageLeft);
+		_title = Ui::Text::String(st::msgMinWidth - st::webPageLeft);
+		_description = Ui::Text::String(st::msgMinWidth - st::webPageLeft);
 		_siteNameWidth = 0;
 	}
 	auto lineHeight = unitedLineHeight();
@@ -191,7 +191,7 @@ QSize HistoryWebPage::countOptimalSize() {
 		}
 		if (isLogEntryOriginal()) {
 			// Fix layout for small bubbles (narrow media caption edit log entries).
-			_description = Text(st::minPhotoSize
+			_description = Ui::Text::String(st::minPhotoSize
 				- st::msgPadding.left()
 				- st::msgPadding.right()
 				- st::webPageLeft);
@@ -561,7 +561,7 @@ TextState HistoryWebPage::textState(QPoint point, StateRequest request) const {
 	}
 	if (_titleLines) {
 		if (point.y() >= tshift && point.y() < tshift + _titleLines * lineHeight) {
-			Text::StateRequestElided titleRequest = request.forText();
+			Ui::Text::StateRequestElided titleRequest = request.forText();
 			titleRequest.lines = _titleLines;
 			result = TextState(_parent, _title.getStateElidedLeft(
 				point - QPoint(padding.left(), tshift),
@@ -577,7 +577,7 @@ TextState HistoryWebPage::textState(QPoint point, StateRequest request) const {
 		auto descriptionHeight = (_descriptionLines > 0) ? _descriptionLines * lineHeight : _description.countHeight(paintw);
 		if (point.y() >= tshift && point.y() < tshift + descriptionHeight) {
 			if (_descriptionLines > 0) {
-				Text::StateRequestElided descriptionRequest = request.forText();
+				Ui::Text::StateRequestElided descriptionRequest = request.forText();
 				descriptionRequest.lines = _descriptionLines;
 				result = TextState(_parent, _description.getStateElidedLeft(
 					point - QPoint(padding.left(), tshift),
@@ -726,7 +726,7 @@ int HistoryWebPage::bottomInfoPadding() const {
 
 QString HistoryWebPage::displayedSiteName() const {
 	return (_data->document && _data->document->isWallPaper())
-		? lang(lng_media_chat_background)
+		? tr::lng_media_chat_background(tr::now)
 		: _data->siteName;
 }
 
