@@ -9,6 +9,7 @@ https://github.com/telegramdesktop/tdesktop/blob/master/LEGAL
 
 #include "ui/rp_widget.h"
 #include "ui/wrap/fade_wrap.h"
+#include "ui/effects/animations.h"
 #include "ui/effects/numbers_animation.h"
 #include "info/info_wrap_widget.h"
 
@@ -82,6 +83,8 @@ public:
 		updateControlsVisibility(anim::type::instant);
 	}
 
+	void showSearch();
+
 protected:
 	int resizeGetHeight(int newWidth) override;
 	void paintEvent(QPaintEvent *e) override;
@@ -128,7 +131,7 @@ private:
 	void registerToggleControlCallback(Widget *widget, IsVisible &&callback);
 
 	const style::InfoTopBar &_st;
-	Animation _a_highlight;
+	Ui::Animations::Simple _a_highlight;
 	bool _highlight = false;
 	QPointer<Ui::FadeWrap<Ui::IconButton>> _back;
 	std::vector<base::unique_qptr<Ui::RpWidget>> _buttons;
