@@ -11,6 +11,8 @@ https://github.com/telegramdesktop/tdesktop/blob/master/LEGAL
 #include "core/application.h"
 #include "auth_session.h"
 #include "ui/wrap/slide_wrap.h"
+#include "ui/text/text_utilities.h"
+#include "lang/lang_keys.h"
 #include "data/data_peer_values.h"
 #include "data/data_shared_media.h"
 #include "data/data_folder.h"
@@ -23,6 +25,25 @@ https://github.com/telegramdesktop/tdesktop/blob/master/LEGAL
 
 namespace Info {
 namespace Profile {
+namespace {
+
+auto PlainBioValue(not_null<UserData*> user) {
+	return Notify::PeerUpdateValue(
+		user,
+		Notify::PeerUpdate::Flag::AboutChanged
+	) | rpl::map([=] { return user->about(); });
+}
+
+auto PlainUsernameValue(not_null<PeerData*> peer) {
+	return Notify::PeerUpdateValue(
+		peer,
+		Notify::PeerUpdate::Flag::UsernameChanged
+	) | rpl::map([=] {
+		return peer->userName();
+	});
+}
+
+} // namespace
 
 rpl::producer<TextWithEntities> NameValue(not_null<PeerData*> peer) {
 	return Notify::PeerUpdateValue(
@@ -30,7 +51,7 @@ rpl::producer<TextWithEntities> NameValue(not_null<PeerData*> peer) {
 		Notify::PeerUpdate::Flag::NameChanged
 	) | rpl::map([=] {
 		return App::peerName(peer);
-	}) | WithEmptyEntities();
+	}) | Ui::Text::ToWithEntities();
 }
 
 rpl::producer<TextWithEntities> PhoneValue(not_null<UserData*> user) {
@@ -39,29 +60,30 @@ rpl::producer<TextWithEntities> PhoneValue(not_null<UserData*> user) {
 			Notify::PeerUpdate::Flag::UserPhoneChanged
 	) | rpl::map([user] {
 		return App::formatPhone(user->phone());
-	}) | WithEmptyEntities();
+	}) | Ui::Text::ToWithEntities();
 }
 
-auto PlainBioValue(not_null<UserData*> user) {
-	return Notify::PeerUpdateValue(
-		user,
-		Notify::PeerUpdate::Flag::AboutChanged
-	) | rpl::map([user] { return user->about(); });
+rpl::producer<TextWithEntities> PhoneOrHiddenValue(not_null<UserData*> user) {
+	return rpl::combine(
+		PhoneValue(user),
+		PlainUsernameValue(user),
+		PlainBioValue(user),
+		tr::lng_info_mobile_hidden()
+	) | rpl::map([](
+			const TextWithEntities &phone,
+			const QString &username,
+			const QString &bio,
+			const QString &hidden) {
+		return (phone.text.isEmpty() && username.isEmpty() && bio.isEmpty())
+			? Ui::Text::WithEntities(hidden)
+			: phone;
+	});
 }
 
 rpl::producer<TextWithEntities> BioValue(not_null<UserData*> user) {
 	return PlainBioValue(user)
 		| ToSingleLine()
-		| WithEmptyEntities();
-}
-
-auto PlainUsernameValue(not_null<PeerData*> peer) {
-	return Notify::PeerUpdateValue(
-			peer,
-			Notify::PeerUpdate::Flag::UsernameChanged
-	) | rpl::map([peer] {
-		return peer->userName();
-	});
+		| Ui::Text::ToWithEntities();
 }
 
 rpl::producer<TextWithEntities> UsernameValue(not_null<UserData*> user) {
@@ -71,7 +93,7 @@ rpl::producer<TextWithEntities> UsernameValue(not_null<UserData*> user) {
 		return username.isEmpty()
 			? QString()
 			: ('@' + username);
-	}) | WithEmptyEntities();
+	}) | Ui::Text::ToWithEntities();
 }
 
 rpl::producer<QString> PlainAboutValue(not_null<PeerData*> peer) {
@@ -95,7 +117,7 @@ rpl::producer<TextWithEntities> AboutValue(not_null<PeerData*> peer) {
 	}
 	return PlainAboutValue(
 		peer
-	) | WithEmptyEntities(
+	) | Ui::Text::ToWithEntities(
 	) | rpl::map([=](TextWithEntities &&text) {
 		TextUtilities::ParseEntities(text, flags);
 		return std::move(text);
@@ -112,6 +134,16 @@ rpl::producer<QString> LinkValue(not_null<PeerData*> peer) {
 	});
 }
 
+rpl::producer<const ChannelLocation*> LocationValue(
+		not_null<ChannelData*> channel) {
+	return Notify::PeerUpdateValue(
+		channel,
+		Notify::PeerUpdate::Flag::ChannelLocation
+	) | rpl::map([=] {
+		return channel->getLocation();
+	});
+}
+
 rpl::producer<bool> NotificationsEnabledValue(not_null<PeerData*> peer) {
 	return rpl::merge(
 		Notify::PeerUpdateValue(
@@ -154,10 +186,12 @@ rpl::producer<bool> CanShareContactValue(not_null<UserData*> user) {
 
 rpl::producer<bool> CanAddContactValue(not_null<UserData*> user) {
 	using namespace rpl::mappers;
-	return rpl::combine(
-		IsContactValue(user),
-		CanShareContactValue(user),
-		!_1 && _2);
+	if (user->isBot() || user->isSelf()) {
+		return rpl::single(false);
+	}
+	return IsContactValue(
+		user
+	) | rpl::map(!_1);
 }
 
 rpl::producer<bool> AmInChannelValue(not_null<ChannelData*> channel) {
@@ -297,14 +331,14 @@ rpl::producer<int> CommonGroupsCountValue(not_null<UserData*> user) {
 }
 
 rpl::producer<bool> CanAddMemberValue(not_null<PeerData*> peer) {
-	if (auto chat = peer->asChat()) {
+	if (const auto chat = peer->asChat()) {
 		return Notify::PeerUpdateValue(
 			chat,
 			Notify::PeerUpdate::Flag::RightsChanged
 		) | rpl::map([=] {
 			return chat->canAddMembers();
 		});
-	} else if (auto channel = peer->asChannel()) {
+	} else if (const auto channel = peer->asChannel()) {
 		return Notify::PeerUpdateValue(
 			channel,
 			Notify::PeerUpdate::Flag::RightsChanged
@@ -316,15 +350,26 @@ rpl::producer<bool> CanAddMemberValue(not_null<PeerData*> peer) {
 }
 
 rpl::producer<bool> VerifiedValue(not_null<PeerData*> peer) {
-	if (auto user = peer->asUser()) {
+	if (const auto user = peer->asUser()) {
 		return Data::PeerFlagValue(user, MTPDuser::Flag::f_verified);
-	} else if (auto channel = peer->asChannel()) {
+	} else if (const auto channel = peer->asChannel()) {
 		return Data::PeerFlagValue(
 			channel,
 			MTPDchannel::Flag::f_verified);
 	}
 	return rpl::single(false);
 }
+
+rpl::producer<bool> ScamValue(not_null<PeerData*> peer) {
+	if (const auto user = peer->asUser()) {
+		return Data::PeerFlagValue(user, MTPDuser::Flag::f_scam);
+	} else if (const auto channel = peer->asChannel()) {
+		return Data::PeerFlagValue(
+			channel,
+			MTPDchannel::Flag::f_scam);
+	}
+	return rpl::single(false);
+}
 // // #feed
 //rpl::producer<int> FeedChannelsCountValue(not_null<Data::Feed*> feed) {
 //	using Flag = Data::FeedUpdateFlag;
