@@ -13,41 +13,13 @@ https://github.com/telegramdesktop/tdesktop/blob/master/LEGAL
 #include "ui/widgets/buttons.h"
 #include "ui/widgets/labels.h"
 #include "ui/wrap/fade_wrap.h"
-#include "core/click_handler_types.h"
+#include "boxes/confirm_phone_box.h"
 #include "boxes/confirm_box.h"
-#include "base/qthelp_url.h"
-#include "platform/platform_specific.h"
 #include "core/application.h"
 
 namespace Intro {
 namespace {
 
-void SendToBannedHelp(const QString &phone) {
-	const auto version = QString::fromLatin1(AppVersionStr)
-		+ (cAlphaVersion()
-			? qsl(" alpha %1").arg(cAlphaVersion())
-			: (AppBetaVersion ? " beta" : ""));
-
-	const auto subject = qsl("Banned phone number: ") + phone;
-
-	const auto body = qsl("\
-I'm trying to use my mobile phone number: ") + phone + qsl("\n\
-But Telegram says it's banned. Please help.\n\
-\n\
-App version: ") + version + qsl("\n\
-OS version: ") + cPlatformString() + qsl("\n\
-Locale: ") + Platform::SystemLanguage();
-
-	const auto url = "mailto:?to="
-		+ qthelp::url_encode("login@stel.com")
-		+ "&subject="
-		+ qthelp::url_encode(subject)
-		+ "&body="
-		+ qthelp::url_encode(body);
-
-	UrlClickHandler::Open(url);
-}
-
 bool AllowPhoneAttempt(const QString &phone) {
 	const auto digits = ranges::count_if(
 		phone,
@@ -72,8 +44,8 @@ PhoneWidget::PhoneWidget(QWidget *parent, Widget::Data *data) : Step(parent, dat
 	connect(_code, SIGNAL(changed()), this, SLOT(onInputChange()));
 	connect(_checkRequest, SIGNAL(timeout()), this, SLOT(onCheckRequest()));
 
-	setTitleText(langFactory(lng_phone_title));
-	setDescriptionText(langFactory(lng_phone_desc));
+	setTitleText(tr::lng_phone_title());
+	setDescriptionText(tr::lng_phone_desc());
 	subscribe(getData()->updated, [this] { countryChanged(); });
 	setErrorCentered(true);
 
@@ -100,9 +72,9 @@ void PhoneWidget::updateSignupGeometry() {
 	}
 }
 
-void PhoneWidget::showPhoneError(Fn<QString()> textFactory) {
+void PhoneWidget::showPhoneError(rpl::producer<QString> text) {
 	_phone->showError();
-	showError(std::move(textFactory));
+	showError(std::move(text));
 }
 
 void PhoneWidget::hidePhoneError() {
@@ -113,23 +85,6 @@ void PhoneWidget::hidePhoneError() {
 	}
 }
 
-//void PhoneWidget::showSignup() {
-//	showPhoneError(langFactory(lng_bad_phone_noreg));
-//	if (!_signup) {
-//		auto signupText = lng_phone_notreg(lt_link_start, textcmdStartLink(1), lt_link_end, textcmdStopLink(), lt_signup_start, textcmdStartLink(2), lt_signup_end, textcmdStopLink());
-//		auto inner = object_ptr<Ui::FlatLabel>(this, signupText, Ui::FlatLabel::InitType::Rich, st::introDescription);
-//		_signup.create(this, std::move(inner));
-//		_signup->entity()->setLink(1, std::make_shared<UrlClickHandler>(qsl("https://telegram.org"), false));
-//		_signup->entity()->setLink(2, std::make_shared<LambdaClickHandler>([this] {
-//			toSignUp();
-//		}));
-//		_signup->hide(anim::type::instant);
-//		updateSignupGeometry();
-//	}
-//	_signup->show(anim::type::normal);
-//	hideDescription();
-//}
-
 void PhoneWidget::countryChanged() {
 	if (!_changed) {
 		selectCountry(getData()->country);
@@ -146,7 +101,7 @@ void PhoneWidget::submit() {
 
 	const auto phone = fullNumber();
 	if (!AllowPhoneAttempt(phone)) {
-		showPhoneError(langFactory(lng_bad_phone));
+		showPhoneError(tr::lng_bad_phone());
 		_phone->setFocus();
 		return;
 	}
@@ -157,7 +112,6 @@ void PhoneWidget::submit() {
 
 	_sentPhone = phone;
 	Core::App().mtp()->setUserPhone(_sentPhone);
-	//_sentRequest = MTP::send(MTPauth_CheckPhone(MTP_string(_sentPhone)), rpcDone(&PhoneWidget::phoneCheckDone), rpcFail(&PhoneWidget::phoneSubmitFail));
 	_sentRequest = MTP::send(
 		MTPauth_SendCode(
 			MTP_string(_sentPhone),
@@ -186,40 +140,25 @@ void PhoneWidget::onCheckRequest() {
 		stopCheck();
 	}
 }
-//
-//void PhoneWidget::phoneCheckDone(const MTPauth_CheckedPhone &result) {
-//	stopCheck();
-//
-//	auto &d = result.c_auth_checkedPhone();
-//	if (mtpIsTrue(d.vphone_registered)) {
-//		hidePhoneError();
-//
-//		_checkRequest->start(1000);
-//
-//		_sentRequest = MTP::send(MTPauth_SendCode(MTP_flags(0), MTP_string(_sentPhone), MTPBool(), MTP_int(ApiId), MTP_string(ApiHash)), rpcDone(&PhoneWidget::phoneSubmitDone), rpcFail(&PhoneWidget::phoneSubmitFail));
-//	} else {
-//		showSignup();
-//		_sentRequest = 0;
-//	}
-//}
 
 void PhoneWidget::phoneSubmitDone(const MTPauth_SentCode &result) {
 	stopCheck();
 	_sentRequest = 0;
 
 	if (result.type() != mtpc_auth_sentCode) {
-		showPhoneError(&Lang::Hard::ServerError);
+		showPhoneError(rpl::single(Lang::Hard::ServerError()));
 		return;
 	}
 
 	const auto &d = result.c_auth_sentCode();
 	fillSentCodeData(d);
 	getData()->phone = _sentPhone;
-	getData()->phoneHash = qba(d.vphone_code_hash);
+	getData()->phoneHash = qba(d.vphone_code_hash());
 	getData()->phoneIsRegistered = d.is_phone_registered();
-	if (d.has_next_type() && d.vnext_type.type() == mtpc_auth_codeTypeCall) {
+	const auto next = d.vnext_type();
+	if (next && next->type() == mtpc_auth_codeTypeCall) {
 		getData()->callStatus = Widget::Data::CallStatus::Waiting;
-		getData()->callTimeout = d.has_timeout() ? d.vtimeout.v : 60;
+		getData()->callTimeout = d.vtimeout().value_or(60);
 	} else {
 		getData()->callStatus = Widget::Data::CallStatus::Disabled;
 		getData()->callTimeout = 0;
@@ -227,19 +166,11 @@ void PhoneWidget::phoneSubmitDone(const MTPauth_SentCode &result) {
 	goNext(new Intro::CodeWidget(parentWidget(), getData()));
 }
 
-//void PhoneWidget::toSignUp() {
-//	hideError(); // Hide error, but leave the signup label visible.
-//
-//	_checkRequest->start(1000);
-//
-//	_sentRequest = MTP::send(MTPauth_SendCode(MTP_flags(0), MTP_string(_sentPhone), MTPBool(), MTP_int(ApiId), MTP_string(ApiHash)), rpcDone(&PhoneWidget::phoneSubmitDone), rpcFail(&PhoneWidget::phoneSubmitFail));
-//}
-
 bool PhoneWidget::phoneSubmitFail(const RPCError &error) {
 	if (MTP::isFloodError(error)) {
 		stopCheck();
 		_sentRequest = 0;
-		showPhoneError(langFactory(lng_flood_error));
+		showPhoneError(tr::lng_flood_error());
 		return true;
 	}
 	if (MTP::isDefaultHandledError(error)) return false;
@@ -248,26 +179,19 @@ bool PhoneWidget::phoneSubmitFail(const RPCError &error) {
 	_sentRequest = 0;
 	auto &err = error.type();
 	if (err == qstr("PHONE_NUMBER_FLOOD")) {
-		Ui::show(Box<InformBox>(lang(lng_error_phone_flood)));
+		Ui::show(Box<InformBox>(tr::lng_error_phone_flood(tr::now)));
 		return true;
 	} else if (err == qstr("PHONE_NUMBER_INVALID")) { // show error
-		showPhoneError(langFactory(lng_bad_phone));
+		showPhoneError(tr::lng_bad_phone());
 		return true;
 	} else if (err == qstr("PHONE_NUMBER_BANNED")) {
-		const auto phone = _sentPhone;
-		Ui::show(Box<ConfirmBox>(
-			lang(lng_signin_banned_text),
-			lang(lng_box_ok),
-			lang(lng_signin_banned_help),
-			[] { Ui::hideLayer(); },
-			[phone] { SendToBannedHelp(phone); Ui::hideLayer(); }));
+		ShowPhoneBannedError(_sentPhone);
 		return true;
 	}
 	if (Logs::DebugEnabled()) { // internal server error
-		auto text = err + ": " + error.description();
-		showPhoneError([text] { return text; });
+		showPhoneError(rpl::single(err + ": " + error.description()));
 	} else {
-		showPhoneError(&Lang::Hard::ServerError);
+		showPhoneError(rpl::single(Lang::Hard::ServerError()));
 	}
 	return false;
 }
