@@ -28,7 +28,7 @@ public:
 	void activate() override;
 	void cancelled() override;
 	void submit() override;
-	QString nextButtonText() const override;
+	rpl::producer<QString> nextButtonText() const override;
 
 protected:
 	void resizeEvent(QResizeEvent *e) override;
