@@ -638,7 +638,7 @@ void TcpConnection::socketPacket(bytes::const_span bytes) {
 		try {
 			const auto res_pq = readPQFakeReply(data);
 			const auto &data = res_pq.c_resPQ();
-			if (data.vnonce == _checkNonce) {
+			if (data.vnonce() == _checkNonce) {
 				DEBUG_LOG(("Connection Info: Valid pq response by TCP."));
 				_status = Status::Ready;
 				disconnect(
