@@ -20,6 +20,7 @@ https://github.com/telegramdesktop/tdesktop/blob/master/LEGAL
 #include "ui/rp_widget.h"
 #include "ui/countryinput.h"
 #include "core/update_checker.h"
+#include "data/data_countries.h"
 #include "layout.h"
 #include "styles/style_boxes.h"
 
@@ -38,25 +39,27 @@ ScanInfo CollectScanInfo(const EditFile &file) {
 	const auto status = [&] {
 		if (file.fields.accessHash) {
 			if (file.fields.downloadOffset < 0) {
-				return lang(lng_attach_failed);
+				return tr::lng_attach_failed(tr::now);
 			} else if (file.fields.downloadOffset < file.fields.size) {
 				return formatDownloadText(
 					file.fields.downloadOffset,
 					file.fields.size);
 			} else {
-				return lng_passport_scan_uploaded(
+				return tr::lng_passport_scan_uploaded(
+					tr::now,
 					lt_date,
 					langDateTimeFull(ParseDateTime(file.fields.date)));
 			}
 		} else if (file.uploadData) {
 			if (file.uploadData->offset < 0) {
-				return lang(lng_attach_failed);
+				return tr::lng_attach_failed(tr::now);
 			} else if (file.uploadData->fullId) {
 				return formatDownloadText(
 					file.uploadData->offset,
 					file.uploadData->bytes.size());
 			} else {
-				return lng_passport_scan_uploaded(
+				return tr::lng_passport_scan_uploaded(
+					tr::now,
 					lt_date,
 					langDateTimeFull(ParseDateTime(file.fields.date)));
 			}
@@ -112,14 +115,14 @@ EditDocumentScheme GetDocumentScheme(
 	using ValueClass = Scheme::ValueClass;
 	const auto DontFormat = nullptr;
 	const auto CountryFormat = [](const QString &value) {
-		const auto result = CountrySelectBox::NameByISO(value);
+		const auto result = Data::CountryNameByISO2(value);
 		return result.isEmpty() ? value : result;
 	};
 	const auto GenderFormat = [](const QString &value) {
 		if (value == qstr("male")) {
-			return lang(lng_passport_gender_male);
+			return tr::lng_passport_gender_male(tr::now);
 		} else if (value == qstr("female")) {
-			return lang(lng_passport_gender_female);
+			return tr::lng_passport_gender_female(tr::now);
 		}
 		return value;
 	};
@@ -143,7 +146,7 @@ EditDocumentScheme GetDocumentScheme(
 		} else if (!QRegularExpression(
 			"^[a-zA-Z0-9\\.,/&\\-' ]+$"
 		).match(value).hasMatch()) {
-			return lang(lng_passport_bad_name);
+			return tr::lng_passport_bad_name(tr::now);
 		}
 		return std::nullopt;
 	};
@@ -183,19 +186,19 @@ EditDocumentScheme GetDocumentScheme(
 	case Scope::Type::PersonalDetails:
 	case Scope::Type::Identity: {
 		auto result = Scheme();
-		result.detailsHeader = lang(lng_passport_personal_details);
-		result.fieldsHeader = lang(lng_passport_document_details);
+		result.detailsHeader = tr::lng_passport_personal_details(tr::now);
+		result.fieldsHeader = tr::lng_passport_document_details(tr::now);
 		if (scansType) {
 			result.scansHeader = [&] {
 				switch (*scansType) {
 				case Value::Type::Passport:
-					return lang(lng_passport_identity_passport);
+					return tr::lng_passport_identity_passport(tr::now);
 				case Value::Type::DriverLicense:
-					return lang(lng_passport_identity_license);
+					return tr::lng_passport_identity_license(tr::now);
 				case Value::Type::IdentityCard:
-					return lang(lng_passport_identity_card);
+					return tr::lng_passport_identity_card(tr::now);
 				case Value::Type::InternalPassport:
-					return lang(lng_passport_identity_internal);
+					return tr::lng_passport_identity_internal(tr::now);
 				default:
 					Unexpected("scansType in GetDocumentScheme:Identity.");
 				}
@@ -206,7 +209,7 @@ EditDocumentScheme GetDocumentScheme(
 				ValueClass::Fields,
 				PanelDetailsType::Text,
 				qsl("first_name"),
-				lang(lng_passport_first_name),
+				tr::lng_passport_first_name(tr::now),
 				NameValidate,
 				DontFormat,
 				kMaxNameSize,
@@ -215,7 +218,7 @@ EditDocumentScheme GetDocumentScheme(
 				ValueClass::Fields,
 				PanelDetailsType::Text,
 				qsl("middle_name"),
-				lang(lng_passport_middle_name),
+				tr::lng_passport_middle_name(tr::now),
 				NameOrEmptyValidate,
 				DontFormat,
 				kMaxNameSize,
@@ -225,7 +228,7 @@ EditDocumentScheme GetDocumentScheme(
 				ValueClass::Fields,
 				PanelDetailsType::Text,
 				qsl("last_name"),
-				lang(lng_passport_last_name),
+				tr::lng_passport_last_name(tr::now),
 				NameValidate,
 				DontFormat,
 				kMaxNameSize,
@@ -235,7 +238,7 @@ EditDocumentScheme GetDocumentScheme(
 				ValueClass::Fields,
 				PanelDetailsType::Date,
 				qsl("birth_date"),
-				lang(lng_passport_birth_date),
+				tr::lng_passport_birth_date(tr::now),
 				DateValidate,
 				DontFormat,
 			},
@@ -243,7 +246,7 @@ EditDocumentScheme GetDocumentScheme(
 				ValueClass::Fields,
 				PanelDetailsType::Gender,
 				qsl("gender"),
-				lang(lng_passport_gender),
+				tr::lng_passport_gender(tr::now),
 				GenderValidate,
 				GenderFormat,
 			},
@@ -251,7 +254,7 @@ EditDocumentScheme GetDocumentScheme(
 				ValueClass::Fields,
 				PanelDetailsType::Country,
 				qsl("country_code"),
-				lang(lng_passport_country),
+				tr::lng_passport_country(tr::now),
 				CountryValidate,
 				CountryFormat,
 			},
@@ -259,7 +262,7 @@ EditDocumentScheme GetDocumentScheme(
 				ValueClass::Fields,
 				PanelDetailsType::Country,
 				qsl("residence_country_code"),
-				lang(lng_passport_residence_country),
+				tr::lng_passport_residence_country(tr::now),
 				CountryValidate,
 				CountryFormat,
 			},
@@ -267,7 +270,7 @@ EditDocumentScheme GetDocumentScheme(
 				ValueClass::Scans,
 				PanelDetailsType::Text,
 				qsl("document_no"),
-				lang(lng_passport_document_number),
+				tr::lng_passport_document_number(tr::now),
 				DocumentValidate,
 				DontFormat,
 				kMaxDocumentSize,
@@ -276,7 +279,7 @@ EditDocumentScheme GetDocumentScheme(
 				ValueClass::Scans,
 				PanelDetailsType::Date,
 				qsl("expiry_date"),
-				lang(lng_passport_expiry_date),
+				tr::lng_passport_expiry_date(tr::now),
 				DateOrEmptyValidate,
 				DontFormat,
 			},
@@ -300,19 +303,21 @@ EditDocumentScheme GetDocumentScheme(
 			result.additionalHeader = [=](const QString &countryCode) {
 				const auto language = languageValue(countryCode);
 				return language.isEmpty()
-					? lang(lng_passport_native_name_title)
-					: lng_passport_native_name_language(
+					? tr::lng_passport_native_name_title(tr::now)
+					: tr::lng_passport_native_name_language(
+						tr::now,
 						lt_language,
 						language);
 			};
 			result.additionalDescription = [=](const QString &countryCode) {
 				const auto language = languageValue(countryCode);
 				if (!language.isEmpty()) {
-					return lang(lng_passport_native_name_language_about);
+					return tr::lng_passport_native_name_language_about(tr::now);
 				}
-				const auto name = CountrySelectBox::NameByISO(countryCode);
+				const auto name = Data::CountryNameByISO2(countryCode);
 				Assert(!name.isEmpty());
-				return lng_passport_native_name_about(
+				return tr::lng_passport_native_name_about(
+					tr::now,
 					lt_country,
 					name);
 			};
@@ -336,7 +341,7 @@ EditDocumentScheme GetDocumentScheme(
 					ValueClass::Additional,
 					PanelDetailsType::Text,
 					qsl("first_name_native"),
-					lang(lng_passport_first_name),
+					tr::lng_passport_first_name(tr::now),
 					NativeNameValidate,
 					DontFormat,
 					kMaxNameSize,
@@ -347,7 +352,7 @@ EditDocumentScheme GetDocumentScheme(
 					ValueClass::Additional,
 					PanelDetailsType::Text,
 					qsl("middle_name_native"),
-					lang(lng_passport_middle_name),
+					tr::lng_passport_middle_name(tr::now),
 					NativeNameOrEmptyValidate,
 					DontFormat,
 					kMaxNameSize,
@@ -358,7 +363,7 @@ EditDocumentScheme GetDocumentScheme(
 					ValueClass::Additional,
 					PanelDetailsType::Text,
 					qsl("last_name_native"),
-					lang(lng_passport_last_name),
+					tr::lng_passport_last_name(tr::now),
 					NativeNameValidate,
 					DontFormat,
 					kMaxNameSize,
@@ -376,23 +381,23 @@ EditDocumentScheme GetDocumentScheme(
 	case Scope::Type::AddressDetails:
 	case Scope::Type::Address: {
 		auto result = Scheme();
-		result.detailsHeader = lang(lng_passport_address);
+		result.detailsHeader = tr::lng_passport_address(tr::now);
 		if (scansType) {
 			switch (*scansType) {
 			case Value::Type::UtilityBill:
-				result.scansHeader = lang(lng_passport_address_bill);
+				result.scansHeader = tr::lng_passport_address_bill(tr::now);
 				break;
 			case Value::Type::BankStatement:
-				result.scansHeader = lang(lng_passport_address_statement);
+				result.scansHeader = tr::lng_passport_address_statement(tr::now);
 				break;
 			case Value::Type::RentalAgreement:
-				result.scansHeader = lang(lng_passport_address_agreement);
+				result.scansHeader = tr::lng_passport_address_agreement(tr::now);
 				break;
 			case Value::Type::PassportRegistration:
-				result.scansHeader = lang(lng_passport_address_registration);
+				result.scansHeader = tr::lng_passport_address_registration(tr::now);
 				break;
 			case Value::Type::TemporaryRegistration:
-				result.scansHeader = lang(lng_passport_address_temporary);
+				result.scansHeader = tr::lng_passport_address_temporary(tr::now);
 				break;
 			default:
 				Unexpected("scansType in GetDocumentScheme:Address.");
@@ -403,7 +408,7 @@ EditDocumentScheme GetDocumentScheme(
 				ValueClass::Fields,
 				PanelDetailsType::Text,
 				qsl("street_line1"),
-				lang(lng_passport_street),
+				tr::lng_passport_street(tr::now),
 				StreetValidate,
 				DontFormat,
 				kMaxStreetSize,
@@ -412,7 +417,7 @@ EditDocumentScheme GetDocumentScheme(
 				ValueClass::Fields,
 				PanelDetailsType::Text,
 				qsl("street_line2"),
-				lang(lng_passport_street),
+				tr::lng_passport_street(tr::now),
 				DontValidate,
 				DontFormat,
 				kMaxStreetSize,
@@ -421,7 +426,7 @@ EditDocumentScheme GetDocumentScheme(
 				ValueClass::Fields,
 				PanelDetailsType::Text,
 				qsl("city"),
-				lang(lng_passport_city),
+				tr::lng_passport_city(tr::now),
 				CityValidate,
 				DontFormat,
 				kMaxStreetSize,
@@ -430,7 +435,7 @@ EditDocumentScheme GetDocumentScheme(
 				ValueClass::Fields,
 				PanelDetailsType::Text,
 				qsl("state"),
-				lang(lng_passport_state),
+				tr::lng_passport_state(tr::now),
 				DontValidate,
 				DontFormat,
 				kMaxStreetSize,
@@ -439,7 +444,7 @@ EditDocumentScheme GetDocumentScheme(
 				ValueClass::Fields,
 				PanelDetailsType::Country,
 				qsl("country_code"),
-				lang(lng_passport_country),
+				tr::lng_passport_country(tr::now),
 				CountryValidate,
 				CountryFormat,
 			},
@@ -447,7 +452,7 @@ EditDocumentScheme GetDocumentScheme(
 				ValueClass::Fields,
 				PanelDetailsType::Postcode,
 				qsl("post_code"),
-				lang(lng_passport_postcode),
+				tr::lng_passport_postcode(tr::now),
 				PostcodeValidate,
 				DontFormat,
 				kMaxPostcodeSize,
@@ -466,9 +471,9 @@ EditContactScheme GetContactScheme(Scope::Type type) {
 	switch (type) {
 	case Scope::Type::Phone: {
 		auto result = Scheme(ValueType::Phone);
-		result.aboutExisting = lang(lng_passport_use_existing_phone);
-		result.newHeader = lang(lng_passport_new_phone);
-		result.aboutNew = lang(lng_passport_new_phone_code);
+		result.aboutExisting = tr::lng_passport_use_existing_phone(tr::now);
+		result.newHeader = tr::lng_passport_new_phone(tr::now);
+		result.aboutNew = tr::lng_passport_new_phone_code(tr::now);
 		result.validate = [](const QString &value) {
 			return QRegularExpression(
 				"^\\d{2,12}$"
@@ -485,10 +490,10 @@ EditContactScheme GetContactScheme(Scope::Type type) {
 
 	case Scope::Type::Email: {
 		auto result = Scheme(ValueType::Text);
-		result.aboutExisting = lang(lng_passport_use_existing_email);
-		result.newHeader = lang(lng_passport_new_email);
-		result.newPlaceholder = langFactory(lng_passport_email_title);
-		result.aboutNew = lang(lng_passport_new_email_code);
+		result.aboutExisting = tr::lng_passport_use_existing_email(tr::now);
+		result.newHeader = tr::lng_passport_new_email(tr::now);
+		result.newPlaceholder = tr::lng_passport_email_title();
+		result.aboutNew = tr::lng_passport_new_email_code(tr::now);
 		result.validate = [](const QString &value) {
 			const auto at = value.indexOf('@');
 			const auto dot = value.lastIndexOf('.');
@@ -712,16 +717,10 @@ void PanelController::setupPassword() {
 		return;
 	}
 
-	const auto hasRecovery = false;
-	const auto notEmptyPassport = false;
-	const auto hint = QString();
-	auto box = show(Box<PasscodeBox>(
-		Core::CloudPasswordCheckRequest(), // current
-		settings.newAlgo,
-		hasRecovery,
-		notEmptyPassport,
-		hint,
-		settings.newSecureAlgo));
+	auto fields = PasscodeBox::CloudFields();
+	fields.newAlgo = settings.newAlgo;
+	fields.newSecureSecretAlgo = settings.newSecureAlgo;
+	auto box = show(Box<PasscodeBox>(fields));
 	box->newPasswordSet(
 	) | rpl::filter([=](const QByteArray &password) {
 		return !password.isEmpty();
@@ -748,8 +747,8 @@ void PanelController::setupPassword() {
 void PanelController::cancelPasswordSubmit() {
 	const auto box = std::make_shared<QPointer<BoxContent>>();
 	*box = show(Box<ConfirmBox>(
-		lang(lng_passport_stop_password_sure),
-		lang(lng_passport_stop),
+		tr::lng_passport_stop_password_sure(tr::now),
+		tr::lng_passport_stop(tr::now),
 		[=] { if (*box) (*box)->closeBox(); _form->cancelPassword(); }));
 }
 
@@ -835,21 +834,21 @@ auto PanelController::deleteValueLabel() const
 	Expects(_editScope != nullptr);
 
 	if (hasValueDocument()) {
-		return Lang::Viewer(lng_passport_delete_document);
+		return tr::lng_passport_delete_document();
 	} else if (!hasValueFields()) {
 		return std::nullopt;
 	}
 	switch (_editScope->type) {
 	case Scope::Type::PersonalDetails:
 	case Scope::Type::Identity:
-		return Lang::Viewer(lng_passport_delete_details);
+		return tr::lng_passport_delete_details();
 	case Scope::Type::AddressDetails:
 	case Scope::Type::Address:
-		return Lang::Viewer(lng_passport_delete_address);
+		return tr::lng_passport_delete_address();
 	case Scope::Type::Email:
-		return Lang::Viewer(lng_passport_delete_email);
+		return tr::lng_passport_delete_email();
 	case Scope::Type::Phone:
-		return Lang::Viewer(lng_passport_delete_phone);
+		return tr::lng_passport_delete_phone();
 	}
 	Unexpected("Type in PanelController::deleteValueLabel.");
 }
@@ -880,26 +879,26 @@ void PanelController::deleteValue() {
 	const auto text = [&] {
 		switch (_editScope->type) {
 		case Scope::Type::PersonalDetails:
-			return lang(lng_passport_delete_details_sure);
+			return tr::lng_passport_delete_details_sure(tr::now);
 		case Scope::Type::Identity:
-			return lang(lng_passport_delete_document_sure);
+			return tr::lng_passport_delete_document_sure(tr::now);
 		case Scope::Type::AddressDetails:
-			return lang(lng_passport_delete_address_sure);
+			return tr::lng_passport_delete_address_sure(tr::now);
 		case Scope::Type::Address:
-			return lang(lng_passport_delete_document_sure);
+			return tr::lng_passport_delete_document_sure(tr::now);
 		case Scope::Type::Phone:
-			return lang(lng_passport_delete_phone_sure);
+			return tr::lng_passport_delete_phone_sure(tr::now);
 		case Scope::Type::Email:
-			return lang(lng_passport_delete_email_sure);
+			return tr::lng_passport_delete_email_sure(tr::now);
 		}
 		Unexpected("Type in deleteValue.");
 	}();
 	const auto checkbox = (hasValueDocument() && hasValueFields()) ? [&] {
 		switch (_editScope->type) {
 		case Scope::Type::Identity:
-			return lang(lng_passport_delete_details);
+			return tr::lng_passport_delete_details(tr::now);
 		case Scope::Type::Address:
-			return lang(lng_passport_delete_address);
+			return tr::lng_passport_delete_address(tr::now);
 		}
 		Unexpected("Type in deleteValue.");
 	}() : QString();
@@ -978,8 +977,8 @@ void PanelController::showUpdateAppBox() {
 	};
 	show(
 		Box<ConfirmBox>(
-			lang(lng_passport_app_out_of_date),
-			lang(lng_menu_update),
+			tr::lng_passport_app_out_of_date(tr::now),
+			tr::lng_menu_update(tr::now),
 			callback,
 			[=] { _form->cancelSure(); }),
 		LayerOption::KeepOther,
@@ -1044,13 +1043,13 @@ void PanelController::requestScopeFilesType(int index) {
 				}) | ranges::view::transform([](Value::Type type) {
 					switch (type) {
 					case Value::Type::Passport:
-						return lang(lng_passport_identity_passport);
+						return tr::lng_passport_identity_passport(tr::now);
 					case Value::Type::IdentityCard:
-						return lang(lng_passport_identity_card);
+						return tr::lng_passport_identity_card(tr::now);
 					case Value::Type::DriverLicense:
-						return lang(lng_passport_identity_license);
+						return tr::lng_passport_identity_license(tr::now);
 					case Value::Type::InternalPassport:
-						return lang(lng_passport_identity_internal);
+						return tr::lng_passport_identity_internal(tr::now);
 					default:
 						Unexpected("IdentityType in requestScopeFilesType");
 					}
@@ -1067,15 +1066,15 @@ void PanelController::requestScopeFilesType(int index) {
 				}) | ranges::view::transform([](Value::Type type) {
 					switch (type) {
 					case Value::Type::UtilityBill:
-						return lang(lng_passport_address_bill);
+						return tr::lng_passport_address_bill(tr::now);
 					case Value::Type::BankStatement:
-						return lang(lng_passport_address_statement);
+						return tr::lng_passport_address_statement(tr::now);
 					case Value::Type::RentalAgreement:
-						return lang(lng_passport_address_agreement);
+						return tr::lng_passport_address_agreement(tr::now);
 					case Value::Type::PassportRegistration:
-						return lang(lng_passport_address_registration);
+						return tr::lng_passport_address_registration(tr::now);
 					case Value::Type::TemporaryRegistration:
-						return lang(lng_passport_address_temporary);
+						return tr::lng_passport_address_temporary(tr::now);
 					default:
 						Unexpected("AddressType in requestScopeFilesType");
 					}
@@ -1115,11 +1114,11 @@ void PanelController::readScanError(ReadScanError error) {
 	show(Box<InformBox>([&] {
 		switch (error) {
 		case ReadScanError::FileTooLarge:
-			return lang(lng_passport_error_too_large);
+			return tr::lng_passport_error_too_large(tr::now);
 		case ReadScanError::BadImageSize:
-			return lang(lng_passport_error_bad_size);
+			return tr::lng_passport_error_bad_size(tr::now);
 		case ReadScanError::CantReadImage:
-			return lang(lng_passport_error_cant_read);
+			return tr::lng_passport_error_cant_read(tr::now);
 		case ReadScanError::Unknown:
 			return Lang::Hard::UnknownSecureScanError();
 		}
@@ -1366,7 +1365,7 @@ void PanelController::processVerificationNeeded(
 					return field->verification.error;
 				}) | rpl::distinct_until_changed(),
 
-				rpl::never<QString>()));
+				nullptr));
 		} else {
 			Unexpected("Type in processVerificationNeeded.");
 		}
@@ -1397,7 +1396,7 @@ void PanelController::saveScope(ValueMap &&data, ValueMap &&filesData) {
 	Expects(_panel != nullptr);
 
 	if (uploadingScopeScan()) {
-		showToast(lang(lng_passport_wait_upload));
+		showToast(tr::lng_passport_wait_upload(tr::now));
 		return;
 	} else if (savingScope()) {
 		return;
@@ -1432,8 +1431,8 @@ void PanelController::cancelEditScope() {
 	if (_panelHasUnsavedChanges && _panelHasUnsavedChanges()) {
 		if (!_confirmForgetChangesBox) {
 			_confirmForgetChangesBox = show(Box<ConfirmBox>(
-				lang(lng_passport_sure_cancel),
-				lang(lng_continue),
+				tr::lng_passport_sure_cancel(tr::now),
+				tr::lng_continue(tr::now),
 				[=] { _panel->showForm(); }));
 			_editScopeBoxes.emplace_back(_confirmForgetChangesBox);
 		}
