@@ -18,7 +18,7 @@ class MainWindow : public Window::MainWindow {
 	Q_OBJECT
 
 public:
-	MainWindow();
+	explicit MainWindow(not_null<Window::Controller*> controller);
 
 	void psFirstShow();
 	void psInitSysMenu();
@@ -37,6 +37,8 @@ public:
 
 	~MainWindow();
 
+	void updateWindowIcon() override;
+
 	class Private;
 
 public slots:
@@ -56,7 +58,6 @@ protected:
 	void handleActiveChangedHook() override;
 	void stateChangedHook(Qt::WindowState state) override;
 	void initHook() override;
-	void updateWindowIcon() override;
 	void titleVisibilityChangedHook() override;
 	void unreadCounterChangedHook() override;
 
@@ -83,12 +84,14 @@ protected:
 	void closeWithoutDestroy() override;
 
 private:
+	friend class Private;
+
+	void initTouchBar();
 	void hideAndDeactivate();
 	void createGlobalMenu();
 	void updateTitleCounter();
 	void updateIconCounters();
 
-	friend class Private;
 	std::unique_ptr<Private> _private;
 
 	mutable bool psIdle;
