@@ -157,16 +157,7 @@ DeclareSetting(QString, StartUrl);
 DeclareSetting(float64, RetinaFactor);
 DeclareSetting(int32, IntRetinaFactor);
 
-DeclareReadSetting(DBIPlatform, Platform);
-DeclareReadSetting(QString, PlatformString);
-DeclareReadSetting(bool, IsElCapitan);
-DeclareReadSetting(bool, IsSnowLeopard);
-
 DeclareSetting(int, OtherOnline);
-
-typedef QMap<uint64, DBIPeerReportSpamStatus> ReportSpamStatuses;
-DeclareRefSetting(ReportSpamStatuses, ReportSpamStatuses);
-
 DeclareSetting(bool, AutoPlayGif);
 
 constexpr auto kInterfaceScaleAuto = 0;
