@@ -69,7 +69,7 @@ void Calls::setupContent() {
 
 	const auto currentOutputName = [&] {
 		if (Global::CallOutputDeviceID() == qsl("default")) {
-			return lang(lng_settings_call_device_default);
+			return tr::lng_settings_call_device_default(tr::now);
 		}
 		const auto &list = VoIPController::EnumerateAudioOutputs();
 		const auto i = ranges::find(
@@ -83,7 +83,7 @@ void Calls::setupContent() {
 
 	const auto currentInputName = [&] {
 		if (Global::CallInputDeviceID() == qsl("default")) {
-			return lang(lng_settings_call_device_default);
+			return tr::lng_settings_call_device_default(tr::now);
 		}
 		const auto &list = VoIPController::EnumerateAudioInputs();
 		const auto i = ranges::find(
@@ -96,10 +96,10 @@ void Calls::setupContent() {
 	}();
 
 	AddSkip(content);
-	AddSubsectionTitle(content, lng_settings_call_section_output);
+	AddSubsectionTitle(content, tr::lng_settings_call_section_output());
 	AddButtonWithLabel(
 		content,
-		lng_settings_call_output_device,
+		tr::lng_settings_call_output_device(),
 		rpl::single(
 			currentOutputName
 		) | rpl::then(
@@ -109,7 +109,7 @@ void Calls::setupContent() {
 	)->addClickHandler([=] {
 		const auto &devices = VoIPController::EnumerateAudioOutputs();
 		const auto options = ranges::view::concat(
-			ranges::view::single(lang(lng_settings_call_device_default)),
+			ranges::view::single(tr::lng_settings_call_device_default(tr::now)),
 			devices | ranges::view::transform(getName)
 		) | ranges::to_vector;
 		const auto i = ranges::find(
@@ -131,7 +131,7 @@ void Calls::setupContent() {
 			}
 		});
 		Ui::show(Box<SingleChoiceBox>(
-			lng_settings_call_output_device,
+			tr::lng_settings_call_output_device(),
 			options,
 			currentOption,
 			save));
@@ -150,7 +150,7 @@ void Calls::setupContent() {
 	const auto updateOutputLabel = [=](int value) {
 		const auto percent = QString::number(value);
 		outputLabel->setText(
-			lng_settings_call_output_volume(lt_percent, percent));
+			tr::lng_settings_call_output_volume(tr::now, lt_percent, percent));
 	};
 	const auto updateOutputVolume = [=](int value) {
 		_needWriteSettings = true;
@@ -171,10 +171,10 @@ void Calls::setupContent() {
 	AddSkip(content);
 	AddDivider(content);
 	AddSkip(content);
-	AddSubsectionTitle(content, lng_settings_call_section_input);
+	AddSubsectionTitle(content, tr::lng_settings_call_section_input());
 	AddButtonWithLabel(
 		content,
-		lng_settings_call_input_device,
+		tr::lng_settings_call_input_device(),
 		rpl::single(
 			currentInputName
 		) | rpl::then(
@@ -184,7 +184,7 @@ void Calls::setupContent() {
 	)->addClickHandler([=] {
 		const auto &devices = VoIPController::EnumerateAudioInputs();
 		const auto options = ranges::view::concat(
-			ranges::view::single(lang(lng_settings_call_device_default)),
+			ranges::view::single(tr::lng_settings_call_device_default(tr::now)),
 			devices | ranges::view::transform(getName)
 		) | ranges::to_vector;
 		const auto i = ranges::find(
@@ -209,7 +209,7 @@ void Calls::setupContent() {
 			}
 		});
 		Ui::show(Box<SingleChoiceBox>(
-			lng_settings_call_input_device,
+			tr::lng_settings_call_input_device(),
 			options,
 			currentOption,
 			save));
@@ -228,7 +228,7 @@ void Calls::setupContent() {
 	const auto updateInputLabel = [=](int value) {
 		const auto percent = QString::number(value);
 		inputLabel->setText(
-			lng_settings_call_input_volume(lt_percent, percent));
+			tr::lng_settings_call_input_volume(tr::now, lt_percent, percent));
 	};
 	const auto updateInputVolume = [=](int value) {
 		_needWriteSettings = true;
@@ -249,7 +249,7 @@ void Calls::setupContent() {
 	AddButton(
 		content,
 		rpl::single(
-			lang(lng_settings_call_test_mic)
+			tr::lng_settings_call_test_mic(tr::now)
 		) | rpl::then(
 			_micTestTextStream.events()
 		),
@@ -276,12 +276,12 @@ void Calls::setupContent() {
 	AddSkip(content);
 	AddDivider(content);
 	AddSkip(content);
-	AddSubsectionTitle(content, lng_settings_call_section_other);
+	AddSubsectionTitle(content, tr::lng_settings_call_section_other());
 
 #if defined Q_OS_MAC && !defined OS_MAC_STORE
 	AddButton(
 		content,
-		lng_settings_call_audio_ducking,
+		tr::lng_settings_call_audio_ducking(),
 		st::settingsButton
 	)->toggleOn(
 		rpl::single(Global::CallAudioDuckingEnabled())
@@ -298,13 +298,13 @@ void Calls::setupContent() {
 
 	AddButton(
 		content,
-		lng_settings_call_open_system_prefs,
+		tr::lng_settings_call_open_system_prefs(),
 		st::settingsButton
 	)->addClickHandler([] {
 		const auto opened = Platform::OpenSystemSettings(
 			Platform::SystemSettingsType::Audio);
 		if (!opened) {
-			Ui::show(Box<InformBox>(lang(lng_linux_no_audio_prefs)));
+			Ui::show(Box<InformBox>(tr::lng_linux_no_audio_prefs(tr::now)));
 		}
 	});
 	AddSkip(content);
@@ -336,25 +336,25 @@ void Calls::requestPermissionAndStartTestingMicrophone() {
 			Ui::hideLayer();
 		};
 		Ui::show(Box<ConfirmBox>(
-			lang(lng_no_mic_permission),
-			lang(lng_menu_settings),
+			tr::lng_no_mic_permission(tr::now),
+			tr::lng_menu_settings(tr::now),
 			showSystemSettings));
 	}
 }
 
 void Calls::startTestingMicrophone() {
-	_micTestTextStream.fire(lang(lng_settings_call_stop_mic_test));
+	_micTestTextStream.fire(tr::lng_settings_call_stop_mic_test(tr::now));
 	_levelUpdateTimer.callEach(50);
 	_micTester = std::make_unique<tgvoip::AudioInputTester>(
 		Global::CallInputDeviceID().toStdString());
 	if (_micTester->Failed()) {
 		stopTestingMicrophone();
-		Ui::show(Box<InformBox>(lang(lng_call_error_audio_io)));
+		Ui::show(Box<InformBox>(tr::lng_call_error_audio_io(tr::now)));
 	}
 }
 
 void Calls::stopTestingMicrophone() {
-	_micTestTextStream.fire(lang(lng_settings_call_test_mic));
+	_micTestTextStream.fire(tr::lng_settings_call_test_mic(tr::now));
 	_levelUpdateTimer.cancel();
 	_micTester.reset();
 	_micTestLevel->setValue(0.0f);
