@@ -9,11 +9,15 @@ https://github.com/telegramdesktop/tdesktop/blob/master/LEGAL
 
 #include "storage/localstorage.h"
 #include "platform/platform_window_title.h"
+#include "platform/platform_info.h"
 #include "history/history.h"
 #include "window/themes/window_theme.h"
-#include "window/window_controller.h"
+#include "window/window_session_controller.h"
 #include "window/window_lock_widgets.h"
+#include "window/window_outdated_bar.h"
+#include "window/window_controller.h"
 #include "boxes/confirm_box.h"
+#include "main/main_account.h" // Account::authSessionValue.
 #include "core/click_handler_types.h"
 #include "core/application.h"
 #include "core/sandbox.h"
@@ -26,10 +30,13 @@ https://github.com/telegramdesktop/tdesktop/blob/master/LEGAL
 #include "styles/style_boxes.h"
 
 namespace Window {
+namespace {
 
 constexpr auto kInactivePressTimeout = crl::time(200);
 constexpr auto kSaveWindowPositionTimeout = crl::time(1000);
 
+} // namespace
+
 QImage LoadLogo() {
 	return QImage(qsl(":/gui/art/logo_256.png"));
 }
@@ -86,26 +93,29 @@ void ConvertIconToBlack(QImage &image) {
 	}
 }
 
-QIcon CreateOfficialIcon() {
+QIcon CreateOfficialIcon(Main::Account *account) {
 	auto image = Core::IsAppLaunched() ? Core::App().logo() : LoadLogo();
-	if (AuthSession::Exists() && Auth().supportMode()) {
+	if (account
+		&& account->sessionExists()
+		&& account->session().supportMode()) {
 		ConvertIconToBlack(image);
 	}
 	return QIcon(App::pixmapFromImageInPlace(std::move(image)));
 }
 
-QIcon CreateIcon() {
-	auto result = CreateOfficialIcon();
-	if (cPlatform() == dbipLinux32 || cPlatform() == dbipLinux64) {
+QIcon CreateIcon(Main::Account *account) {
+	auto result = CreateOfficialIcon(account);
+	if (Platform::IsLinux()) {
 		return QIcon::fromTheme("telegram", result);
 	}
 	return result;
 }
 
-MainWindow::MainWindow()
-: _positionUpdatedTimer([=] { savePosition(); })
+MainWindow::MainWindow(not_null<Controller*> controller)
+: _controller(controller)
+, _positionUpdatedTimer([=] { savePosition(); })
+, _outdated(CreateOutdatedBar(this))
 , _body(this)
-, _icon(CreateIcon())
 , _titleText(qsl("Telegram")) {
 	subscribe(Theme::Background(), [=](
 			const Theme::BackgroundUpdate &data) {
@@ -119,24 +129,39 @@ MainWindow::MainWindow()
 	subscribe(Global::RefWorkMode(), [=](DBIWorkMode mode) {
 		workmodeUpdated(mode);
 	});
-	subscribe(Core::App().authSessionChanged(), [=] {
-		checkAuthSession();
-		updateWindowIcon();
-	});
-	checkAuthSession();
 
 	Core::App().termsLockValue(
 	) | rpl::start_with_next([=] {
 		checkLockByTerms();
 	}, lifetime());
 
+	if (_outdated) {
+		_outdated->heightValue(
+		) | rpl::filter([=] {
+			return window()->windowHandle() != nullptr;
+		}) | rpl::start_with_next([=](int height) {
+			if (!height) {
+				crl::on_main(this, [=] { _outdated.destroy(); });
+			}
+			updateControlsGeometry();
+		}, _outdated->lifetime());
+	}
+
 	_isActiveTimer.setCallback([this] { updateIsActive(0); });
 	_inactivePressTimer.setCallback([this] { setInactivePress(false); });
 }
 
+Main::Account &MainWindow::account() const {
+	return _controller->account();
+}
+
+Window::SessionController *MainWindow::sessionController() const {
+	return _controller->sessionController();
+}
+
 void MainWindow::checkLockByTerms() {
 	const auto data = Core::App().termsLocked();
-	if (!data || !AuthSession::Exists()) {
+	if (!data || !account().sessionExists()) {
 		if (_termsBox) {
 			_termsBox->closeBox();
 		}
@@ -145,8 +170,8 @@ void MainWindow::checkLockByTerms() {
 	Ui::hideSettingsAndLayer(anim::type::instant);
 	const auto box = Ui::show(Box<TermsBox>(
 		*data,
-		langFactory(lng_terms_agree),
-		langFactory(lng_terms_decline)));
+		tr::lng_terms_agree(),
+		tr::lng_terms_decline()));
 
 	box->setCloseByEscape(false);
 	box->setCloseByOutsideClick(false);
@@ -155,8 +180,8 @@ void MainWindow::checkLockByTerms() {
 	box->agreeClicks(
 	) | rpl::start_with_next([=] {
 		const auto mention = box ? box->lastClickedMention() : QString();
-		if (AuthSession::Exists()) {
-			Auth().api().acceptTerms(id);
+		if (account().sessionExists()) {
+			account().session().api().acceptTerms(id);
 			if (!mention.isEmpty()) {
 				MentionClickHandler(mention).onClick({});
 			}
@@ -179,9 +204,9 @@ void MainWindow::checkLockByTerms() {
 void MainWindow::showTermsDecline() {
 	const auto box = Ui::show(
 		Box<Window::TermsBox>(
-			TextWithEntities{ lang(lng_terms_update_sorry) },
-			langFactory(lng_terms_decline_and_delete),
-			langFactory(lng_terms_back),
+			TextWithEntities{ tr::lng_terms_update_sorry(tr::now) },
+			tr::lng_terms_decline_and_delete(),
+			tr::lng_terms_back(),
 			true),
 		LayerOption::KeepOther);
 
@@ -204,16 +229,16 @@ void MainWindow::showTermsDecline() {
 void MainWindow::showTermsDelete() {
 	const auto box = std::make_shared<QPointer<BoxContent>>();
 	const auto deleteByTerms = [=] {
-		if (AuthSession::Exists()) {
-			Auth().termsDeleteNow();
+		if (account().sessionExists()) {
+			account().session().termsDeleteNow();
 		} else {
 			Ui::hideLayer();
 		}
 	};
 	*box = Ui::show(
 		Box<ConfirmBox>(
-			lang(lng_terms_delete_warning),
-			lang(lng_terms_delete_now),
+			tr::lng_terms_delete_warning(tr::now),
+			tr::lng_terms_delete_now(tr::now),
 			st::attentionBoxButton,
 			deleteByTerms,
 			[=] { if (*box) (*box)->closeBox(); }),
@@ -229,7 +254,7 @@ bool MainWindow::hideNoQuit() {
 			Ui::showChatsList();
 			return true;
 		}
-	} else if (cPlatform() == dbipMac || cPlatform() == dbipMacOld) {
+	} else if (Platform::IsMac()) {
 		closeWithoutDestroy();
 		updateIsActive(Global::OfflineBlurTimeout());
 		updateGlobalMenu();
@@ -258,9 +283,10 @@ bool MainWindow::computeIsActive() const {
 }
 
 void MainWindow::updateWindowIcon() {
-	const auto supportIcon = AuthSession::Exists() && Auth().supportMode();
-	if (supportIcon != _usingSupportIcon) {
-		_icon = CreateIcon();
+	const auto supportIcon = account().sessionExists()
+		&& account().session().supportMode();
+	if (supportIcon != _usingSupportIcon || _icon.isNull()) {
+		_icon = CreateIcon(&account());
 		_usingSupportIcon = supportIcon;
 	}
 	setWindowIcon(_icon);
@@ -335,9 +361,21 @@ HitTestResult MainWindow::hitTest(const QPoint &p) const {
 	return Window::HitTestResult::None;
 }
 
+int MainWindow::computeMinHeight() const {
+	const auto title = _title ? _title->height() : 0;
+	const auto outdated = [&] {
+		if (!_outdated) {
+			return 0;
+		}
+		_outdated->resizeToWidth(st::windowMinWidth);
+		return _outdated->height();
+	}();
+	return title + outdated + st::windowMinHeight;
+}
+
 void MainWindow::initSize() {
 	setMinimumWidth(st::windowMinWidth);
-	setMinimumHeight((_title ? _title->height() : 0) + st::windowMinHeight);
+	setMinimumHeight(computeMinHeight());
 
 	auto position = cWindowPos();
 	DEBUG_LOG(("Window Pos: Initializing first %1, %2, %3, %4 (maximized %5)").arg(position.x).arg(position.y).arg(position.w).arg(position.h).arg(Logs::b(position.maximized)));
@@ -437,6 +475,12 @@ void MainWindow::updateControlsGeometry() {
 		_title->setGeometry(0, bodyTop, width(), _title->height());
 		bodyTop += _title->height();
 	}
+	if (_outdated) {
+		Ui::SendPendingMoveResizeEvents(_outdated.data());
+		_outdated->resizeToWidth(width());
+		_outdated->moveToLeft(0, bodyTop);
+		bodyTop += _outdated->height();
+	}
 	if (_rightColumn) {
 		bodyWidth -= _rightColumn->width();
 		_rightColumn->setGeometry(bodyWidth, bodyTop, width() - bodyWidth, height() - bodyTop);
@@ -447,8 +491,8 @@ void MainWindow::updateControlsGeometry() {
 void MainWindow::updateUnreadCounter() {
 	if (!Global::started() || App::quitting()) return;
 
-	const auto counter = AuthSession::Exists()
-		? Auth().data().unreadBadge()
+	const auto counter = account().sessionExists()
+		? account().session().data().unreadBadge()
 		: 0;
 	_titleText = (counter > 0) ? qsl("Telegram (%1)").arg(counter) : qsl("Telegram");
 
@@ -603,14 +647,6 @@ void MainWindow::launchDrag(std::unique_ptr<QMimeData> data) {
 	}
 }
 
-void MainWindow::checkAuthSession() {
-	if (AuthSession::Exists()) {
-		_controller = std::make_unique<Window::Controller>(&Auth(), this);
-	} else {
-		_controller = nullptr;
-	}
-}
-
 void MainWindow::setInactivePress(bool inactive) {
 	_wasInactivePress = inactive;
 	if (_wasInactivePress) {
