@@ -13,26 +13,33 @@ https://github.com/telegramdesktop/tdesktop/blob/master/LEGAL
 
 class BoxContent;
 
+namespace Main {
+class Account;
+} // namespace Main
+
 namespace Window {
 
 class Controller;
+class SessionController;
 class TitleWidget;
 struct TermsLock;
 
 QImage LoadLogo();
 QImage LoadLogoNoMargin();
-QIcon CreateIcon();
+QIcon CreateIcon(Main::Account *account = nullptr);
 void ConvertIconToBlack(QImage &image);
 
 class MainWindow : public Ui::RpWidget, protected base::Subscriber {
 	Q_OBJECT
 
 public:
-	MainWindow();
+	explicit MainWindow(not_null<Controller*> controller);
 
-	Window::Controller *controller() const {
-		return _controller.get();
+	Window::Controller &controller() const {
+		return *_controller;
 	}
+	Main::Account &account() const;
+	Window::SessionController *sessionController() const;
 	void setInactivePress(bool inactive);
 	bool wasInactivePress() const {
 		return _wasInactivePress;
@@ -83,6 +90,8 @@ public:
 
 	rpl::producer<> leaveEvents() const;
 
+	virtual void updateWindowIcon();
+
 public slots:
 	bool minimizeToTray();
 	void updateGlobalMenu() {
@@ -110,8 +119,6 @@ protected:
 	virtual void clearWidgetsHook() {
 	}
 
-	virtual void updateWindowIcon();
-
 	virtual void stateChangedHook(Qt::WindowState state) {
 	}
 
@@ -148,7 +155,6 @@ protected:
 		QSystemTrayIcon::ActivationReason reason) = 0;
 
 private:
-	void checkAuthSession();
 	void updatePalette();
 	void updateUnreadCounter();
 	void initSize();
@@ -158,11 +164,15 @@ private:
 	void showTermsDecline();
 	void showTermsDelete();
 
+	int computeMinHeight() const;
+
+	not_null<Window::Controller*> _controller;
+
 	base::Timer _positionUpdatedTimer;
 	bool _positionInited = false;
 
-	std::unique_ptr<Window::Controller> _controller;
 	object_ptr<TitleWidget> _title = { nullptr };
+	object_ptr<Ui::RpWidget> _outdated;
 	object_ptr<TWidget> _body;
 	object_ptr<TWidget> _rightColumn = { nullptr };
 	QPointer<BoxContent> _termsBox;
