@@ -8,10 +8,12 @@ https://github.com/telegramdesktop/tdesktop/blob/master/LEGAL
 #include "window/window_main_menu.h"
 
 #include "window/themes/window_theme.h"
-#include "window/window_controller.h"
+#include "window/window_session_controller.h"
 #include "ui/widgets/buttons.h"
 #include "ui/widgets/labels.h"
 #include "ui/widgets/menu.h"
+#include "ui/widgets/popup_menu.h"
+#include "ui/text/text_utilities.h"
 #include "ui/special_buttons.h"
 #include "ui/empty_userpic.h"
 #include "mainwindow.h"
@@ -26,6 +28,8 @@ https://github.com/telegramdesktop/tdesktop/blob/master/LEGAL
 #include "core/click_handler_types.h"
 #include "observer_peer.h"
 #include "auth_session.h"
+#include "data/data_folder.h"
+#include "data/data_session.h"
 #include "data/data_user.h"
 #include "mainwidget.h"
 #include "styles/style_window.h"
@@ -33,6 +37,30 @@ https://github.com/telegramdesktop/tdesktop/blob/master/LEGAL
 #include "styles/style_settings.h"
 #include "styles/style_boxes.h"
 
+namespace {
+
+constexpr auto kMinDiffIntensity = 0.25;
+
+float64 IntensityOfColor(QColor color) {
+	return (0.299 * color.red()
+			+ 0.587 * color.green()
+			+ 0.114 * color.blue()) / 255.0;
+}
+
+bool IsShadowShown(const QImage &img, const QRect r, float64 intensityText) {
+	for (auto x = r.x(); x < r.x() + r.width(); x++) {
+		for (auto y = r.y(); y < r.y() + r.height(); y++) {
+			const auto intensity = IntensityOfColor(QColor(img.pixel(x, y)));
+			if ((std::abs(intensity - intensityText)) < kMinDiffIntensity) {
+				return true;
+			}
+		}
+	}
+	return false;
+}
+
+} // namespace
+
 namespace Window {
 
 class MainMenu::ResetScaleButton : public Ui::AbstractButton {
@@ -92,7 +120,7 @@ void MainMenu::ResetScaleButton::paintEvent(QPaintEvent *e) {
 
 MainMenu::MainMenu(
 	QWidget *parent,
-	not_null<Controller*> controller)
+	not_null<SessionController*> controller)
 : RpWidget(parent)
 , _controller(controller)
 , _menu(this, st::mainMenu)
@@ -100,9 +128,21 @@ MainMenu::MainMenu(
 , _version(this, st::mainMenuVersionLabel) {
 	setAttribute(Qt::WA_OpaquePaintEvent);
 
-	auto showSelfChat = [] {
+	const auto showSelfChat = [] {
 		App::main()->choosePeer(Auth().userPeerId(), ShowAtUnreadMsgId);
 	};
+	const auto showArchive = [=] {
+		if (const auto folder = Auth().data().folderLoaded(Data::Folder::kId)) {
+			App::wnd()->sessionController()->openFolder(folder);
+			Ui::hideSettingsAndLayer();
+		}
+	};
+	const auto checkArchive = [=] {
+		const auto folder = Auth().data().folderLoaded(Data::Folder::kId);
+		return folder
+			&& !folder->chatsList()->empty()
+			&& _controller->session().settings().archiveInMainMenu();
+	};
 	_userpicButton.create(
 		this,
 		_controller,
@@ -115,6 +155,27 @@ MainMenu::MainMenu(
 	_cloudButton->setClickedCallback(showSelfChat);
 	_cloudButton->show();
 
+	_archiveButton.create(this, st::mainMenuCloudButton);
+	_archiveButton->setHidden(!checkArchive());
+	_archiveButton->setAcceptBoth(true);
+	_archiveButton->clicks(
+	) | rpl::start_with_next([=](Qt::MouseButton which) {
+		if (which == Qt::LeftButton) {
+			showArchive();
+			return;
+		} else if (which != Qt::RightButton) {
+			return;
+		}
+		_contextMenu = base::make_unique_q<Ui::PopupMenu>(this);
+		_contextMenu->addAction(
+			tr::lng_context_archive_to_list(tr::now), [=] {
+			_controller->session().settings().setArchiveInMainMenu(false);
+			_controller->session().saveSettingsDelayed();
+			Ui::hideSettingsAndLayer();
+		});
+		_contextMenu->popup(QCursor::pos());
+	}, _archiveButton->lifetime());
+
 	_nightThemeSwitch.setCallback([this] {
 		if (const auto action = *_nightThemeAction) {
 			const auto nightMode = Window::Theme::IsNightMode();
@@ -130,10 +191,13 @@ MainMenu::MainMenu(
 		emit action->triggered();
 	});
 	refreshMenu();
+	refreshBackground();
 
-	_telegram->setRichText(textcmdLink(1, qsl("Telegram Desktop")));
-	_telegram->setLink(1, std::make_shared<UrlClickHandler>(qsl("https://desktop.telegram.org")));
-	_version->setRichText(textcmdLink(1, lng_settings_current_version(lt_version, currentVersionText())) + QChar(' ') + QChar(8211) + QChar(' ') + textcmdLink(2, lang(lng_menu_about)));
+	_telegram->setMarkedText(Ui::Text::Link(
+		qsl("Telegram Desktop"),
+		qsl("https://desktop.telegram.org")));
+	_telegram->setLinksTrusted();
+	_version->setRichText(textcmdLink(1, tr::lng_settings_current_version(tr::now, lt_version, currentVersionText())) + QChar(' ') + QChar(8211) + QChar(' ') + textcmdLink(2, tr::lng_menu_about(tr::now)));
 	_version->setLink(1, std::make_shared<UrlClickHandler>(qsl("https://desktop.telegram.org/changelog")));
 	_version->setLink(2, std::make_shared<LambdaClickHandler>([] { Ui::show(Box<AboutBox>()); }));
 
@@ -149,7 +213,17 @@ MainMenu::MainMenu(
 		if (update.type == Window::Theme::BackgroundUpdate::Type::ApplyingTheme) {
 			refreshMenu();
 		}
+		if (update.type == Window::Theme::BackgroundUpdate::Type::New) {
+			refreshBackground();
+		}
 	});
+	Auth().data().chatsListChanges(
+	) | rpl::filter([](Data::Folder *folder) {
+		return folder && (folder->id() == Data::Folder::kId);
+	}) | rpl::start_with_next([=](Data::Folder *folder) {
+		_archiveButton->setHidden(!checkArchive());
+		update();
+	}, lifetime());
 	updatePhone();
 	initResetScaleButton();
 }
@@ -157,26 +231,26 @@ MainMenu::MainMenu(
 void MainMenu::refreshMenu() {
 	_menu->clearActions();
 	if (!Auth().supportMode()) {
-		_menu->addAction(lang(lng_create_group_title), [] {
+		_menu->addAction(tr::lng_create_group_title(tr::now), [] {
 			App::wnd()->onShowNewGroup();
 		}, &st::mainMenuNewGroup, &st::mainMenuNewGroupOver);
-		_menu->addAction(lang(lng_create_channel_title), [] {
+		_menu->addAction(tr::lng_create_channel_title(tr::now), [] {
 			App::wnd()->onShowNewChannel();
 		}, &st::mainMenuNewChannel, &st::mainMenuNewChannelOver);
-		_menu->addAction(lang(lng_menu_contacts), [] {
+		_menu->addAction(tr::lng_menu_contacts(tr::now), [] {
 			Ui::show(Box<PeerListBox>(std::make_unique<ContactsBoxController>(), [](not_null<PeerListBox*> box) {
-				box->addButton(langFactory(lng_close), [box] { box->closeBox(); });
-				box->addLeftButton(langFactory(lng_profile_add_contact), [] { App::wnd()->onShowAddContact(); });
+				box->addButton(tr::lng_close(), [box] { box->closeBox(); });
+				box->addLeftButton(tr::lng_profile_add_contact(), [] { App::wnd()->onShowAddContact(); });
 			}));
 		}, &st::mainMenuContacts, &st::mainMenuContactsOver);
 		if (Global::PhoneCallsEnabled()) {
-			_menu->addAction(lang(lng_menu_calls), [] {
+			_menu->addAction(tr::lng_menu_calls(tr::now), [] {
 				Ui::show(Box<PeerListBox>(std::make_unique<Calls::BoxController>(), [](not_null<PeerListBox*> box) {
-					box->addButton(langFactory(lng_close), [=] {
+					box->addButton(tr::lng_close(), [=] {
 						box->closeBox();
 					});
 					box->addTopButton(st::callSettingsButton, [=] {
-						App::wnd()->controller()->showSettings(
+						App::wnd()->sessionController()->showSettings(
 							Settings::Type::Calls,
 							Window::SectionShow(anim::type::instant));
 					});
@@ -184,7 +258,7 @@ void MainMenu::refreshMenu() {
 			}, &st::mainMenuCalls, &st::mainMenuCallsOver);
 		}
 	} else {
-		_menu->addAction(lang(lng_profile_add_contact), [] {
+		_menu->addAction(tr::lng_profile_add_contact(tr::now), [] {
 			App::wnd()->onShowAddContact();
 		}, &st::mainMenuContacts, &st::mainMenuContactsOver);
 
@@ -201,12 +275,12 @@ void MainMenu::refreshMenu() {
 			Auth().supportTemplates().reload();
 		}, &st::mainMenuReload, &st::mainMenuReloadOver);
 	}
-	_menu->addAction(lang(lng_menu_settings), [] {
+	_menu->addAction(tr::lng_menu_settings(tr::now), [] {
 		App::wnd()->showSettings();
 	}, &st::mainMenuSettings, &st::mainMenuSettingsOver);
 
 	_nightThemeAction = std::make_shared<QPointer<QAction>>();
-	auto action = _menu->addAction(lang(lng_menu_night_mode), [=] {
+	auto action = _menu->addAction(tr::lng_menu_night_mode(tr::now), [=] {
 		if (auto action = *_nightThemeAction) {
 			action->setChecked(!action->isChecked());
 			_nightThemeSwitch.callOnce(st::mainMenu.itemToggle.duration);
@@ -220,6 +294,58 @@ void MainMenu::refreshMenu() {
 	updatePhone();
 }
 
+void MainMenu::refreshBackground() {
+	const auto fill = QRect(0, 0, width(), st::mainMenuCoverHeight);
+	const auto intensityText = IntensityOfColor(st::mainMenuCoverFg->c);
+	QImage backgroundImage(
+		st::mainMenuWidth * cIntRetinaFactor(),
+		st::mainMenuCoverHeight * cIntRetinaFactor(),
+		QImage::Format_ARGB32_Premultiplied);
+	QPainter p(&backgroundImage);
+
+	const auto drawShadow = [](QPainter &p) {
+		st::mainMenuShadow.paint(
+			p,
+			0,
+			st::mainMenuCoverHeight - st::mainMenuShadow.height(),
+			st::mainMenuWidth,
+			IntensityOfColor(st::mainMenuCoverFg->c) < 0.5
+				? Qt::white
+				: Qt::black);
+	};
+
+	// Solid color.
+	if (const auto color = Window::Theme::Background()->colorForFill()) {
+		const auto intensity = IntensityOfColor(*color);
+		p.fillRect(fill, *color);
+		if (std::abs(intensity - intensityText) < kMinDiffIntensity) {
+			drawShadow(p);
+		}
+		_background = backgroundImage;
+		return;
+	}
+
+	// Background image.
+	const auto &pixmap = Window::Theme::Background()->pixmap();
+	QRect to, from;
+	Window::Theme::ComputeBackgroundRects(fill, pixmap.size(), to, from);
+
+	// Cut off the part of the background that is under text.
+	const QRect underText(
+		st::mainMenuCoverTextLeft,
+		st::mainMenuCoverNameTop,
+		std::max(
+			st::semiboldFont->width(Auth().user()->nameText().toString()),
+			st::normalFont->width(_phoneText)),
+		st::semiboldFont->height * 2);
+
+	p.drawPixmap(to, pixmap, from);
+	if (IsShadowShown(backgroundImage, underText, intensityText)) {
+		drawShadow(p);
+	}
+	_background = backgroundImage;
+}
+
 void MainMenu::resizeEvent(QResizeEvent *e) {
 	_menu->setForceWidth(width());
 	updateControlsGeometry();
@@ -230,7 +356,16 @@ void MainMenu::updateControlsGeometry() {
 		_userpicButton->moveToLeft(st::mainMenuUserpicLeft, st::mainMenuUserpicTop);
 	}
 	if (_cloudButton) {
-		_cloudButton->moveToRight(0, st::mainMenuCoverHeight - _cloudButton->height());
+		const auto offset = st::mainMenuCloudSize / 4;
+		const auto y = st::mainMenuCoverHeight
+			- _cloudButton->height()
+			- offset;
+		_cloudButton->moveToRight(offset, y);
+		if (_archiveButton) {
+			_archiveButton->moveToRight(
+				offset,
+				y - _cloudButton->height());
+		}
 	}
 	if (_resetScaleButton) {
 		_resetScaleButton->moveToRight(0, 0);
@@ -247,17 +382,37 @@ void MainMenu::updatePhone() {
 
 void MainMenu::paintEvent(QPaintEvent *e) {
 	Painter p(this);
-	auto clip = e->rect();
-	auto cover = QRect(0, 0, width(), st::mainMenuCoverHeight).intersected(clip);
+	const auto clip = e->rect();
+	const auto cover = QRect(0, 0, width(), st::mainMenuCoverHeight)
+		.intersected(e->rect());
+
+	const auto background = Window::Theme::Background();
+	const auto isFill = background->tile()
+		|| background->colorForFill().has_value()
+		|| background->isMonoColorImage()
+		|| background->paper().isPattern()
+		|| Data::IsLegacy1DefaultWallPaper(background->paper());
+
+	if (!isFill && !_background.isNull()) {
+		PainterHighQualityEnabler hq(p);
+		p.drawImage(0, 0, _background);
+	}
+
 	if (!cover.isEmpty()) {
-		p.fillRect(cover, st::mainMenuCoverBg);
+		const auto widthText = _cloudButton
+			? _cloudButton->x() - st::mainMenuCloudSize
+			: width() - 2 * st::mainMenuCoverTextLeft;
+
+		if (isFill) {
+			p.fillRect(cover, st::mainMenuCoverBg);
+		}
 		p.setPen(st::mainMenuCoverFg);
 		p.setFont(st::semiboldFont);
-		Auth().user()->nameText.drawLeftElided(
+		Auth().user()->nameText().drawLeftElided(
 			p,
 			st::mainMenuCoverTextLeft,
 			st::mainMenuCoverNameTop,
-			width() - 2 * st::mainMenuCoverTextLeft,
+			widthText,
 			width());
 		p.setFont(st::normalFont);
 		p.drawTextLeft(st::mainMenuCoverTextLeft, st::mainMenuCoverStatusTop, width(), _phoneText);
@@ -268,8 +423,21 @@ void MainMenu::paintEvent(QPaintEvent *e) {
 				_cloudButton->y() + (_cloudButton->height() - st::mainMenuCloudSize) / 2,
 				width(),
 				st::mainMenuCloudSize,
-				st::mainMenuCloudBg,
-				st::mainMenuCloudFg);
+				isFill ? st::mainMenuCloudBg : st::msgServiceBg,
+				isFill ? st::mainMenuCloudFg : st::msgServiceFg);
+		}
+
+		// Draw Archive button.
+		if (!_archiveButton->isHidden()) {
+			if (const auto folder = Auth().data().folderLoaded(Data::Folder::kId)) {
+				folder->paintUserpic(
+					p,
+					_archiveButton->x() + (_archiveButton->width() - st::mainMenuCloudSize) / 2,
+					_archiveButton->y() + (_archiveButton->height() - st::mainMenuCloudSize) / 2,
+					st::mainMenuCloudSize,
+					isFill ? st::mainMenuCloudBg : st::msgServiceBg,
+					isFill ? st::mainMenuCloudFg : st::msgServiceFg);
+			}
 		}
 	}
 	auto other = QRect(0, st::mainMenuCoverHeight, width(), height() - st::mainMenuCoverHeight).intersected(clip);
